/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/***************************************************************************/
/*               UNIFIED ITS SOURCE CODE 21-Mar-1995                       */
/***************************************************************************/
/* ITSFUNC.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.4*29/07/97*Fixed communication of grade/asc info between     *60000*NO */
/*   *        *translator & formatter.                           *     *   */
/***************************************************************************/
/*2.3*11/07/95*Installed acronym table in TRANS.C; introduced    *30420*NO */
/*   *        *ACRONYM.H; BRAILLE_IT directive controls whether  *     *   */
/*   *        *acronym processing occurs before or during        *     *   */
/*   *        *translation.                                      *     *   */
/***************************************************************************/
/*2.2*12/05/95*Removed reference to ITSLite - now Brl_it.        *     *KJM*/
/*   *        *Also the intermediate file is now deleted here    *     *   */
/*   *        *rather than in the macro.                         *     *   */
/***************************************************************************/
/*2.1*04/05/95*Remove unwanted funcs./args. for Braille It.      *30418*NO */
/***************************************************************************/
/*2.0*29/03/94*Moon PScript tidy: PCL := Psc; LMargin, RMargin,  *30412*NO */
/*   *        *TopMargin, BottomMargin are P_Page params.        *     *   */
/*   *        *PCL files renamed; help (/H) updated.             *     *   */
/***************************************************************************/
/*1.8*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*1.7*08/03/93*Auxiliary aux_sourcetransdest (Transform).	 *30301*MW */
/***************************************************************************/
/*1.6*22/05/92*New formatting of error and confirm messages.     *30301*MJH*/
/***************************************************************************/
/*1.5*21/05/92*Auxiliary aux_sourcedest modified.                *30301*MJH*/
/***************************************************************************/
/*1.4*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.3*19/05/92*Format of error and confirm messsages corrected.  *30301*MJH*/
/***************************************************************************/
/*1.2*18/05/92*Hardware cursor movement.			 *30301*MJH*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*12/05/92*Creation Date.                                    *30301*MJH*/
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* ITSFUNC.OBJ                                                             */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

#include "its.h"
#include "itsfunc.h"
#include "itsauto.h"
#include "ittrans.h"
#include "itbembfm.h"
#include "tranfile.h"
#include "translat.h"
#if !defined(BRAILLE_IT)
#include "itsmenu.h"
#include "brlembop.h"
#include "itbembop.h"
#include "itmpscop.h"
#include "itpf.h"
#include "itmpscfm.h"
#endif

/* ------------------------------------------------------------------------ */
/* BIOS HARDWARE CURSOR MOVEMENT FUNCTION */
void movecursor(int row, int col)
{
	union _REGS reg;

	reg.h.ah = 2;
	reg.h.bh = 0;
	reg.h.dh = (char) row;
	reg.h.dl = (char) col;
    _int86((int)0x10, &reg, &reg);
}
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
/* Provide ITS format error message */
void ITS_Error(char *msg, char *Module, int row, int col)
{
#ifdef CSCAPEVER
	menu_type errormenu;
	sed_type error_screen;
	char TmpMsg[80];
	char ModMsg[80];
	int msglines, msgwidth, tmpwidth, tmplines;
	int maxwidth = 80;
	int maxlines = 25;
	int borderwidth = 2;
	int titlelines = 3;
   char *p, *formstr;
	char Title[] = "**** ITS Error Message ****";

/* Calculate maximum area for message */
	if (row > 0)
	   maxlines = maxlines - (2 * row);
	if (col > 0)
		maxwidth = maxwidth - (2 * col);

/* Find largest width out of title, module name and max message width*/
	sprintf(ModMsg, "Module: %s", Module);
	msgwidth = max(strlen(Title), strlen(ModMsg));
   msgwidth = max(msgwidth, (maxwidth - borderwidth - 2));

/* Format message string according to maximum width possible */
   if ((formstr = strwrap(msg, &msglines, msgwidth)) == NULL)
    	return;

/* Improve format by using greatest width - each line is then centred. */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
	{	if (*p == '\n')
		{	if (tmpwidth > msgwidth)
				msgwidth = tmpwidth;
			tmpwidth = 0;
			++tmplines;
		}
		else
			++tmpwidth;
	}
	if ((tmpwidth > 0) && (tmpwidth > msgwidth))
		msgwidth = tmpwidth;

	if ((errormenu = menu_Open()) == NULL)
      return;
/* Format the title */
	strcpy(TmpMsg, Title);
	strcenter(TmpMsg, msgwidth);
	menu_Printf(errormenu, "@p[1]%s@f[#]\n", TmpMsg, NULL, &nowrite_funcs);

/* Format the module name */
	strcenter(ModMsg, msgwidth);
	menu_Printf(errormenu, "@p[1]%s\n\n", ModMsg);

/* Copy and format each line in message */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
  {   if (*p == '\n')
		{	TmpMsg[tmpwidth] = '\0';
			strcenter(TmpMsg, msgwidth);
			menu_Printf(errormenu, "@p[1]%s\n", TmpMsg);
			tmpwidth = 0;
         ++tmplines;
		}
		else
		{	if (*p == '\t')
				TmpMsg[tmpwidth] = ' ';
			else
				TmpMsg[tmpwidth] = *p;
			++tmpwidth;
		}
	}
	if (tmpwidth > 0)
	{	TmpMsg[tmpwidth] = '\0';
		strcenter(TmpMsg, msgwidth);
		menu_Printf(errormenu, "@p[1]%s\n", TmpMsg);
		tmpwidth = 0;
	}

	if ((error_screen = sed_Open(errormenu)) == NULL)
	{	menu_Flush(errormenu);
		return;
	}

/* Calculate 'centred' position (include title lines and border width) */
	if (row < 0)
		row = (int) (maxlines - (msglines + titlelines)) / 2;
   if (col < 0)
		col = (int) (maxwidth - (msgwidth + borderwidth + 1)) / 2;

   sed_SetLabel(error_screen, 6);
	sed_SetBorder(error_screen, bd_prompt);
	sed_SetPosition(error_screen, row, col);
	sed_SetHeight(error_screen, msglines + titlelines);
	sed_SetWidth(error_screen, msgwidth + borderwidth);

	sed_SetBorderColor(error_screen, ERRORBACK_CLR);
	sed_SetColors(error_screen, ERRORREG_CLR, ERRORBACK_CLR, ERRORHI_CLR);
	sed_SetShadow(error_screen, 1);
	sed_SetShadowAttr(error_screen, 0x08);

	sed_SetSpecial(error_screen, spc_Abort);
	sed_SetCursorType(error_screen, CURSOR_NONE);

	tone();
	sed_Repaint(error_screen);
	movecursor(row + 1, col + 1);
	sed_Go(error_screen);
	sed_Close(error_screen);
#else
	printf("\n **** ITS Error Message ****");
	printf("\n in function %s",Module);
	printf("\n error : %s\n",msg);
#endif
	return;
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#if defined(CSCAPEVER)
/* Provide prompt box with OK button */
void ITS_Prompt(char *Msg, int row, int col)
{
    pop_Prompt(Msg, row, col, -1, -1, POPPROMPT_CLR, bd_prompt);
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
/* Provide confirmation box with YES/NO return */
boolean Confirm(char *msg, int row, int col)
{
#ifdef CSCAPEVER
	menu_type confirmmenu;
	sed_type confirm_screen;
	boolean yorn = TRUE;
    char Prompt[] = "Are you sure (y\\n) ? ###";
	int ret;
	char TmpMsg[80];
	int msglines, msgwidth, tmpwidth, tmplines;
	int maxwidth = 80;
	int maxlines = 25;
	int borderwidth = 2;
	int promptline = 1;
	char *p, *formstr;

/* Calculate maximum area for message */
	if (row > 0)
	   maxlines = maxlines - (2 * row);
	if (col > 0)
		maxwidth = maxwidth - (2 * col);

/* Find largest width out of prompt and max message width */
	msgwidth = strlen(Prompt);
   msgwidth = max(msgwidth, (maxwidth - borderwidth - 2));

/* Format message string according to maximum width possible */
   if ((formstr = strwrap(msg, &msglines, msgwidth)) == NULL)
	{	ITS_Error("Error trying to set up confirmation screen",
				"Confirm", -1, -1);
		menu_Flush(confirmmenu);
		return(TRUE);
	}

/* Improve format by using greatest width - each line is then centred. */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
	{	if (*p == '\n')
		{	if (tmpwidth > msgwidth)
				msgwidth = tmpwidth;
			tmpwidth = 0;
			++tmplines;
		}
		else
			++tmpwidth;
	}
	if ((tmpwidth > 0) && (tmpwidth > msgwidth))
		msgwidth = tmpwidth;

	if ((confirmmenu = menu_Open()) == NULL)
	{	ITS_Error("Error trying to set up confirmation screen",
				"Confirm", -1, -1);
		menu_Flush(confirmmenu);
		return(TRUE);
	}

/* Copy and format each line in message */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
  {   if (*p == '\n')
		{	TmpMsg[tmpwidth] = '\0';
			strcenter(TmpMsg, msgwidth);
			menu_Printf(confirmmenu, "@p[1]%s\n", TmpMsg);
			tmpwidth = 0;
         ++tmplines;
		}
		else
		{	if (*p == '\t')
				TmpMsg[tmpwidth] = ' ';
			else
				TmpMsg[tmpwidth] = *p;
			++tmpwidth;
		}
	}
	if (tmpwidth > 0)
	{	TmpMsg[tmpwidth] = '\0';
		strcenter(TmpMsg, msgwidth);
		menu_Printf(confirmmenu, "@p[1]%s\n", TmpMsg);
		tmpwidth = 0;
	}

/* Format the prompt portion */
	strcpy(TmpMsg, Prompt);
	strcenter(TmpMsg, msgwidth);
   *strstr(TmpMsg, " ### ") = '\0';
	menu_Printf(confirmmenu,"@p[1]%s@f[ ### ]", TmpMsg, &yorn, &yesno_funcs);

/* Calculate 'centred' position (include prompt line and border width) */
	if (row < 0)
		row = (int) (maxlines - (msglines + promptline)) / 2;
   if (col < 0)
		col = (int) (maxwidth - (msgwidth + borderwidth + 1)) / 2;

	if ((confirm_screen = sed_Open(confirmmenu)) == NULL)
	{	ITS_Error("Error trying to set up confirmation screen",
				"Confirm", -1, -1);
		menu_Flush(confirmmenu);
		return(TRUE);
	}

	sed_SetLabel(confirm_screen, 5);
	sed_SetBorder(confirm_screen, bd_prompt);
	sed_SetPosition(confirm_screen, row, col);
	sed_SetHeight(confirm_screen, msglines + promptline);
	sed_SetWidth(confirm_screen, msgwidth + borderwidth);

	sed_SetBorderColor(confirm_screen, CONFIRMBACK_CLR);
	sed_SetColors(confirm_screen, CONFIRMREG_CLR, CONFIRMBACK_CLR, CONFIRMHI_CLR);
	sed_SetShadow(confirm_screen, 1);
	sed_SetShadowAttr(confirm_screen, 0x08);

	sed_SetSpecial(confirm_screen, spc_Abort);
	sed_SetCursorType(confirm_screen, CURSOR_NONE);

	sed_Repaint(confirm_screen);
	movecursor(row + msglines + promptline, col + 1);
	ret = sed_Go(confirm_screen);
	sed_Close(confirm_screen);

	if ((ret == SED_ABORT) || (yorn == FALSE))
		return(FALSE);
#else
	char yornchar;

	printf("\n **** ITS Confirmation Message ****");
	printf("\n %s",msg);
	do	{
		printf("Are you sure (y\\n) ? ");
#ifndef KevTest
            yornchar = (char)_getche();
#else
			yornchar = 'Y';
#endif
		} while((toupper(yornchar) != 'Y') &&
			(toupper(yornchar) != 'N'));
    if (toupper(yornchar) == 'Y')
		return(TRUE);
	else
		return(FALSE);
#endif
}
/* ------------------------------------------------------------------------ */


/* Open file in specified mode */
/* Returns: a pointer to the open stream if successful; null otherwise. */
FILE *ITS_fopen (const char *filename, const char *mode)
{
  static FILE *f;

  if ((f = fopen(filename, mode)) == NULL) {
	 sprintf(errmsg, "Error while opening file: %s", filename);
	 ITS_Error(errmsg, "ITS_fopen", -1, -1);
	 if (errno != EZERO)
	 { errno = 0;
		return NULL;
	 }
  }
  return f;
}



/* Close file in specified mode */
/* Returns: 0 if successful; EOF otherwise. */
int ITS_fclose (FILE *stream)
{
  int i;

  if ((i = fclose(stream)) == EOF) {
	 ITS_Error("Error while closing file", "ITS_fclose", -1, -1);
	 errno = 0;
  }
  return i;
}


/* Control-C handler */
#ifndef KevTest
void __cdecl ctrlchandler(int sig)
{   signal(SIGINT, SIG_IGN);
    if (sig == SIGINT)
    	abrt = TRUE; /* Indicates user abort */
	signal(SIGINT, ctrlchandler);
}
#endif

/* ------------------------------------------------------------------------ */
boolean LocateFile(char *Name)
{	char tmp[FILENAMELENGTH];
	struct FilePath temp = { "\0", "\0", "\0", "\0" };

/* Determine if supplied name has drive &/or directory */
	_splitpath(Name, temp.drv, temp.dir, temp.name, temp.ext);

	if ((temp.drv[0] != '\0') || (temp.dir[0] != '\0'))
	{
/* Is file in specified drive &/or directory ? */
        if (_access(Name, (int) 00) != 0)
		{	errno = 0;
			sprintf(errmsg," Cannot find file %s", Name);
			ITS_Error(errmsg, "LocateFile", -1, -1);
			return(FALSE);
		}
		return(TRUE);
	}

/* Is file in current directory ? */
	strcpy(tmp, current.drv);
	strcat(tmp, current.dir);
	strcat(tmp, "\\");
	strcat(tmp, Name);

    if (_access(tmp, (int) 0) != 0)
	{	errno = 0;
/* Is file in program directory ? */
		strcpy(tmp, prog.drv);
		strcat(tmp, prog.dir);
		strcat(tmp, Name);

        if (_access(tmp, (int) 0) != 0)
		{	errno = 0;
			sprintf(errmsg," Cannot find file %s", Name);
			ITS_Error(errmsg, "LocateFile", -1, -1);
			return(FALSE);
		}
	}

	strcpy(Name, tmp);
	return(TRUE);
}
/* ------------------------------------------------------------------------ */



/* Removes multiple spaces from a null terminated string. */
char *trimstr(char *str)
{
 char *tempstr, *p;
 /* Make sure the allocated space is initialised to all zeros. */
 tempstr = (char *) calloc(strlen(str)+2,1);
 if (tempstr == NULL) {
	  ITS_Error("Memory Allocation error!", "trimstr", -1, -1);
     return(NULL);
    }

 strcpy(tempstr,"");
 p = strtok(str," ");

 while (p != NULL) {
    strcat(tempstr,p);
    strcat(tempstr," "); /* Add a space after each 'word' */
    p = strtok(NULL," ");
 } /* End while */

 if (strlen(tempstr) > 0)
    tempstr[strlen(tempstr)-1] = '\0'; /* Remove last space */

 strcpy(str, tempstr);
 free(tempstr);
 return (str);
} /* End of trimstr*/


#ifdef CSCAPEVER
char *charncpy(char *dest, const char *src, const size_t maxlen)
{
 size_t loop;

 for (loop = 0; loop != maxlen; loop++)
   *(dest+loop) = *(src+loop);

 *(dest+loop) = '\0';
 return(dest);
} /* End of charncpy */
#endif

/* Data & Functions associated with confirmation screens for processing */
/* -------------------------------------------------------------------- */

struct FileInfo SourceFinfo;
struct FileInfo TransFinfo;
struct FileInfo DestFinfo;

/* ------------------------------------------------------------------------ */
void ProduceFullName(char *name, struct FileInfo finfo)
{	struct FilePath temp = { "\0", "\0", "0", "\0" };

/* Split supplied name into drive, directory, base name and extension */
	_splitpath(name, temp.drv, temp.dir, temp.name, temp.ext);

/* Determine if supplied name has drive or directory specified */
	if ((temp.drv[0] == '\0') && (temp.dir[0] == '\0'))
	{
/* If default provided then use it, otherwise use current drive & dir */
		if (finfo.Dir[0] == '\0')
		{	strcpy(temp.drv, current.drv);
			strcpy(temp.dir, current.dir);
		}
		else
			strcpy(temp.dir, finfo.Dir);
	}

/* Determine if supplied name has extension specified */
	if ((temp.ext[0] == '\0') && (finfo.Ext[0] != '\0'))
			strcpy(temp.ext, finfo.Ext);

/* Reconstruct name according to revised details */
	_makepath(name, temp.drv, temp.dir, temp.name, temp.ext);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
void StoreFullName(char *name, struct FileInfo *finfo)
{	struct FilePath temp = { "\0", "\0", "0", "\0" };
	char tempdrvdir[FILENAMELENGTH];
	boolean tmpslashremoved = FALSE;
	boolean finfoslashremoved = FALSE;

/* Split supplied name into drive, directory, base name and extension */
	_splitpath(name, temp.drv, temp.dir, temp.name, temp.ext);

/* Clear name */
	strcpy(finfo->Name, "\0");
/* Determine if supplied name has drive or directory specified */
	if ((temp.drv[0] != '\0') || (temp.dir[0] != '\0'))
	{	strcpy(tempdrvdir, temp.drv);
		strcat(tempdrvdir, temp.dir);
/* If default drive and directory provided then ignore, else use */
		/* remove backslashes */
		if (tempdrvdir[strlen(tempdrvdir) - 1] == '\\')
		{	tempdrvdir[strlen(tempdrvdir) - 1] = '\0';
			tmpslashremoved = TRUE;
		}
		if (finfo->Dir[strlen(finfo->Dir) - 1] == '\\')
		{	finfo->Dir[strlen(finfo->Dir) - 1] = '\0';
			finfoslashremoved = TRUE;
		}
        if (_stricmp(finfo->Dir, tempdrvdir) != 0)
		{	strcat(finfo->Name, tempdrvdir);
			/* Replace backslash ? */
			if (tmpslashremoved)
				strcat(finfo->Name, "\\");
		}
		/* Replace backslash ? */
		if (finfoslashremoved)
			strcat(finfo->Dir,"\\");
	}

/* Add base name to name */
	strcat(finfo->Name, temp.name);

/* Determine if supplied name has extension specified */
	if (temp.ext[0] != '\0')
	{
/* If default extension provided then ignore, else use */
        if (_stricmp(finfo->Ext, &(temp.ext[1])) != 0)
			strcat(finfo->Name, temp.ext);
	}

}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
/* Screen auxilliary to move hardware cursor */
int aux_cursor(sed_type sed, int msg, VOID *indata, VOID *outdata)
{	int row;
	int col;
	oak_notused(indata);
	oak_notused(outdata);
	row = sed_GetFieldRow(sed, sed_GetFieldNo(sed));
	col = sed_GetFieldCol(sed, sed_GetFieldNo(sed));
	movecursor(row, col);
	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
int aux_sourcetransdest(sed_type sed, int msg, VOID *indata, VOID *outdata)
{
	oak_notused(indata);
	oak_notused(outdata);

	switch (msg)
	{
/* Leaving 'source' or 'destination' fields (but not abort) ? */
		case SED_POSTFEXIT :
				if ((sed_GetBaton(sed) != SED_ABORT) &&
					((sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source")) ||
					 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination")) ||
					 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "TransFile"))))
				{
/* Expand full name for source */
					if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source"))
					{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
						if (*((char *) sed_GetNameVar(sed, "Source")) != '\0')
						{	ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
										 SourceFinfo);
							sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                            if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
							{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
								tone();
								errno = 0;
								return(FALSE);
							}
						}
					}
/* Expand full name for destination */
					if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination"))
					{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
						if (*((char *) sed_GetNameVar(sed, "Destination")) != '\0')
						{	ProduceFullName((char *) sed_GetNameVar(sed, "Destination"),
										 DestFinfo);
							sed_RepaintField(sed, sed_GetNameNo(sed, "Destination"));
						}
					}
/* Source and destination names the same (but not blank) ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) != '\0') &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Source"),
									(char *) sed_GetNameVar(sed, "Destination")) == 0))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), SOURCEDESTERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
					}
					else
					{
/* Clear any outstanding error message */
						strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					}

/* Expand full name for translate file */
					if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "TransFile"))
					{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "TransFile"));
						if (*((char *) sed_GetNameVar(sed, "TransFile")) != '\0')
						{	ProduceFullName((char *) sed_GetNameVar(sed, "TransFile"),
										 TransFinfo);
							sed_RepaintField(sed, sed_GetNameNo(sed, "TransFile"));
						}
					}
/* Source and translation names the same (but not blank) ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) != '\0')
					    &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Source"), (char *) sed_GetNameVar(sed, "TransFile")) == 0))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), SOURCETRANSERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
					}
					else
					{
/* Clear any outstanding error message */
						strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					}
/* Destination and translation names the same (but not blank) ? */
					if ((*((char *) sed_GetNameVar(sed, "TransFile")) != '\0')
					    &&
					    (*((char *) sed_GetNameVar(sed, "Destination")) != '\0')
					    &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Destination"),
								  (char *) sed_GetNameVar(sed, "TransFile")) == 0))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), TRANSDESTERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
					}
					else
					{
/* Clear any outstanding error message */
						strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					}

				}
				return(TRUE);

/* Leaving screen ? */
		case SED_PRESEXIT :
				if (sed_GetBaton(sed) != SED_ABORT)
				{
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "TransFile"));
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
/* Source and destination provided ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) == '\0') ||
					    (*((char *) sed_GetNameVar(sed, "Destination")) == '\0') ||
					    (*((char *) sed_GetNameVar(sed, "TransFile")) == '\0'))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), ALLFILESERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
/* Leaving 'source' or 'destination' fields ? */
					if ((sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source")) ||
					    (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination")) ||
					    (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "TransFile")))
					{
						if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source"))
						{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
							if (*((char *) sed_GetNameVar(sed, "Source")) != '\0')
							{	ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
											 SourceFinfo);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                                if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
								{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
									sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
									tone();
									errno = 0;
									return(FALSE);
								}
							}
						}
						if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination"))
						{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
							if (*((char *) sed_GetNameVar(sed, "Destination")) != '\0')
							{	ProduceFullName((char *) sed_GetNameVar(sed, "Destination"),
											 DestFinfo);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Destination"));
							}
						}
						if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "TransFile"))
						{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "TransFile"));
							if (*((char *) sed_GetNameVar(sed, "TransFile")) != '\0')
							{	ProduceFullName((char *) sed_GetNameVar(sed, "TransFile"),
											 TransFinfo);
								sed_RepaintField(sed, sed_GetNameNo(sed, "TransFile"));
							}
						}

					}
/* Source and destination provided ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) == '\0') ||
					    (*((char *) sed_GetNameVar(sed, "Destination")) == '\0') ||
					    (*((char *) sed_GetNameVar(sed, "TransFile")) == '\0'))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), ALLFILESERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
/* Source, TransFile and destination names the same ? */
                    if ((_stricmp((char *) sed_GetNameVar(sed, "Source"), (char *) sed_GetNameVar(sed, "Destination")) == 0)
					    &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Source"), (char *) sed_GetNameVar(sed, "TransFile")) == 0)
					    &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Destination"), (char *) sed_GetNameVar(sed, "TransFile")) == 0))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), SOURCETRANSDESTERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
				}
				return(TRUE);

		default	:	break;
	}

	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
int aux_sourcedest(sed_type sed, int msg, VOID *indata, VOID *outdata)
{
	oak_notused(indata);
	oak_notused(outdata);

	switch (msg)
	{
/* Leaving 'source' or 'destination' fields (but not abort) ? */
		case SED_POSTFEXIT :
				if ((sed_GetBaton(sed) != SED_ABORT) &&
					((sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source")) ||
					 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination"))))
				{
/* Expand full name for source */
					if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source"))
					{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
						if (*((char *) sed_GetNameVar(sed, "Source")) != '\0')
						{	ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
										 SourceFinfo);
							sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                            if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
							{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
								tone();
								errno = 0;
								return(FALSE);
							}
						}
					}
/* Expand full name for destination */
					if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination"))
					{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
						if (*((char *) sed_GetNameVar(sed, "Destination")) != '\0')
						{	ProduceFullName((char *) sed_GetNameVar(sed, "Destination"),
										 DestFinfo);
							sed_RepaintField(sed, sed_GetNameNo(sed, "Destination"));
						}
					}
/* Source and destination names the same (but not blank) ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) != '\0') &&
                        (_stricmp((char *) sed_GetNameVar(sed, "Source"),
									(char *) sed_GetNameVar(sed, "Destination")) == 0))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), SOURCEDESTERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
					}
					else
					{
/* Clear any outstanding error message */
						strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					}
				}
				return(TRUE);

/* Leaving screen ? */
		case SED_PRESEXIT :
				if (sed_GetBaton(sed) != SED_ABORT)
				{
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
/* Source and destination provided ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) == '\0') ||
						(*((char *) sed_GetNameVar(sed, "Destination")) == '\0'))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), BOTHFILESERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
/* Leaving 'source' or 'destination' fields ? */
					if ((sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source")) ||
						 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination")))
					{
						if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source"))
						{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
							if (*((char *) sed_GetNameVar(sed, "Source")) != '\0')
							{	ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
											 SourceFinfo);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                                if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
								{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
									sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
									tone();
									errno = 0;
									return(FALSE);
								}
							}
						}
						if (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Destination"))
						{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Destination"));
							if (*((char *) sed_GetNameVar(sed, "Destination")) != '\0')
							{	ProduceFullName((char *) sed_GetNameVar(sed, "Destination"),
											 DestFinfo);
								sed_RepaintField(sed, sed_GetNameNo(sed, "Destination"));
							}
						}
					}
/* Source and destination provided ? */
					if ((*((char *) sed_GetNameVar(sed, "Source")) == '\0') ||
						(*((char *) sed_GetNameVar(sed, "Destination")) == '\0'))
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), BOTHFILESERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
/* Source and destination names the same ? */
                    if (_stricmp((char *) sed_GetNameVar(sed, "Source"), (char *) sed_GetNameVar(sed, "Destination")) == 0)
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), SOURCEDESTERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
				}
				return(TRUE);

		default	:	break;
	}

	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */



/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
int aux_output(sed_type sed, int msg, VOID *indata, VOID *outdata)
{
	oak_notused(indata);
	oak_notused(outdata);

	switch (msg)
	{
/* Leaving destination port field (but not abort) ? */
		case SED_PREFEXIT :
				if ((sed_GetBaton(sed) != SED_ABORT) &&
					(sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Port")))
				{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Port"));
/* Only allow filename entry if redirected output to FILE */
					if (strcmp((char *) sed_GetNameVar(sed, "Port"), "FILE") != 0)
						sed_ProtectField(sed, sed_GetNameNo(sed, "File"));
					else
						sed_UnProtectField(sed, sed_GetNameNo(sed, "File"));
					return(TRUE);
				}

/* Leaving source field ? */
				if ((sed_GetBaton(sed) != SED_ABORT) &&
					 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Source")))
				{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
/* Source provided ? */
					if (*((char *) sed_GetNameVar(sed, "Source")) == '\0')
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), NOSOURCEERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
					ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
											 SourceFinfo);
					sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                    if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						errno = 0;
						return(FALSE);
					}
/* Clear any outstanding error message */
					strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
					sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					return(TRUE);
				}

/* Leaving page range field ? */
				if ((sed_GetBaton(sed) != SED_ABORT) &&
					 (sed_GetFieldNo(sed) == sed_GetNameNo(sed, "Pages")))
				{	sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Pages"));
/* Validate Page Range */
					if (ValidPageRange(sed_GetNameVar(sed, "Pages")) == INVALID)
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), PAGESERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
/* Clear any outstanding error message */
					strcpy((char *) sed_GetNameVar(sed, "Error"), NOERRMSG);
					sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
					return(TRUE);
				}
				return(TRUE);

/* Leaving screen ? */
		case SED_PRESEXIT :
				if (sed_GetBaton(sed) != SED_ABORT)
				{
					sed_DoFieldSexit(sed, sed_GetNameNo(sed, "Source"));
/* Source provided ? */
					if (*((char *) sed_GetNameVar(sed, "Source")) == '\0')
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), NOSOURCEERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						return(FALSE);
					}
					ProduceFullName((char *) sed_GetNameVar(sed, "Source"),
											 SourceFinfo);
					sed_RepaintField(sed, sed_GetNameNo(sed, "Source"));
/* Source exists ? */
                    if (_access((char *) sed_GetNameVar(sed, "Source"), (int) 00) != 0)
					{	strcpy((char *) sed_GetNameVar(sed, "Error"), BADSOURCEERRMSG);
						sed_RepaintField(sed, sed_GetNameNo(sed, "Error"));
						tone();
						errno = 0;
						return(FALSE);
					}
				}
				return(TRUE);

		default	:	break;
	}

	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
#if !defined(BRAILLE_IT)
size_t cv_fpstrtou(char *str)
{
    char *tmpstr, *expptr;

    tmpstr = _strdup(str);
    expptr = strchr(tmpstr, '.');
    if (expptr != NULL) {
        *expptr = '\0'; /* tmpstr is now the integer string */
        ++expptr; /* now points to decimal fraction, if any */
    }

    free(tmpstr);
    return (size_t) ( atoi(tmpstr) * 10*strlen(expptr) + atoi(expptr) );
}
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
int cv_fpstrtotwip(char *str)
{
    char *tmpstr, *expptr;
    long l1, l2, divpow10;
    size_t explen, roundup, i;

    tmpstr = _strdup(str);
    expptr = strchr(tmpstr, '.');
    if (expptr != NULL) {
        *expptr = '\0'; /* tmpstr is now the integer string */
        ++expptr; /* now points to decimal fraction, if any */
    }

    l1 = atol(tmpstr);
    l2 = atol(expptr);
    explen = strlen(expptr);
    for (divpow10=1, i=explen; i; i--)
        divpow10 *= 10;
    /* Calc round up if any */
    for (roundup=5, i=explen-1; i; i--)
        roundup *= 10;

    free(tmpstr);
    return (int)(l1 * TWIP + ((l2 * TWIP + roundup) / divpow10));
        /* This expression avoids using floating point */
}
#endif
/* ------------------------------------------------------------------------ */

/* End of ITSFUNC.C */
/* ITS.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.3*29/03/94*Moon PScript tidy: PCL := Psc; LMargin, RMargin,  *30412*NO */
/*   *        *TopMargin, BottomMargin are P_Page params.        *     *   */
/*   *        *PCL files renamed; help (/H) updated.             *     *   */
/***************************************************************************/
/*2.2*15/12/93*Check the available memory when BRAILLE_IT used,  *30409*KJM*/
/*   *        *and quit 'nicely' if not enough.			         *     *   */
/***************************************************************************/
/*2.1*13/10/93*BRAILLE_IT compiler directive now comments out    *30409*KJM*/
/*   *        *code not specifically needed in 'ItsLite'.        *     *   */
/***************************************************************************/
/*2.0*20/07/93*Moon config. menu amended.                        *30406*MJH*/
/***************************************************************************/
/*1.9*15/07/93*Moon Outputting menu option added.                *30406*MJH*/
/***************************************************************************/
/*1.8*15/06/93*Moon PCL formatting menu options added.           *30105*MJH*/
/***************************************************************************/
/*1.7*08/06/93*Moon PCL formatting added.                        *30105*NO */
/***************************************************************************/
/*1.6*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*1.5*01/06/92*Moon translation added.                           *30301*MJH*/
/***************************************************************************/
/*1.4*28/05/92*Display initialisation changed.                   *30301*MJH*/
/***************************************************************************/
/*1.3*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.2*19/05/92*Minor adjustment for aux_cursor prototype.        *30301*MJH*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*08/05/92*Creation Date.                                    *30301*MJH*/
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* ITS.OBJ                                                                 */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* NOTE: The following compiler directives may be used:-                   */
/* CSCAPEVER         Cscape version of ITS                                 */
/* BRAILLE_IT       Braille it version                                     */
/***************************************************************************/


/* ------------------------ INCLUDE FILES --------------------------------- */
/* --------------------- END OF INCLUDE FILES ----------------------------- */

/* Global data declarations */
#ifdef CSCAPEVER
sfile_type lnffile;
sed_type background;
struct hv_struct hvd = {0, 0, 23, 78, 0x3f, bd_prompt};
#endif
boolean abrt;
char errmsg[ERRMSGSIZE];

#ifndef BRAILLE_IT /*test180195*/
struct XrefEntry ConfigXref[NUMBERCONFIGDECODEPARAM];
struct XrefEntry BrlXref[NUMBERBRLDECODEPARAM];
struct XrefEntry MoonXref[NUMBERMOONDECODEPARAM];
#endif
struct SystemParams SysParam;
struct BrlParams BrlParam;
#ifndef BRAILLE_IT
	struct MoonParams MoonParam;
#endif
struct AutoFlags Auto;
struct FilePath prog;
struct FilePath current;


/* ------------------------------------------------------------------------ */
void IntParam(char *Entry, int *point)
{       int tempval;
	char *ptr;
/* Ensure null provided directly after integer element on line */
	ptr = Entry;
	while ((isspace(*ptr) == 0) && (*ptr) && (*ptr != '\n'))
		ptr++;
	*ptr = '\0';
/* Only update parameter if valid integer can be formed */
	if ((tempval = atoi(Entry)) != 0)
		*point = tempval;
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
void StrParam(char *Entry, char *point, size_t amount)
{       strncpy(point, Entry, amount);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
void SystemFileNameParam(char *Entry, char *point, size_t amount)
{       strncpy(point, Entry, amount);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
void FileNameParam(char *Entry, char *point, size_t amount)
{       strncpy(point, Entry, amount);
	LocateFile(point);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#if !defined(BRAILLE_IT)
void CharParam(char *Entry, char *point)
{       *point = *Entry;
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
void BooleanParam(char *Entry, boolean *point)
{       /* Only update parameter if valid entry found */
    if (_strnicmp(Entry, "YES", 3) == 0)
		*point = TRUE;
	else
        if (_strnicmp(Entry, "NO", 2) == 0)
			*point = FALSE;
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
char * strnxtelmnt(char *Line)
{/* Move pointer to next 'text' element of line */
	while ((isspace(*Line) != 0) && (*Line) && (*Line != '\n'))
		Line++;
/* Only return pointer if element located */
	if ((*Line)     && (*Line != '\n'))
		return(Line);
	return(NULL);
}
/* ------------------------------------------------------------------------ */


/*--------------------------- MODULE SEE -----------------------------------*/
boolean DecodeLine(char *Line, struct XrefEntry *Table, int Maxcount)
{       int count;
	char *ptr;

/* Locate first element on line, only continue if found */
	ptr = Line;
	if ((Line = strnxtelmnt(ptr)) == NULL)
		return(TRUE);

/* Scan table for matching entry */
	for (count = 0; (count < Maxcount); Table++, count++)
	{
/* Test for description in table entry and line */
        if (_strnicmp(Line, Table->pardesc, strlen(Table->pardesc)) == 0)
		{       /* set exit condition */
			count = Maxcount + 1;
/* Adjust line pointer to next element of line after description text */
			ptr = Line + strlen(Table->pardesc);
			if ((Line = strnxtelmnt(ptr)) == NULL)
				return(TRUE);

/* Decode parameter entry according to 'type' indicator */
			switch (Table->partype)
			{
/* Integer parameter */
				case 'I':       IntParam(Line, (int *) Table->parloc);
								break;
/* String parameter */
				case 'S':       if ((ptr = strchr(Line, '\n')) != NULL)
									*ptr = '\0';
								StrParam(Line, (char *) Table->parloc,
												(size_t) Table->parsize);
								break;
/* Filename parameter */
				case 'F':       if ((ptr = strchr(Line, '\n')) != NULL)
									*ptr = '\0';
								FileNameParam(Line, (char *) Table->parloc,
												(size_t) Table->parsize);
								break;
/* System Filename parameter */
				case 'P':       if ((ptr = strchr(Line, '\n')) != NULL)
									*ptr = '\0';
								SystemFileNameParam(Line, (char *) Table->parloc,
												(size_t) Table->parsize);
								break;
/* Character parameter */
#if !defined(BRAILLE_IT)
				case 'C':       CharParam(Line, (char *) Table->parloc);
								break;
#endif
/* Boolean flag parameter */
				case 'B':       BooleanParam(Line, (boolean *) Table->parloc);
								break;

                case '0': /* Do not store */
                default: break;
			}
		}
	}
/* Unidentified parameter ? */
	if (count == Maxcount)
		return(FALSE);

	return(TRUE);
}
/*----------------------- END OF MODULE SEE --------------------------------*/



/*--------------------------- MODULE SE ------------------------------------*/
int DecodeParameterFile(char *FName, struct XrefEntry *Xref, int MaxEntry)
{       FILE* ParamFile;
	char Entry[PARAMENTRYLENGTH];

/* Locate file in specified, current or program directory */
	if (!LocateFile(FName))
		return(FALSE);

/* Open file for reading */
	if ((ParamFile = ITS_fopen(FName, "r")) == NULL)
		return(FALSE);

/* Read each line from file until error or EOF */
	Entry[0] = '\0';
	while (((fgets(Entry, PARAMENTRYLENGTH-1, ParamFile)) != NULL))
	{
/* Decode line according to cross reference table */
		if (!DecodeLine(Entry, Xref, MaxEntry))
		{       sprintf(errmsg, " Invalid parameter line:\n %s", Entry);
			ITS_Error(errmsg, "DecodeParameterFile", -1, -1);
		}
		Entry[0] = '\0';
	}

	ITS_fclose(ParamFile);

/* Only file error could cause return FALSE */
	return(TRUE);
}
/*----------------------- END OF MODULE SE ---------------------------------*/



/*--------------------------- MODULE AAA -----------------------------------*/
int Form_Config_DecodeTable(struct SystemParams *SP, struct XrefEntry *Xref)
{
    int i;
    for(i=0; i< NUMBERCONFIGDECODEPARAM; ++i)
        memset(Xref+i, sizeof(*Xref), '\0');

	strcpy(Xref->pardesc, SOURCEDIR);
	Xref->parloc = &(SP->Source.Dir[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

	strcpy(Xref->pardesc, SOURCEEXT);
	Xref->parloc = &(SP->Source.Ext[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) EXTLENGTH;
	Xref++;

    strcpy(Xref->pardesc, NETWORK);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->Network);
	Xref->partype = 'B';
#else
    Xref->partype = '0';
#endif
    Xref++;

    strcpy(Xref->pardesc, CONFIRM);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->Confirm);
	Xref->partype = 'B';
#else
    Xref->partype = '0';
#endif
    Xref++;

    strcpy(Xref->pardesc, PAUSE);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->Pause);
	Xref->partype = 'B';
#else
    Xref->partype = '0';
#endif
    Xref++;

	strcpy(Xref->pardesc, EMBOSSNAME);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->EmbossName[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
    Xref++;

    strcpy(Xref->pardesc, EMBOSSPORT);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->EmbossPort[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) PORTLENGTH;
#else
    Xref->partype = '0';
#endif
    Xref++;

    strcpy(Xref->pardesc, PSCNAME);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->PscName[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
    Xref++;

    strcpy(Xref->pardesc, PSCPORT);
#if !defined(BRAILLE_IT)
    Xref->parloc = &(SP->PscPort[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) PORTLENGTH;
#else
    Xref->partype = '0';
#endif

	return(TRUE);
}
/*----------------------- END OF MODULE AAA --------------------------------*/



/*--------------------------- MODULE AAC -----------------------------------*/
int Form_Brl_DecodeTable(struct BrlParams *BP, struct XrefEntry *Xref)
{
    int i;
    for(i=0; i< NUMBERBRLDECODEPARAM; ++i)
        memset(Xref+i, sizeof(*Xref), '\0');


	strcpy(Xref->pardesc, BRLTRANSDIR);
	Xref->parloc = (char *) &(BP->Trans.Dir[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

	strcpy(Xref->pardesc, BRLTRANSEXT);
	Xref->parloc = (char *) &(BP->Trans.Ext[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) EXTLENGTH;
	Xref++;

	strcpy(Xref->pardesc, BRLEMBFORMDIR);
	Xref->parloc = (char *) &(BP->EmbForm.Dir[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

	strcpy(Xref->pardesc, BRLEMBFORMEXT);
	Xref->parloc = (char *) &(BP->EmbForm.Ext[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) EXTLENGTH;
	Xref++;

	strcpy(Xref->pardesc, BRLEMBPAGEWIDTH);
	Xref->parloc = (int *) &(BP->EmbPage.Width);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, BRLEMBPAGELENGTH);
	Xref->parloc = (int *) &(BP->EmbPage.Length);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, BRLEMBFIRSTPAGE);
	Xref->parloc = (int *) &(BP->EmbPage.FirstPage);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, BRLHEADERTYPE);
	Xref->parloc = (int *) &(BP->HeaderType);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, BRLHEADERREFLECT);
	Xref->parloc = (boolean *) &(BP->HeaderReflect);
	Xref->partype = 'B';
	Xref++;

	strcpy(Xref->pardesc, BRLTRANSGRADE);
	Xref->parloc = (int *) &(BP->TransGrade);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, BRLTRANSTABLE);
	Xref->parloc = (char *) &(BP->TransTable[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, BRLACRONYMTABLE);
    Xref->parloc = (char *) &(BP->AcronymTable[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, BRLLINENOS);
#if !defined(BRAILLE_IT)
    Xref->parloc = (boolean *) &(BP->LineNos);
    Xref->partype = 'B';
#else
    Xref->partype = '0';
#endif
	Xref++;

	strcpy(Xref->pardesc, BRLEMBSTARTBANNER);
#if !defined(BRAILLE_IT)
    Xref->parloc = (char *) &(BP->EmbStartBanner[0]);
	Xref->partype = 'F';
    Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
	Xref++;

	strcpy(Xref->pardesc, BRLEMBENDBANNER);
#if !defined(BRAILLE_IT)
    Xref->parloc = (char *) &(BP->EmbEndBanner[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
	Xref++;

	strcpy(Xref->pardesc, BRLEMBSTARTSEQ);
#if !defined(BRAILLE_IT)
    Xref->parloc = (char *) &(BP->EmbStartSeq[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
	Xref++;

	strcpy(Xref->pardesc, BRLEMBENDSEQ);
#if !defined(BRAILLE_IT)
    Xref->parloc = (char *) &(BP->EmbEndSeq[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif
	Xref++;

	strcpy(Xref->pardesc, BRLEMBFCODE);
	Xref->parloc = (char *) &(BP->EmbFormatCode[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, BRLEMBCWT);
#if !defined(BRAILLE_IT)
    Xref->parloc = (char *) &(BP->EmbCWT[0]);
    Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
#else
    Xref->partype = '0';
#endif

	return(TRUE);
}
/*----------------------- END OF MODULE AAC --------------------------------*/


#ifndef BRAILLE_IT
/*--------------------------- MODULE ??? -----------------------------------*/
int Form_Moon_DecodeTable(struct MoonParams *MP, struct XrefEntry *Xref)
{
    int i;
    for(i=0; i< NUMBERMOONDECODEPARAM; ++i)
        memset(Xref+i, sizeof(*Xref), '\0');


	strcpy(Xref->pardesc, MOONTRANSDIR);
	Xref->parloc = (char *) &(MP->Trans.Dir[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

	strcpy(Xref->pardesc, MOONTRANSEXT);
	Xref->parloc = (char *) &(MP->Trans.Ext[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) EXTLENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCFORMDIR);
    Xref->parloc = (char *) &(MP->PscForm.Dir[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCFORMEXT);
    Xref->parloc = (char *) &(MP->PscForm.Ext[0]);
	Xref->partype = 'S';
	Xref->parsize = (size_t) EXTLENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCLMARGIN);
    Xref->parloc = (char *) MP->PscPage.LMargin;
    Xref->partype = 'S';
    Xref->parsize = NUMBER_STRLEN;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCRMARGIN);
    Xref->parloc = (char *) MP->PscPage.RMargin;
    Xref->partype = 'S';
    Xref->parsize = NUMBER_STRLEN;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCTOPMARGIN);
    Xref->parloc = (char *) MP->PscPage.TopMargin;
    Xref->partype = 'S';
    Xref->parsize = NUMBER_STRLEN;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCBOTTOMMARGIN);
    Xref->parloc = (char *) MP->PscPage.BottomMargin;
    Xref->partype = 'S';
    Xref->parsize = NUMBER_STRLEN;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCLINEHEIGHT);
    Xref->parloc = (int *) MP->PscPage.LineHeight;
    Xref->partype = 'S';
    Xref->parsize = 7u;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCFIRSTPAGE);
    Xref->parloc = (int *) &(MP->PscPage.FirstPage);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, MOONHEADERTYPE);
	Xref->parloc = (int *) &(MP->HeaderType);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, MOONHEADERREFLECT);
	Xref->parloc = (boolean *) &(MP->HeaderReflect);
	Xref->partype = 'B';
	Xref++;

	strcpy(Xref->pardesc, MOONTRANSGRADE);
	Xref->parloc = (int *) &(MP->TransGrade);
	Xref->partype = 'I';
	Xref++;

	strcpy(Xref->pardesc, MOONTRANSTABLE);
	Xref->parloc = (char *) &(MP->TransTable[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

	strcpy(Xref->pardesc, MOONLINENOS);
	Xref->parloc = (boolean *) &(MP->LineNos);
	Xref->partype = 'B';
	Xref++;

    strcpy(Xref->pardesc, MOONPSCSTARTBANNER);
    Xref->parloc = (char *) &(MP->PscStartBanner[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCENDBANNER);
    Xref->parloc = (char *) &(MP->PscEndBanner[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCSTARTSEQ);
    Xref->parloc = (char *) &(MP->PscStartSeq[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCENDSEQ);
    Xref->parloc = (char *) &(MP->PscEndSeq[0]);
	Xref->partype = 'F';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCFCODE);
    Xref->parloc = (char *) &(MP->PscFormatCode[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
	Xref++;

    strcpy(Xref->pardesc, MOONPSCCWT);
    Xref->parloc = (char *) &(MP->PscCWT[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;
    Xref++;

    strcpy(Xref->pardesc, MOONPSCFONT);
    Xref->parloc = (char *) &(MP->PscFontFile[0]);
	Xref->partype = 'P';
	Xref->parsize = (size_t) FILENAMELENGTH;

	return(TRUE);
}
/*----------------------- END OF MODULE ??? --------------------------------*/
#endif
    

/* ------------------------------------------------------------------------ */
boolean ConfigArg(char * source, char * dest)
{       char tmp[FILENAMELENGTH];
	char *temp;

/* Argument is in form /CXxxxxxxxxxx */
	temp = strchr(source, '/');
	temp += 3;

	strcpy(tmp, temp);
	if (!LocateFile(tmp))
		return(FALSE);

	strcpy(dest, tmp);
	return(TRUE);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#if !defined(BRAILLE_IT)
boolean SourceNameArg(char * source, char * dest, boolean *namedok)
{       char *temp;

/* Argument is in form /FSxxxxxxxxxx */
	temp = strchr(source, '/');
	temp += 3;

	strcpy(dest, temp);
	*namedok = TRUE;
	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
boolean BrailleNameArg(char * source, struct BrlParams *BP)
{       char *temp;

/* Argument is in form /FBXxxxxxxxxxx */
	temp = strchr(source, '/');
	source = temp + 4;

	temp += 3;
	switch (*temp)
	{
		case 'T' :      strcpy(BP->Trans.Name, source);
						break;
		case 'E' :      strcpy(BP->EmbForm.Name, source);
						break;
		default :       break;
	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */
                  
                  
#ifndef BRAILLE_IT
/* ------------------------------------------------------------------------ */
boolean MoonNameArg(char * source, struct MoonParams *MP)
{       char *temp;

/* Argument is in form /FMXxxxxxxxxxx */
	temp = strchr(source, '/');
	source = temp + 4;

	temp += 3;
	switch (*temp)
	{
		case 'T' :  strcpy(MP->Trans.Name, source);
						break;
        case 'P' :      strcpy(MP->PscForm.Name, source);
						break;
        default :   break;
	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */
#endif
      
      
/* ------------------------------------------------------------------------ */
#if !defined(BRAILLE_IT)
boolean OutputDestArg(char * source, struct SystemParams *SP)
{       char *temp, *temp2;

/* Argument is in form /OXxxxxxxxxxx */
    temp = strchr(source, '/');
    if (temp && *temp) /* Not null pointer; not null char */
        ++temp;
    if (temp && *temp) /* Not null pointer; not null char */
        ++temp;
    if (temp && *temp) /* Not null pointer; not null char */
        ++temp;
	temp2 = source + 3;

/* Look for communications port descriptor */
	if (((strstr(source, "COM") == source) || (strstr(source, "LPT") == source))
				&& (strlen(source) < PORTLENGTH) && (isdigit( *(source + 3))))
	{       temp += 2;
		switch (*temp)
		{
			case 'E' :      strcpy(SP->EmbossPort, source);
							break;
            case 'P' :      strcpy(SP->PscPort, source);
							break;
            default :       break;
		}
		return(TRUE);
	}

	temp += 2;
	switch (*temp)
	{
		case 'E' :      strncpy(SP->EmbossName, source, sizeof SP->EmbossName);
						strcpy(SP->EmbossPort, "FILE");
						break;
        case 'P' :      strncpy(SP->PCLName, source, sizeof SP->PCLName);
                        strcpy(SP->PCLPort, "FILE");
						break;
        default :       break;
	}

	return(TRUE);
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
boolean BrlAutoArg(char * source, struct AutoFlags *Autoflg)
{       char *temp;

/* Argument is in form /Bxx */
	temp = strchr(source, '/');
	source = temp + 2;

/* Translate required ? */
	if ((strchr(source, 'T')) != NULL)
		Autoflg->BrlTrans = TRUE;

/* Emboss Format required ? */
	if ((strchr(source, 'E')) != NULL)
		Autoflg->BrlEmbForm = TRUE;

/* Is /B set with no switches (transform) */
	if ((!Autoflg->BrlEmbForm) && (!Autoflg->BrlTrans))
	{	Autoflg->BrlTrans = TRUE;
		Autoflg->BrlEmbForm = TRUE;
	}

/* Set automatic processing flags */
	if ((Autoflg->BrlTrans) || (Autoflg->BrlEmbForm))
	{       Autoflg->Brl = TRUE;
		Autoflg->Required = TRUE;
	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */


#ifndef BRAILLE_IT
/* ------------------------------------------------------------------------ */
boolean MoonAutoArg(char * source, struct AutoFlags *Autoflg)
{  char *temp;

/* Argument is in form /Mxx */
	temp = strchr(source, '/');
	source = temp + 2;

/* Translate required ? */
	if ((strchr(source, 'T')) != NULL)
		Autoflg->MoonTrans = TRUE;

/* Moon Format required ? */
    if ((strchr(source, 'P')) != NULL)
        Autoflg->MoonPscForm = TRUE;

/* Is /M set with no switches (transform) */
    if ((!Autoflg->MoonPscForm) && (!Autoflg->MoonTrans))
    {   Autoflg->MoonTrans = TRUE;
        Autoflg->MoonPscForm = TRUE;
	}

/* Set automatic processing flags */
    if ((Autoflg->MoonTrans) || (Autoflg->MoonPscForm))
	{  Autoflg->Moon = TRUE;
		Autoflg->Required = TRUE;
	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */
#endif

/* ------------------------------------------------------------------------ */
boolean UnknownArg(char * source, char * dest, boolean alreadynamed)
{
/* Ignore other possible '/' argument */
	if (strchr(source, '/') != NULL)
		return(TRUE);

/* Assume this must be source file name - only store if no /FS */
	if (alreadynamed)
		return(TRUE);
	strcpy(dest, source);

	return(TRUE);
}
/* ------------------------------------------------------------------------ */



/*-------------------------- MODULE ACA ------------------------------------*/
int DecodeCommandLine(struct AutoFlags *Autoflg, struct SystemParams *SParam,
                                struct BrlParams *BParam,
#if !defined(BRAILLE_IT)
                                struct MoonParams *MParam,
#endif
								int argc, char **argv)
{       int count;
	boolean sourcenamed = FALSE;

/* Ignore first argument (program name) */
	argv++;
	argc--;

#ifdef BRAILLE_IT
	/* Assume /B when running ITSBrlIt. */
	BrlAutoArg("/B", Autoflg);		
#endif


/* For each command line argument */
	for (count = 1; count <= argc; count++, argv++)
	{

#ifndef BRAILLE_IT /* Ignore /B?? arguments in ITSBrlIt, assumed to be /B above.*/
	/* Braille auto processing argument ? */
			if (strstr(*argv, "/B") != NULL)
			{  BrlAutoArg(*argv, Autoflg);
				continue;
			}
#endif
#ifndef BRAILLE_IT
	/* Moon auto processing argument ? */
			if (strstr(*argv, "/M") != NULL)
			{  MoonAutoArg(*argv, Autoflg);
				continue;
			}

/* Source file name argument ? */
		if (strstr(*argv, "/FS") != NULL)
		{  SourceNameArg(*argv, SParam->Source.Name, &sourcenamed);
			continue;
		}
#endif

/* Braille file name argument ? */
		if (strstr(*argv, "/FB") != NULL)
		{  BrailleNameArg(*argv, BParam);
			continue;
		}

#ifndef BRAILLE_IT
	/* Moon file name argument ? */
			if (strstr(*argv, "/FM") != NULL)
			{  MoonNameArg(*argv, MParam);
				continue;
			}
#endif
      
      
#ifndef BRAILLE_IT /* No output option in ITSBrlIt. */
/* Output destination argument ? */
		if (strstr(*argv, "/O") != NULL)
		{       OutputDestArg(*argv, SParam);
			continue;
		}
#endif

/* Screen IO argument ? */
		if (strstr(*argv, "/Q") != NULL)
		{       Autoflg->ScreenOn = FALSE;
			continue;
		}

/* Unidentified argument (could well be source file name) */
		if (!sourcenamed)
			UnknownArg(*argv, SParam->Source.Name, sourcenamed);
	}

	return(TRUE);
}
/*----------------------- END OF MODULE ACA --------------------------------*/


/* ------------------------------------------------------------------------ */
int GetParamNames(struct SystemParams *SParam, struct BrlParams *BParam,
#if !defined(BRAILLE_IT)
            struct MoonParams *MParam,
#endif
								int argc, char **argv)
{       int count;

/* Ignore first argument */
	argv++;
	argc--;

/* For each command line argument */
	for (count = 1; count <= argc; count++, argv++)
	{
/* Convert to all upper case */
        _strupr(*argv);

/* System configuration file name argument ? */
		if (strstr(*argv, "/CS") != NULL)
		{  ConfigArg(*argv, SParam->ParamName);
			continue;
		}

/* Braille configuration file name argument ? */
		if (strstr(*argv, "/CB") != NULL)
		{  ConfigArg(*argv, BParam->ParamName);
			continue;
		}
                  
                  
#ifndef BRAILLE_IT
	/* Moon configuration file name argument ? */
			if (strstr(*argv, "/CM") != NULL)
			{  ConfigArg(*argv, MParam->ParamName);
				continue;
			}
#endif

	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */


/*-------------------------- MODULE AC -------------------------------------*/
int ParameterDecode(struct AutoFlags *Auto, struct SystemParams *SParam,
                    struct BrlParams *BParam,
#if !defined(BRAILLE_IT)
                    struct MoonParams *MParam,
#endif
					struct XrefEntry *CXref, struct XrefEntry *BXref,
#if !defined(BRAILLE_IT)
           struct XrefEntry *MXref,
#endif
					int argc, char **argv)
{
/* Pre-analyse command line for parameter file names */
	if (argc > 1)
        if (!GetParamNames(SParam, BParam,
#if !defined(BRAILLE_IT)
          MParam,
#endif
          argc, argv))
			return(FALSE);

	if (!DecodeParameterFile(SParam->ParamName, CXref, (int) NUMBERCONFIGDECODEPARAM))
		return(FALSE);
   if (!DecodeParameterFile(BParam->ParamName, BXref, (int) NUMBERBRLDECODEPARAM))
		return(FALSE);

#ifndef BRAILLE_IT
   	if (!DecodeParameterFile(MParam->ParamName, MXref, (int) NUMBERMOONDECODEPARAM))
			return(FALSE);
#endif

/* Post-analyse command line for other parameters */
	if (argc > 1)
        if (!DecodeCommandLine(Auto, SParam, BParam,
#if !defined(BRAILLE_IT)
          MParam,
#endif
          argc, argv))
			return(FALSE);

/* If auto processing then inhibit confirm and pause */
#if !defined(BRAILLE_IT)
	if (Auto->Required)
	{       SParam->Confirm = FALSE;
		SParam->Pause = FALSE;
    }
#endif
	return(TRUE);
}
/*----------------------- END OF MODULE AC ---------------------------------*/


/*--------------------------- MODULE AA ------------------------------------*/
int Initialise(struct AutoFlags *Auto, struct SystemParams *SParam,
                    struct BrlParams *BParam,
#if !defined(BRAILLE_IT)
                    struct MoonParams *MParam,
                    struct XrefEntry *ConfigXref,
                    struct XrefEntry *BrlXref,
                    struct XrefEntry *MoonXref,
#endif
                    int argc, char *argv[]
)
{
/* NOTE : THE ORDER IN WHICH THE FOLLOWING STRUCTURES ARE PROVIDED
		MUST MATCH THE ORDER OF THE ORIGINAL STRUCTURES AS IN ITS.H   */

	struct AutoFlags TempAuto = {
		DEFAULTAUTOREQUIRED,
		DEFAULTAUTOBRLREQUIRED,
		DEFAULTAUTOBRLTRANS,
		DEFAULTAUTOBRLEMBFORM,
        #if !defined(BRAILLE_IT)
	        DEFAULTAUTOBRLPCLFORM,
    	    DEFAULTAUTOBRLPSCFORM,
            DEFAULTAUTOMOONREQUIRED,
        	DEFAULTAUTOMOONTRANS,
            DEFAULTAUTOMOONPSCFORM,
            DEFAULTAUTOMOONPCLFORM,
        #endif
		DEFAULTAUTOSCREENON
	};

	struct SystemParams TempSysParam = {
		DEFAULTCONFIGPARAMNAME,
        DEFAULTSOURCEDIR,
		DEFAULTSOURCEEXT,
		DEFAULTSOURCENAME,
        DEFAULTEMBOSSNAME
        #if !defined(BRAILLE_IT)
            ,DEFAULTPAGERANGE,
    	    DEFAULTCOPIES,
            DEFAULTNETWORK,
    	    DEFAULTCONFIRM,
        	DEFAULTPAUSE,
	        DEFAULTPCLNAME,
    	    DEFAULTPSCNAME,
        	DEFAULTEMBOSSPORT,
        	DEFAULTPCLPORT,
        	DEFAULTPSCPORT
    	#endif
    };

	struct BrlParams TempBrlParam = {
		DEFAULTBRLPARAMNAME,
		DEFAULTBRLTRANSINIT,
		DEFAULTBRLTRANSDIR,
		DEFAULTBRLTRANSEXT,
		DEFAULTBRLTRANSNAME,
		DEFAULTBRLEMBFORMDIR,
		DEFAULTBRLEMBFORMEXT,
		DEFAULTBRLEMBFORMNAME,
        DEFAULTBRLHEADERTYPE,
		DEFAULTBRLHEADERREFLECT,
		DEFAULTBRLTRANSGRADE,
		DEFAULTBRLTRANSTABLE,
        DEFAULTACRONYMTABLE,
        DEFAULTBRLEMBPAGEWIDTH,
		DEFAULTBRLEMBPAGELENGTH,
		DEFAULTBRLEMBFIRSTPAGE,
		DEFAULTBRLEMBFCODE,
#if !defined(BRAILLE_IT)
		DEFAULTBRLEMBCWT,
#endif
		DEFAULTBRLEMBREP
        #if !defined(BRAILLE_IT)
            ,DEFAULTBRLLINENOS,
        	DEFAULTBRLEMBSTARTBANNER,
        	DEFAULTBRLEMBENDBANNER,
        	DEFAULTBRLEMBSTARTSEQ,
        	DEFAULTBRLEMBENDSEQ,
            DEFAULTBRLPCLFORMDIR,
        	DEFAULTBRLPCLFORMEXT,
        	DEFAULTBRLPCLFORMNAME,
        	DEFAULTBRLPSCRIPTFORMDIR,
        	DEFAULTBRLPSCRIPTFORMEXT,
            DEFAULTBRLPSCRIPTFORMNAME
        #endif
    };

#if !defined(BRAILLE_IT)
		struct MoonParams TempMoonParam = {
			DEFAULTMOONPARAMNAME,
			DEFAULTMOONTRANSINIT,
			DEFAULTMOONTRANSDIR,
			DEFAULTMOONTRANSEXT,
			DEFAULTMOONTRANSNAME,
        	DEFAULTMOONPCLFORMDIR,
        	DEFAULTMOONPCLFORMEXT,
        	DEFAULTMOONPCLFORMNAME,
            DEFAULTMOONPSCFORMDIR,
            DEFAULTMOONPSCFORMEXT,
            DEFAULTMOONPSCFORMNAME,
			DEFAULTMOONHEADERTYPE,
			DEFAULTMOONHEADERREFLECT,
			DEFAULTMOONTRANSGRADE,
			DEFAULTMOONTRANSTABLE,
			DEFAULTMOONLINENOS,
            DEFAULTMOONPSCSTARTBANNER,
            DEFAULTMOONPSCENDBANNER,
            DEFAULTMOONPSCSTARTSEQ,
            DEFAULTMOONPSCENDSEQ,
            DEFAULTMOONPSCFONT,
            DEFAULTMOONPSCLMARGIN,
            DEFAULTMOONPSCRMARGIN,
            DEFAULTMOONPSCTOPMARGIN,
            DEFAULTMOONPSCBOTTOMMARGIN,
            DEFAULTMOONPSCLINEHEIGHT,
            DEFAULTMOONPSCFIRSTPAGE,
            DEFAULTMOONPSCFCODE,
            DEFAULTMOONPSCCWT,
            DEFAULTMOONPSCREP
		};
#else
    struct XrefEntry ConfigXref[NUMBERCONFIGDECODEPARAM];
	struct XrefEntry BrlXref[NUMBERBRLDECODEPARAM];
#endif

/* Set parameters to their default values */
	*Auto = TempAuto;
	*SParam = TempSysParam;
	*BParam = TempBrlParam;

	#ifndef BRAILLE_IT
		*MParam = TempMoonParam;
	#endif

/* Form parameter decode tables */
    if (!Form_Config_DecodeTable(SParam, ConfigXref))
		return(FALSE);
    if (!Form_Brl_DecodeTable(BParam, BrlXref))
		return(FALSE);

	#ifndef BRAILLE_IT
        if (!Form_Moon_DecodeTable(MParam, MoonXref))
			return(FALSE);
	#endif

#if !defined(BRAILLE_IT)
    if (!ParameterDecode(Auto, SParam, BParam,
      MParam,
      ConfigXref, BrlXref,
      MoonXref,
      argc, argv))
#else
    if (!ParameterDecode(Auto, SParam, BParam,
      ConfigXref, BrlXref,
      argc, argv))
#endif
        return(FALSE);

#ifndef KevTest
	/* Set up control-c handler */
  	if (signal(SIGINT, ctrlchandler) == SIG_ERR) {
	 	sprintf(errmsg, "Error while setting up control-c handling");
	 	ITS_Error(errmsg, "Initialise", -1, -1);
	 	return(FALSE);
  	}
#endif

	return(TRUE);
}
/*----------------------- END OF MODULE AA ---------------------------------*/


/*--------------------------------------------------------------------------*/
void CommandHelp()
{
#ifndef BRAILLE_IT
        printf( "\n\nIntegrated Translation System\n"
				"-----------------------------\n\n"
				"Command line structure:\n\n"
                "ITS [sourcename] [/CSname] [/CBname] [/B[T][E]] [/FBTname] [/FBEname]\n"
                "    [/M[T][P]] [/FMTname] [/FMPname] "
                "[/Q]\n\n"
				"In all of the above 'name' represents a file name\n"
                "source\t\t- name of source file for processing\n"
                "/CS\t\t- name of system configuration file\n"
                "/CB\t\t- name of braille configuration file\n"
                "/CM\t\t- name of Moon configuration file\n"
                "/BT\t\t- automatic braille translation\n"
                "/BE\t\t- automatic braille embosser formatting\n"
                "/B, /BTE\t- automatic braille translation and embosser formatting\n"
                "/FBT\t\t- translated braille file name (other than default)\n"
                "/FBE\t\t- embosser formatted braille file name (other than default)\n"
                "/MT\t\t- automatic Moon translation\n"
                "/MP\t\t- automatic Moon PostScript formatting\n"
                "/M, /MTP\t- automatic Moon translation and PostScript formatting\n"
                "/FMT\t\t- translated Moon file name (other than default)\n"
                "/FMP\t\t- PostScript formatted Moon file name (other than default)\n"
                "/Q\t\t- quiet automatic operation i.e. no screens"
				);
#else
        printf( "\n\nIntegrated Translation System - Braille It\n"
				"------------------------------------------\n\n"
				"Command line structure:\n\n"
                "ITSBRLIT [sourcename] [/CSname] [/CBname] [/FBTname] [/FBEname] [/Q]\n\n"
				"In all of the above 'name' represents a file name\n"
                "source\t\t- name of source file for processing\n"
                "/CS\t\t- name of system configuration file\n"
                "/CB\t\t- name of braille configuration file\n"
                "/FBT\t\t- translated braille file name (other than default)\n"
                "/FBE\t\t- embosser formatted braille file name (other than default)\n"
                "/Q\t\t- quiet automatic operation i.e. no screens\n\n"
				"Automatic braille translation and embosser formatting is assumed.\n"
                "For normal operation use: ITSBRLIT sourcename.\n"
                );
#endif
	return;
}
/*--------------------------------------------------------------------------*/
#ifdef CSCAPEVER
/*
	its.c
	generated by Look & Feel version 3.2 from its.lnf
	05/19/95
*/


fsyminit_struct symtab[] = {

	{FSYM_FIELDFUNCS },
	{"framer_funcs",	FNULL, (VOID *) &framer_funcs},
	{"list_funcs",		FNULL, (VOID *) &list_funcs},
	{"nowrite_funcs",	FNULL, (VOID *) &nowrite_funcs},
	{"pint_funcs",		FNULL, (VOID *) &pint_funcs},
	{"string_funcs",	FNULL, (VOID *) &string_funcs},
	{"togint_funcs",	FNULL, (VOID *) &togint_funcs},
	{"yesno_funcs",		FNULL, (VOID *) &yesno_funcs},

	{FSYM_USER },
	{"EmbFormatBrl",	(VOID_FPTR) EmbFormatBrl, NULL},
	{"EmbOutputBrl",	(VOID_FPTR) EmbOutputBrl, NULL},
	{"EmbTransFormatBrl",(VOID_FPTR) EmbTransFormatBrl, NULL},
	{"ExitFunc",		(VOID_FPTR) ExitFunc, NULL},
	{"Load_Brl_Param",	(VOID_FPTR) Load_Brl_Param, NULL},
	{"Load_Config_Param",(VOID_FPTR) Load_Config_Param, NULL},
	{"Load_Moon_Param",	(VOID_FPTR) Load_Moon_Param, NULL},
	{"PscFormatMoon",	(VOID_FPTR) PscFormatMoon, NULL},
	{"PscOutputMoon",	(VOID_FPTR) PscOutputMoon, NULL},
	{"PscTransFormatMoon",(VOID_FPTR) PscTransFormatMoon, NULL},
	{"Save_Brl_Param",	(VOID_FPTR) Save_Brl_Param, NULL},
	{"Save_Config_Param",(VOID_FPTR) Save_Config_Param, NULL},
	{"Save_Moon_Param",	(VOID_FPTR) Save_Moon_Param, NULL},
	{"Set_Brl_EmbBanSeq",(VOID_FPTR) Set_Brl_EmbBanSeq, NULL},
	{"Set_Brl_EmbPageForm",(VOID_FPTR) Set_Brl_EmbPageForm, NULL},
	{"Set_Brl_FileMan",	(VOID_FPTR) Set_Brl_FileMan, NULL},
	{"Set_Brl_TransOpt",(VOID_FPTR) Set_Brl_TransOpt, NULL},
	{"Set_Config_FileMan",(VOID_FPTR) Set_Config_FileMan, NULL},
	{"Set_Config_Operating",(VOID_FPTR) Set_Config_Operating, NULL},
	{"Set_Config_Output",(VOID_FPTR) Set_Config_Output, NULL},
	{"Set_Moon_FileMan",(VOID_FPTR) Set_Moon_FileMan, NULL},
	{"Set_Moon_PscBanSeq",(VOID_FPTR) Set_Moon_PscBanSeq, NULL},
	{"Set_Moon_PscPageForm",(VOID_FPTR) Set_Moon_PscPageForm, NULL},
	{"Set_Moon_TransOpt",(VOID_FPTR) Set_Moon_TransOpt, NULL},
	{"SetupBrl",		(VOID_FPTR) SetupBrl, NULL},
	{"SetupMoon",		(VOID_FPTR) SetupMoon, NULL},
	{"SetupSystem",		(VOID_FPTR) SetupSystem, NULL},
	{"TranslateBrl",	(VOID_FPTR) TranslateBrl, NULL},
	{"TranslateMoon",	(VOID_FPTR) TranslateMoon, NULL},
	{"aboutsys",		(VOID_FPTR) aboutsys, NULL},

	{FSYM_CLASS },
	{"sedwin_Class",	(VOID_FPTR) sedwin_Class, NULL},
	{"ufunc_Class",		(VOID_FPTR) ufunc_Class, NULL},

	{FSYM_BORDER },
	{"bd_null",			(VOID_FPTR) bd_null, NULL},
	{"bd_prompt",		(VOID_FPTR) bd_prompt, NULL},

	{FSYM_MOUSE },
	{"sedmou_Framer",	(VOID_FPTR) sedmou_Framer, NULL},
	{"sedmou_GreedyTrack",(VOID_FPTR) sedmou_GreedyTrack, NULL},
	{"sedmou_Track",	(VOID_FPTR) sedmou_Track, NULL},

	{FSYM_MOVEMETHOD },

	{FSYM_EXPLODE },

	{FSYM_SPECIAL },
	{"spc_Abort",		(VOID_FPTR) spc_Abort, NULL},
	{"spc_itsmenu",		(VOID_FPTR) spc_itsmenu, NULL},
	{"spc_setupmenu",	(VOID_FPTR) spc_setupmenu, NULL},

	{FSYM_AUX },
	{"aux_cursor",		(VOID_FPTR) aux_cursor, NULL},

	{FSYM_LISTEND}
};

/* ...................... */

#endif

#if defined(BRAILLE_IT)
    /* Ver 2.2 */
    struct MemBlockStruct
	{
   		struct MemBlockStruct *Next; /* Pointer to the next element */
   		char Block[250];	/* Block of memory equal to 1/4K - above pointer and malloc maintenance info.*/
 	};
 
int check_brailleit_mem(void)
{
 	struct MemBlockStruct *CurrentBlock;
 	struct MemBlockStruct *OldBlock;
 	struct MemBlockStruct *TempBlock;
    int Blocks;

 /* Ver 2.2 */

/* The number of 1/4k blocks needed to be available for ITSBRLIT to run.*/
#define MEMBLOCKSNEEDED 112 /* As at 2/5/95, 14:27 PM */

 Blocks = 0;            
 CurrentBlock = malloc(sizeof(struct MemBlockStruct));
 TempBlock = CurrentBlock;

 printf("Checking available memory...\n");
      
 while(CurrentBlock != NULL) {
 	OldBlock = CurrentBlock;
	CurrentBlock = malloc(sizeof(struct MemBlockStruct));
 	OldBlock->Next = CurrentBlock;
 	Blocks++;
 }/* End While */

 CurrentBlock = TempBlock;
 while(CurrentBlock != NULL) {
	TempBlock = CurrentBlock->Next;
	free(CurrentBlock);
	CurrentBlock = TempBlock;	
 }

 if (Blocks < MEMBLOCKSNEEDED) {/* MEMBLOCKSNEEDED is defined in this function! */
 	printf("Not enough memory available to run ITSBRLIT (only %d blocks, need at least %d).\nIncrease available memory or run the translate phase outside of WordPerfect.\n", Blocks, MEMBLOCKSNEEDED);
    return FALSE;
 }
 else
    printf("Enough memory is available (%d blocks).\n", Blocks);
    return TRUE;
#undef MEMBLOCKSNEEDED
}

void _setenvp(void) {}

#endif


/* ------------------------------------------------------------------------ */
int __cdecl main(int argc, char *argv[])
{
	#ifdef CSCAPEVER
	char ScreenName[FILENAMELENGTH];
	char HelpName[FILENAMELENGTH];
	FILE *helpfile;
	#endif
	char currentdrvdir[FILENAMELENGTH];


/* Command line help requested ? */
	if ((argc >= 2) &&
            ((_stricmp(argv[1], "/h") == 0) || (_stricmp(argv[1], "/?") == 0)))
	{       CommandHelp();
		exit(0);
	}

/* Determine current drive and directory */
	_getdcwd(_getdrive(), currentdrvdir, FILENAMELENGTH);
	if ((currentdrvdir[0] != '\0') && (currentdrvdir[1] == ':'))
	{       strncpy(current.drv, currentdrvdir, 2);
		strcpy(current.dir, &(currentdrvdir[2]));
	}
	else
		strcpy(current.dir, currentdrvdir);
/* Find location of this program */
	_splitpath(argv[0], prog.drv, prog.dir, prog.name, prog.ext);

#if defined(CSCAPEVER)
/* Initialise display */
	disp_Init(def_ModeText, cmwin_Class);
#endif

#if defined(BRAILLE_IT)
    if (!check_brailleit_mem())
        exit(-1);
#endif

/* Initialise system */                            

/****************************************************/
/* Following code either acknowledges Moon, or not! */
/* If you change one, change the other accordingly! */
/****************************************************/
    if (Initialise(&(Auto), &(SysParam), &(BrlParam),
#ifndef BRAILLE_IT
                &(MoonParam),
                ConfigXref,
                BrlXref,
                MoonXref,
#endif
                argc, argv))
    {
            if (Auto.Required){
				#ifndef CSCAPEVER
				if(Auto.ScreenOn)
                    printf("ITS Integrated Translation System    Version 3.11\n"); /* version */
				#endif
				ITSAutoProcess();
				}
			#ifdef CSCAPEVER
			else
			{
/* Locate screen file prior to opening (c-scape will create otherwise !!) */
				strcpy(ScreenName, SCREENFILENAME);
				if (!LocateFile(ScreenName))
				{       disp_Close();
					exit(-1);
				}
				if ((lnffile = sfile_Open(ScreenName, symtab)) == NULL)
				{       ITS_Error(" Error trying to open screen file", "main", -1, -1);
					disp_Close();
					exit(-1);
				}

/* Setup help system */
				strcpy(HelpName, HELPFILENAME);
				if (!LocateFile(HelpName))
				{       disp_Close();
					exit(-1);
				}
				if ((helpfile = ITS_fopen(HelpName, "rb")) != NULL)
				{       if (help_Init(helpfile, help_View, 1600, (VOID *) &hvd) != HELP_OK)
						ITS_Error("Cannot initialise help file", "ITSMenu", -1, -1);
				}

/* Load menu and background screen */
				if ((background = sfile_LoadSed(lnffile, "background", SED_ALLOC)) == NULL)
				{       ITS_Error(" Error trying to load background screen", "main", -1, -1);
					disp_Close();
					exit(-1);
				}

				sed_Repaint(background);
				ITSMenu();
				sed_Close(background);
			}
			#endif
	}

	#ifdef CSCAPEVER
	sfile_Close(lnffile);
	disp_Close();
	#endif

	return(0);
}
/* ------------------------------------------------------------------------ */


/* End of file ITS.C */
/* ITSAUTO.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.1*29/03/94*Moon PScript tidy: PCL := Psc; LMargin, RMargin,  *30412*NO */
/*   *        *TopMargin, BottomMargin are P_Page params.        *     *   */
/*   *        *PCL files renamed; help (/H) updated.             *     *   */
/***************************************************************************/
/*2.0*13/10/93*BRAILLE_IT compiler directive now comments out    *?????*KJM*/
/*   *        *code not specifically needed in 'ItsLite'.        *     *   */
/***************************************************************************/
/*1.5*29/05/92*Moon translation added. ITTRANS.H -> TRANFILE.H   *30301*MJH*/
/***************************************************************************/
/*1.4*29/05/92*Made functions for translate, format & output     *30301*MJH*/
/*   *        *follow 'standard' naming & calling convention.    *     *   */
/***************************************************************************/
/*1.3*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.2*18/05/92*ITBEMBOP.H reference removed.                     *30301*KJM*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*01/05/92*Creation Date.                                    *30301*MJH*/
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* ITSAUTO.OBJ                                                             */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

/* ------------------------ INCLUDE FILES --------------------------------- */

#ifndef BRAILLE_IT
    #include "itmpscfm.h"
#endif
/* --------------------- END OF INCLUDE FILES ----------------------------- */

/* ------------------------- GLOBAL DATA ---------------------------------- */
#ifdef BRAILLE_IT
	char EmbossableFilename[FILENAMELENGTH];
	char IntermediateFilename[FILENAMELENGTH];
#endif
/* ---------------------- END OF GLOBAL DATA ------------------------------ */

/* ------------------------------------------------------------------------ */
boolean BrlAutoProcess()
{	char tempsource[FILENAMELENGTH];
	char tempdest[FILENAMELENGTH];

#ifdef BRAILLE_IT
	strcpy(EmbossableFilename, "");
	strcpy(IntermediateFilename, "");
#endif

	if (Auto.BrlTrans)
	{	if (!ConfirmVitalTrans(tempsource, tempdest,
                            &(SysParam.Source), &(BrlParam.Trans)
#if !defined(BRAILLE_IT)
                            ,&(BrlParam.TransGrade),
                            BrlParam.TransTable
#endif
            ))
			return(FALSE);
			
#ifndef BRAILLE_IT
		if (!braille_translate(tempsource, tempdest, BrlParam.TransTable,
                                      &(BrlParam.TransInit), &(MoonParam.TransInit),
                             BrlParam.TransGrade,
                                      Auto.ScreenOn, FALSE))
#else
		if (!braille_translate(tempsource, tempdest, BrlParam.TransTable,
                                      &(BrlParam.TransInit),
                             BrlParam.TransGrade,
                                      Auto.ScreenOn))
#endif									 
			return(FALSE);
	}

	if (Auto.BrlEmbForm)
	{	if (BrlParam.Trans.Name[0] == '\0')
			strcpy(BrlParam.Trans.Name, SysParam.Source.Name);

		if (!ConfirmVitalEmbForm(tempsource, tempdest,
                            &(BrlParam.Trans), &(BrlParam.EmbForm)
#if !defined(BRAILLE_IT)
                            , &(BrlParam.EmbPage.Length), &(BrlParam.EmbPage.Width),
							&(BrlParam.EmbPage.FirstPage),
                            &(BrlParam.HeaderType), &(BrlParam.HeaderReflect)
#endif
             ))
			return(FALSE);
			
#ifdef BRAILLE_IT
		strcpy(EmbossableFilename, tempdest);
		sprintf(IntermediateFilename, "%s.%s", BrlParam.Trans.Name, BrlParam.Trans.Ext);
#endif
		if (!braille_emb_format(tempsource, tempdest,
                            BrlParam.EmbFormatCode ,
#if !defined(BRAILLE_IT)
                        	BrlParam.EmbCWT ,
#endif
							BrlParam.EmbReport,
							BrlParam.EmbPage.Width, BrlParam.EmbPage.Length,
							BrlParam.EmbPage.FirstPage,
							BrlParam.HeaderType, BrlParam.HeaderReflect,
                            FALSE, Auto.ScreenOn
#if !defined(BRAILLE_IT)
                            ,FALSE
#endif
          ))
			return(FALSE);
	}

	return(TRUE);
}
/* ------------------------------------------------------------------------ */


#ifndef BRAILLE_IT
/* ------------------------------------------------------------------------ */
boolean MoonAutoProcess()
{	char tempsource[FILENAMELENGTH];
	char tempdest[FILENAMELENGTH];

	if (Auto.MoonTrans)
	{	if (!ConfirmVitalTrans(tempsource, tempdest,
							&(SysParam.Source), &(MoonParam.Trans),
							&(MoonParam.TransGrade),
							MoonParam.TransTable))
			return(FALSE);
		if (!moon_translate(tempsource, tempdest, MoonParam.TransTable,
									  &(MoonParam.TransInit), &(BrlParam.TransInit),
                             MoonParam.TransGrade,
									  Auto.ScreenOn, FALSE))
			return(FALSE);
	}

    if (Auto.MoonPscForm)
    {   if (MoonParam.Trans.Name[0] == '\0')
            strcpy(MoonParam.Trans.Name, SysParam.Source.Name);

        if (!ConfirmVitalPscForm(tempsource, tempdest,
                            &(MoonParam.Trans), &(MoonParam.PscForm),
                            &(MoonParam.PscPage.FirstPage),
                            &(MoonParam.HeaderType), &(MoonParam.HeaderReflect)))
			return(FALSE);
        if (!Moon_Psc_format(tempsource, tempdest,
                            MoonParam.PscFormatCode , MoonParam.PscCWT ,
                            MoonParam.PscReport,
                            MoonParam.PscPage.LMargin, MoonParam.PscPage.RMargin,
                            MoonParam.PscPage.LineHeight,
                            MoonParam.PscPage.FirstPage,
                            MoonParam.HeaderType, MoonParam.HeaderReflect,
							FALSE, Auto.ScreenOn, FALSE))
			return(FALSE);
	}

    return(TRUE);
}
/* ------------------------------------------------------------------------ */
#endif


#ifdef BRAILLE_IT
/* ------------------------------------------------------------------------ */
void	StoreMacroFilenames()
{
	FILE *NameStore;    
	struct FilePath temp = {"\0", "\0", "\0", "\0"};
                                                  
	/* Handle the embossable filename */
    _splitpath(EmbossableFilename, temp.drv, temp.dir, temp.name, temp.ext);
    
    if (temp.drv[0] == '\0')
    	strcpy(temp.drv, current.drv);
    if (temp.dir[0] == '\0')
    	strcpy(temp.dir, current.dir);
    
	_makepath(EmbossableFilename, temp.drv, temp.dir, temp.name, temp.ext);
        
                                                  
    /* Handle the intermediate filename */
    _splitpath(IntermediateFilename, temp.drv, temp.dir, temp.name, temp.ext);
    
    if (temp.drv[0] == '\0')
    	strcpy(temp.drv, current.drv);
    if (temp.dir[0] == '\0')
    	strcpy(temp.dir, current.dir);
    
	_makepath(IntermediateFilename, temp.drv, temp.dir, temp.name, temp.ext);
                          
                                                  
	if ((NameStore = ITS_fopen("c:\\brl_it\\ITSNAMES.FIL", "wt")) == NULL)
		return;

	fprintf(NameStore, "%s\n", EmbossableFilename);	
	/*fprintf(NameStore, "%s\n", IntermediateFilename);*/
	/* The intermediate file is unconditionally deleted in the macro, so
	   why not just do it here and be done with it! */
    remove(IntermediateFilename);/* Verion  2.2 */
	
	fclose(NameStore);
	return;
}
/* ------------------------------------------------------------------------ */
#endif                      

                      
/* ------------------------------------------------------------------------ */
void	ITSAutoProcess()
{
/* Braille processing required ? */
	if (Auto.Brl)
		BrlAutoProcess();

#ifndef BRAILLE_IT
	/* Moon processing required ? */
		if (Auto.Moon)
			MoonAutoProcess();
#endif

#ifdef BRAILLE_IT	
	/* Write the contents of 'EmbossableFilename' and 'IntermediateFilename'
	   out to a file for the macro to pick up.*/
	StoreMacroFilenames();
	if (Auto.ScreenOn)
		{
		printf("\n");
#ifndef KevTest
				movecursor(24,0);
				printf("\aPress any key to continue ");
        		_getch();
#endif
			}
#endif
}
/* ------------------------------------------------------------------------ */

/* End of file ITSAUTO.C */
