/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/***************************************************************************/
/*               UNIFIED ITS SOURCE CODE 21-Mar-1995                       */
/***************************************************************************/
/* ITS - Scan format code table to see if input string is format code. */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.5*29/07/97*Bugfix (gdade/asc): not properly communicated from*60000*NO */
/* . *  /  /  *translator to formatter.                          *     *   */
/***************************************************************************/
/*1.4*??/11/95*BRL_Embform(): changed !0 to FALSE                *?????*NO */
/***************************************************************************/
/*1.3*04/05/95*Remove cWT for Braille It. Reduce INBUFSIZE for   *30418*NO */
/* . *  /  /  *Braille It. Amend inf. loop/l.h. margin bug.      *     *   */
/* . *  /  /  *Remove refs. to IT_report for Braille It.         *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*10/12/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      10/12/91 */
/* (Module AGEDHBBB) */
/* Returned: Pointer to the entry on success, null otherwise */

#include "its.h"
#include "itsfunc.h"
#include "itbf.h"
#include "itbembfm.h"

struct IT_FCode	*isFC (char *instr, char *FCbuffer, struct IT_FCode **FCtable)
{
  struct IT_FCode *FCp;
  size_t i, e;
  flag FCflg = false;

  e = 0;
  do {
    FCp = FCtable[e++];
    for (i=0;
	(FCp->in[i] != 'n' && FCp->in[i] != 'c' && FCp->in[i] != 's'
	&& FCp->in[i] != '\0')
	&& toupper(instr[i]) == FCp->in[i];
	++i)
      ;
    if (i > 0 && (FCp->in[i] == 'n' || FCp->in[i] == 'c'
	|| FCp->in[i] == 's' || FCp->in[i] == '\0'))
      FCflg = true;
  } while (FCtable[e] != NULL && FCflg == false);

  if (FCflg == false)
    return NULL;
  /* Assign the output from the FC table to FCbuffer.
     Space should already be allocated for this string. */
  strcpy(FCbuffer, FCp->out);

  return FCp;
}

/* ITS - IT_BEmbCentre .. Centres text in specified line-length */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*02/12/93*Odd-numbered line-length needs extra space on left*3040?*NO */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*22/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* (Module AGEDHF) */
/* N. Octon	22/11/91 */
/* Return: *centred line on success, null otherwise. */
#define MINSPACES 5 /* Minimum room on line for centring to be allowed */

const char space = ' ';

char *IT_BEmbCentre (char *text, size_t maxlen, size_t startpos
#if !defined(BRAILLE_IT)
, int *cwt
#endif
)
{
  static char ct[64];
  size_t tmplen, spaces, cwu;
  char modulename[] = "IT_BEmbCentre";

  if (strcpy(ct, text) == NULL) {
	 IT_error(modulename, "Error: memory allocation");
    return NULL;
  }
  /* We now have a private copy of text & space to centre it in. */

  /* Calc. string length in CWT units. */
  tmplen = TextLength(ct, cwt);

  /* Is there too much text to process within maxlen? */
  if (startpos + tmplen + (MINSPACES * 2) > maxlen) /* Not enough room */
  {
    return NULL;
  }

  /* Insert required no. of spaces at beginning of string */
  cwu = ((maxlen - tmplen - startpos) / 2);
  spaces = cwu / cw(space);
  /* Insert extra space for odd-numbered line-length: v1.3*/
  if ((tmplen = strlen(ct)) % 2 == 1)
    ++spaces; /*v1.3*/
  memmove(ct + spaces, ct, strlen(ct) + 1);
  memset(ct, space, spaces);

  return ct;
}
/* ITS - IT_error - Focus all errors here to send them:-
	(a) to the screen-handling functions for window display, and
	(b) if a system error, display it and exit program
*/
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*26/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon	26/11/91 */
/* Return: 0 if error is recovered from, -1 if non-system error, errno otherwise. */


int _cdecl IT_error (char *modulename, char *fmt, ...)
{
  char *IT_errorstr, *syserrstr;
  va_list marker;
  int sverrno;

  sverrno = errno; /* Save system error indicator */

  if ((IT_errorstr = (char *)malloc(128)) == NULL) {
	 ITS_Error("Memory allocation error", "IT_error", -1, -1);
    return errno;
  }
  va_start(marker, fmt);
  vsprintf(IT_errorstr, fmt, marker);
  va_end(marker);

  /* If system error, make system error message string */
  if (errno != 0) {
    syserrstr = _strerror(IT_errorstr);
    syserrstr[strlen(syserrstr)-1] = '\0'; /* Eliminate newline */
    strcpy(IT_errorstr, syserrstr);
  }

  /* Call CScape function here */
  ITS_Error(IT_errorstr, modulename, -1, -1);

  free(IT_errorstr);
  return sverrno;
}
/* ITS - IT_BEmbPad .. Pads line with spaces in specified line-length */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*27/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* (Module AGEDHH) */
/* N. Octon	27/11/91 */
/* Return: *text on success, null otherwise. */


const char *space_str = " ";

char *IT_BEmbPad (char *text, size_t length
#if !defined(BRAILLE_IT)
  , int *cwt
#endif
)
{
  size_t l;

  /* Is there too much text to process within length? */
  if ((l = TextLength(text, cwt)) >= length) /* Not enough room */
  {
    return NULL;
  }

  /* Put required no. of spaces at end of string */
  for (l=l; l<length; l+=cw(space))
    strcat(text, space_str);
  text[l] = '\0';

  return text;
}
/* ITS - IT_BEmbRjust .. Right-justifies text in specified line-length */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*22/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* (Module AGEDHG) */
/* N. Octon	22/11/91 */
/* Return: *right-justified line on success, null otherwise. */
#define MINSPACES 5 /* Minimum room on line for right-justifying to be allowed */


char *IT_BEmbRjust (char *text, size_t maxlen, size_t startpos
#if !defined(BRAILLE_IT)
  , int *cwt
#endif
)
{
  static char rt[64];
  size_t tmplen, spaces, cwu;
  char modulename[] = "IT_BEmbRjust";
  char tmpstr[64];

  if (strcpy(rt, text) == NULL) {
	 IT_error(modulename, "Error: memory allocation");
    return NULL;
  }
  /* We now have a private copy of text & space to Right-justify it in. */

  /* Calc. string length in CWT units. */
  tmplen = TextLength(rt, cwt);

  /* Is there too much text to process within maxlen? */
  if (startpos + tmplen + (MINSPACES) > maxlen) /* Not enough room */
  {
    return NULL;
  }

  /* Insert required no. of spaces at beginning of string */
  cwu = (maxlen - tmplen - startpos);
  spaces = cwu / cw(space);
  memset(tmpstr, space, spaces);
  tmpstr[spaces] = '\0';

  strcat(tmpstr, rt);
  strcpy(rt, tmpstr);

  return rt;
}
/* ITS - IT_confirmpage finally determines end-of-page. */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*07/06/94*Amend introduced bug where centring more than a   *60000*NO */
/* . *  /  /  *page causes infinite loop                         *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*11/03/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      11/03/92 */
/* (Module AGEDHM) */
/* Returned: LINECOUNT (MODIFIED OR UNMODIFIED ON SUCCESS; 0 ON ERROR.) */


int IT_confirmpage(struct txtline line[], unsigned long *modeflags,
   size_t pagesize,
   size_t *linecount, size_t pagecount)
{
  int i;
  size_t blocksize;

  i = (int) *linecount - 1;
  blocksize = pagesize - *linecount;

  while (i >= 0
  && line[i].attribs & T_HEADER
  || (line[i].attribs & BLOCK && *modeflags & BLOCK)
  || *modeflags & FOOTER) {
    if (blocksize && blocksize <= pagesize
    && !(*modeflags & BLOCK)) {
       /* Block will fit on page */
      i -= (int) blocksize;
    }
    else if (blocksize == pagesize && *modeflags & BLOCK) {
       /* Block too big */
#if !defined(BRAILLE_IT)
      IT_report("Warning: block of text split over pages %d-%d",
	 pagecount, pagecount+1);
#endif
      break;
    }
    *modeflags = line[i].attribs;
    --i;
  }

  if (i != -1)
  	*linecount = i + 1;

  return __min(i, (int) *linecount); /* Allows 0 return on error */
}
/* IT_getFCparams - stores all format code params in FCparam structure */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*06/12/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* (module AGEDHBBF) */
/* Returned: pointer to the filled structure */
/* N. Octon	06/12/91 */

struct FCparam *IT_getFCparams(char *FCstring, struct FCparam *params,
    char *FCcore, struct IT_FCode *FCentry)
{
/* When scanning the generic format code, n = numeric (int), c = character,
   s = trailing string.  These constants are lower case. */
  size_t i, j, k;
  int *p; /* Points to successive integer parameters in FCparam */

  p = &params->npar1;
  i = j = k = 0;
  while (j<=strlen(FCstring)
  && FCentry->in[i] != '\0') {
    if (FCstring[j] == 'l') { /* Leave param setting */
      ++j;
      i = strlen(FCentry->in);
      continue;
    }
    if (FCstring[j] == 'i' /* increment numeric */
    || FCstring[j] == 'd' /* decrement numeric */
    ) {
      while (FCstring[j] == 'i') {
	++*p;
	++j;
      }
      while (FCstring[j] == 'd' && *p > 0) {
	--*p;
	++j;
      }
    }
    else if (FCentry->in[i] == 'n') { /* numeric */
      *p = atoi(&FCstring[j]);
      if (FCstring[j] != '0' && *p == 0) /* '0' is valid */
	*p = FCentry->dflt;
      while (isdigit(FCstring[j]))
	++j;

      /* Superfluous string attached to format code? */
      if (FCentry->in[i+1] == '\0' /* No more params to process */
      && FCstring[j] != space && FCstring[j] != '\0')
	 /* FCstring has a non-param string attached */
#if !defined(BRAILLE_IT)
    IT_report("Warning: Word joined to format code\n"
	   "Word omitted from output file: %s", FCstring);
#endif
      /* Keep param within r1-r2 */
      if (*p < FCentry->r1)
	*p = FCentry->r1;
      else if (FCentry->r2 > 0 && *p > FCentry->r2)
	*p = FCentry->r2;

      if (p == &params->npar1)
	++p;
    }
    else if (FCentry->in[i] == 'c') {
      params->spar[0] = FCstring[j++];
      params->spar[1] = '\0';
    }
    else if (FCentry->in[i] == 's') {
      strcpy(params->spar, &FCstring[j]);
      i = strlen(FCentry->in);
      j = strlen(FCstring);
    }
    else /* Not symbolic char; copy to FCcore */
      FCcore[k++] = FCstring[j++];
    ++i;
  }
  FCcore[k] = '\0';

  return params;
}

/* ITS - IT_formatexec: Carry out formats on input text & write it to output */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.5*07/06/94*IT_confirmpage() returns 0 on error.              *60000*NO */
/***************************************************************************/
/*2.4*03/12/93*$HDO fix revised; header-type 2 mod.              *3040?*NO */
/***************************************************************************/
/*2.3*27/10/93*$HDO fix.                                         *3040?*NO */
/***************************************************************************/
/*2.2*25/05/93*$HD forces its header to be read even at end-of-pg*30404*NO */
/***************************************************************************/
/*2.1*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*2.0*09/07/92*P_HEADER_OVRWRT indicates page-header overwrite   *30301*NO */
/***************************************************************************/
/* . *  /  /  *nostore prevents storing overflow forced p_header line      */
/***************************************************************************/
/*1.9*29/06/92*p_header_ind indicates page-header overflow       *30301*NO */
/***************************************************************************/
/*1.8*08/06/92*oldpp in pg_settings ensures retrospctv. updating *30301*NO */
/* . *  /  /  *of printpage no. when anticipat. headers not used.*     *   */
/***************************************************************************/
/*1.7*03/06/92*The flag prevforcedpage ensures softpage suppress *30301*NO */
/* . *  /  /  *after hardpage, but hardpage/hardpage allowed     *     *   */
/***************************************************************************/
/*1.6*26/05/92*The flag header_swap ensures that old_header is   *30301*NO */
/* . *  /  /  *is updated even when storecount is never 1        *     *   */
/***************************************************************************/
/* . *  /  /  *Initialise old_header on entry to IT_formatexec() *     *   */
/***************************************************************************/
/*1.5*22/05/92*Save modeflags to restore after IT_confirmpage()  *30301*NO */
/***************************************************************************/
/*1.4*21/05/92*Move screen pagecount updating to bottom of loop  *30301*NO */
/*   *        *Screen pagecount is pagecount-1                   *     *   */
/***************************************************************************/
/*1.3*20/05/92*Move condition clearing hdr buf up in hierarch.   *30301*NO */
/***************************************************************************/
/*1.2*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.1*06/05/92*Integration work.                                 *30301*MJH*/
/***************************************************************************/
/*1.0*30/01/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

/* N. Octon      30/01/92 */
/* (Module AGEDH) */
/* Returned: 0 on success, else nonzero. */
/* Integration work.  M.J. Harrhy 6/5/92 */


char *form_ver = "1.7";

int IT_formatexec(FILE *source, FILE *dest, char *infl_name, char *outfl_name,
   size_t linelength, size_t pagelength, size_t startpage, int header_ind,
   flag header_ref,
   struct IT_FCode **FCtable,
#if !defined(BRAILLE_IT)
   int *cwt,
#endif
   boolean ScreenOn
#if !defined(BRAILLE_IT)
   ,boolean SysPause
#endif
)
{
    #ifdef CSCAPEVER
    char formstr[FORMBOXWIDTH + 1];
	char tostr[FORMBOXWIDTH + 1];
	char titlestr[FORMBOXWIDTH + 1];
	char formsource[FILENAMELENGTH];
	char formdest[FILENAMELENGTH];
    #endif

	#ifdef CSCAPEVER
	menu_type process_menu;
	sed_type process_sed;
	#endif

  flag forcedpage, prevforcedpage, EndOfText, no_pg_breaks;
  static struct txtline line[MAXPAGELEN], *linep;
  static struct txtline blankline;
		/* static ensures initialisation to 0 / '\0' */
  static char cur_header[MAXLINELEN+1], old_header[MAXLINELEN+1],
	 spaceheader[MAXLINELEN+1], ovrwrt_header[MAXLINELEN+1];
  static char *header;
  static struct pg_settings p;
  unsigned long modeflags, svmode;
  flag t_hdrind;
  char *ptr, *printpageptr;
  char modulename[] = "IT_formatexec";
  flag header_swap;
  flag p_header_ovflg, nostore;
  int p_header_ind = 0;
  size_t lpn; /* Length of Page Number */

  /* Initialisation */
  EndOfText = false;
  modeflags = svmode = NORMAL;
  p.blocksize = p.blanklines = 0;
  p.linecount = p.storecount = 0;
  p.pagecount = max(1, startpage);
  p.lmarg = p.rmarg = 0;
  p.abslinelength = linelength;
  memset(p.pp, '\0', MAXLINELEN/4);
  memset(p.oldpp, '\0', MAXLINELEN/4);
  memset(spaceheader, space, linelength);
  p.paginat_ind = ARABIC; /* Default */
  p.header_ind = header_ind;
  if (pagelength > 0 && p.header_ind == 3)
    --pagelength;
  if (pagelength == 0) {
    no_pg_breaks = true;
    pagelength = MAXPAGELEN;
  }
  else
    no_pg_breaks = false;
  p.no_pg_breaks = no_pg_breaks;
  t_hdrind = false;
  cur_header[0] = '\0';
  strcpy(old_header, spaceheader);
  strcpy(ovrwrt_header, spaceheader);
  header_swap = false;
  prevforcedpage = false;
  printpageptr = (header_ref) ? p.pp : p.oldpp;
  p_header_ovflg = false;
    abrt = FALSE;

#ifdef CSCAPEVER
/* Clear keyboard buffer */
    while (_kbhit())
        _getch();

/* Setup screen */
	if (ScreenOn)
	{       process_menu = menu_Open();
	process_sed = sed_Open(process_menu);

	sed_SetBorder(process_sed, bd_null);
	sed_SetBorderColor(process_sed, FORMBOXPROMPTCLR);

	sed_SetColors(process_sed, FORMBOXREGCLR, FORMBOXBACKCLR, FORMBOXHICLR);
	sed_SetPosition(process_sed, FORMBOXROWPOS, FORMBOXCOLPOS);
	sed_SetShadow(process_sed, 1);
	sed_SetShadowAttr(process_sed, 0x07);
	sed_SetHeight(process_sed, FORMBOXHEIGHT);
	sed_SetWidth(process_sed, FORMBOXWIDTH);

	sprintf(titlestr, "ITS Formatting Process   Version %s", form_ver);
	strcenter(titlestr, FORMBOXWIDTH);
	menu_Printf(process_menu, "@p[1,0]%s", titlestr);

	strcpy(formstr, "Formatting the following source file:");
	strcenter(formstr, FORMBOXWIDTH);
	menu_Printf(process_menu, "@p[3,0]%s", formstr);

	strcpy(formsource, infl_name);
	strcenter(formsource, FORMBOXWIDTH);
	menu_Printf(process_menu, "@p[4,0]%s", formsource);

	strcpy(tostr, "To the following destination file:");
	strcenter(tostr, FORMBOXWIDTH);
	menu_Printf(process_menu, "@p[5,0]%s", tostr);

	strcpy(formdest, outfl_name);
	strcenter(formdest, FORMBOXWIDTH);
	menu_Printf(process_menu, "@p[6,0]%s", formdest);

	menu_Printf(process_menu, "@p[8,20]Page Count: %6d", p.pagecount);
	movecursor(FORMBOXROWPOS + 8, FORMBOXCOLPOS + 33);

	sed_Repaint(process_sed);
	sed_BorderPrompt(process_sed, " Press ESCAPE to abort process");
	sed_Repaint(process_sed);
	}
#else
	if(ScreenOn)
		{
		printf("\n\nITS Formatting Process   Version %s", form_ver);
		printf("\nFormatting the following source file: %s",infl_name);
		printf("\nTo the following destination file: %s\n",outfl_name);
		}
#endif

  /* Don't process an empty file */
  if (!_filelength(_fileno(source))) {
	 IT_error(modulename, "Empty file .. Nothing to format!");
	 #ifdef CSCAPEVER
	 if (ScreenOn)
		sed_Close(process_sed);
	 #endif
	 return 0;
  }

  while (!EndOfText) {

    forcedpage = false;
    while (!EndOfText && !forcedpage
    && (p.storecount < pagelength || modeflags & P_HEADER)) {
	  lpn = (p.paginat_ind == ARABIC) ? strlen(IToBrl(p.pagecount))
          : strlen(IToBRom(p.pagecount));
      linelength = (modeflags & (CENTRE | P_HEADER | P_HEADER_OVRWRT))
          ? p.abslinelength - 10
          : (modeflags & RJUST) ? p.abslinelength - 5
          : (p.header_ind == 2 && p.linecount == 0) ? p.abslinelength - lpn - 2
          : p.abslinelength;

	  /* Build line */
	  nostore = false;
      if (p.blanklines == 0 /* if !0 next part of condition will not be tried */
      && (linep = IT_bembline(linelength, pagelength, &p,
#if !defined(BRAILLE_IT)
        cwt,
#endif
     	FCtable,
	 source, &modeflags, &forcedpage, &EndOfText)) == NULL
	&& EndOfText == false) {
	#ifdef CSCAPEVER
	if (ScreenOn)
	  sed_Close(process_sed);
	pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
	#endif
	return !0;
	}

	/* Clear p_header_ind for non-page-header line */
    if (p_header_ind
    && !(modeflags ^ linep->attribs & (P_HEADER | P_HEADER_OVRWRT))
    )
        p_header_ind = 0;

	if (linep != NULL
	&& linep->text[0] != '\0') { /* Don't want to centre, r-just. etc
		 a blank line */
		if (linep->attribs & P_HEADER_OVRWRT) { /* Force writing on page header */
          if (p_header_ovflg) { /* If already set, this line is overflow */
			nostore = true;
          }
        }
        if (linep->attribs & (P_HEADER | P_HEADER_OVRWRT)) {
			/* Store centred & padded page header ..
				 (Module AGEDHJ)
			*/
			if (p_header_ind == 0) { /* Not an overflow */
            if ((ptr = IT_BEmbCentre(linep->text, p.abslinelength, 0
#if !defined(BRAILLE_IT)
              , cwt
#endif
            )) == NULL)
			{
				IT_error(modulename, "Centring error: page %d page-header",
					 p.pagecount);
				#ifdef CSCAPEVER
				if (ScreenOn)
					sed_Close(process_sed);
				pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
				#endif
				return !0;
			}
            if ((ptr = IT_BEmbPad(ptr, p.abslinelength
#if !defined(BRAILLE_IT)
              , cwt
#endif
            )) == NULL)
			{
			  IT_error(modulename, "Space-fill error: page %d page-header",
			     p.pagecount);
			  #ifdef CSCAPEVER
			  if (ScreenOn)
			    sed_Close(process_sed);
			  pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
			  #endif
			  return !0;
            }
            if (!(linep->attribs & P_HEADER_OVRWRT))
              strcpy(cur_header, ptr);
            else if (!p_header_ovflg) {
              strcpy(ovrwrt_header, ptr);
              p_header_ovflg = true;
            }
			linep = NULL;
			p.lptr = 0;
			++p_header_ind;
			}
		}
		else if (linep->attribs & CENTRE) {
            if ((ptr = IT_BEmbCentre(linep->text, p.abslinelength, 0
#if !defined(BRAILLE_IT)
              , cwt
#endif
            )) == NULL)
			{
				IT_error(modulename, "Centring error: page %d line %d",
					 p.pagecount, p.storecount);
				#ifdef CSCAPEVER
				if (ScreenOn)
					sed_Close(process_sed);
				pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
				#endif
				return !0;
			}
			strcpy(linep->text, ptr);
		}
		else if (linep->attribs & RJUST) {
            if ((ptr = IT_BEmbRjust(linep->text, p.abslinelength, 0
#if !defined(BRAILLE_IT)
              , cwt
#endif
            )) == NULL)
			{
				IT_error(modulename, "Right-just.  error: page %d line %d",
					 p.pagecount, p.storecount);
				#ifdef CSCAPEVER
				if (ScreenOn)
					sed_Close(process_sed);
				pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
				#endif
				return !0;
			}
			strcpy(linep->text, ptr);
		}
	}

	if (nostore)
	  linep = NULL;

      if (forcedpage
      && p.lptr == 0
      && p.linecount == 0 && p.pagecount == startpage)
	forcedpage = false; /* Suppress initial page-throw */
      if (forcedpage) {
	/* Clear away attributes which prevent page-split here
	   (MODULE AGEDHK)
	*/
	if (linep != NULL)
	  linep->attribs &= ~(T_HEADER | BLOCK);
	p.linecount = p.storecount;
      }

      if (linep != NULL && linep->text[0] != '\0'
	  && !nostore
      && !(linep->attribs & (P_HEADER | P_HEADER_OVRWRT))) {
	  if (IT_storeline(line, linep, &p.linecount, &p.storecount,
	&p.abslinecount) == NULL) {
	  #ifdef CSCAPEVER
	  if (ScreenOn)
		 sed_Close(process_sed);
	  pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
	  #endif
	  return !0;
	}
	 }
      /* Blank lines */
      if (p.blanklines && modeflags & BLANK) {
	if (p.linecount + p.blanklines < pagelength) {
	  for (p.blanklines=p.blanklines; p.blanklines && p.linecount < pagelength;
	     --p.blanklines) {
	    if (IT_storeline(line, &blankline, &p.linecount, &p.storecount,
	       &p.abslinecount)
	 == NULL) {
		#ifdef CSCAPEVER
		if (ScreenOn)
		  sed_Close(process_sed);
		pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
		#endif
		return !0;
	 }
	    line[p.storecount-1].attribs |= modeflags;
	  }
	  modeflags ^= BLANK;
	}
	else { /* Reduce blanklines to 1 & force new page */
	  p.blanklines = 1;
	  forcedpage = true;
	  linep = NULL; /* Prevent previous nonblank line
	     from being stored again */
	}
      }

      /* Clear cur_header? */
      else if (modeflags & P_HEADER && p_header_ind == 0
      && linep != NULL && !(linep->attribs & (CENTRE | RJUST | BLOCK))
      && !EndOfText && !forcedpage
      && p.storecount < pagelength
      )
		cur_header[0] = '\0';

      /* Update header if necessary */
      if (p.storecount >= 1 && header_swap == true) {
        header_swap = false;
        if (header_ref == false ) { /* Using old_header, not cur_header */
          strcpy(old_header, cur_header);
          strcpy(p.oldpp, p.pp);
        }
      }

      /* Report new T_HEADER entry */
      if (t_hdrind && !(modeflags & NEWTITLE)) {
	t_hdrind = false;
#if !defined(BRAILLE_IT)
    IT_report("New heading %10s Page %4d", "..........", p.pagecount);
#endif
      }
      else if (linep != NULL && (linep->attribs & NEWTITLE))
	t_hdrind = true;
    } /* End-of-page? */

    /* EndOfText? */
    if (EndOfText)
      p.linecount = p.storecount; /* Ensure last line(s) are not missed */

    /* Confirm end-of-page */
    svmode = modeflags; /* Save modeflags because it is not yet stored
				& may be overwritten by IT_confirmpage() */
    if (!forcedpage
    && !EndOfText
    && !IT_confirmpage(line, &modeflags, p.storecount,
       &p.linecount, p.pagecount)) {
      IT_error(modulename,
          "Page boundary cannot be set: Too many headers/footers: page %d", p.pagecount);
      #ifdef CSCAPEVER
      if (ScreenOn)
	sed_Close(process_sed);
      pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
      #endif
      return !0;
    }

	header = (p.header_ind == 1 || p.header_ind == 2) ? NULL
            : (strspn(ovrwrt_header, " ") != p.abslinelength)
                 ? ovrwrt_header
			: ((line[0].attribs & NEWTITLE) && (line[0].attribs & BLANK))
				 ? spaceheader
			: (header_ref && cur_header[0] != '\0') ? cur_header
			: (!header_ref) ? old_header
			: spaceheader;
    if ((p.linecount > 0
    || (forcedpage && prevforcedpage))
	&& IT_mkpage(line, header, printpageptr, p.paginat_ind, p.header_ind,
       (forcedpage) ? 0 : no_pg_breaks, /* no_pg_breaks may be true/false */
       &p.pagecount, &p.linecount, &p.storecount, p.abslinelength, dest) != 0)
    {
      #ifdef CSCAPEVER
      if (ScreenOn)
	sed_Close(process_sed);
      pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
      #endif
      /* Re-init. buffers by special call to IT_bembline() */
      IT_bembline(linelength, pagelength, &p,
#if !defined(BRAILLE_IT)
        cwt,
#endif
     FCtable, NULL, &modeflags, &forcedpage, &EndOfText);
      return !0;
    }

/* Update page count on screen */
#ifdef CSCAPEVER
    if (ScreenOn)
    {       menu_Printf(process_menu, "@p[8,33]%6d", (p.pagecount-1));
      movecursor(FORMBOXROWPOS + 8, FORMBOXCOLPOS + 33);
      sed_Repaint(process_sed);
    }
#else
	if(ScreenOn)
		{
		printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bPage Count: %6d", (p.pagecount-1));
		}
#endif

    /* Restore saved modeflags */
    modeflags = svmode;
    svmode = NORMAL;

	/* header_swap flag is set here if needed */
    if (header_ref == false)
      header_swap = true;

    /* Set prevforcedpage = forcedpage to reflect previous page status */
	prevforcedpage = forcedpage;

	/* Reset p_header_ovflg if set */
    if (p_header_ovflg) {
	  p_header_ovflg = false;
    }
    /* Re-init. ovrwrt_header */
    memset(ovrwrt_header, ' ', p.abslinelength);
  } /* End-of-file */
/* Remove any outstanding report message */
  #ifdef CSCAPEVER
  pop_Message(NULL, 21, 1, 1, 76, 0x4F, bd_box);
  #endif

/* Provide closing message and wait */
	#ifdef CSCAPEVER
	if (ScreenOn)
	{
		if (SysPause)
		{       menu_Printf(process_menu, "@p[8,14]Final");
			menu_Printf(process_menu, "@p[8,41]@fd1[ OK ]", NULL, &nowrite_funcs,
				" Process completed ... Press ESCAPE or ENTER to continue.");
			movecursor(FORMBOXROWPOS + 8, FORMBOXCOLPOS + 33);
			tone();
			sed_Repaint(process_sed);
			sed_Go(process_sed);
		}
		sed_Close(process_sed);
	}
	#endif

  /* Re-init. buffers by special call to IT_bembline() */
  IT_bembline(linelength, pagelength, &p,
#if !defined(BRAILLE_IT)
    cwt,
#endif
     FCtable, NULL, &modeflags, &forcedpage, &EndOfText);

  return 0;
}
/* ITS - IT_getword - Gets wordstring from input block. */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*09/12/94*Format/dash lost text - cured in nxtword().       *60000*NO */
/***************************************************************************/
/*1.4*20/07/93*Wbuffer now dimensioned to 256 to prevent overflow*30406*NO */
/***************************************************************************/
/*1.3*22/05/92*nxtword() includes 0x1A (eof) as white space.     *30301*NO */
/* . *	/  /  * 	 Prevent *bufleft from wrapping round	 *    *    */
/***************************************************************************/
/*   *	      *IT_readin() puts \0 after every readin		 *30301*   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.			 *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*04/12/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      04/12/91 */
/* (Module AGEDHBB) */
/* Returned: pointer to wordstring on success (word or format code found),
	 null on error or on re-init.
   or on end-of-text with no more words left.
   On end-of-text, the EndOfText flag is set.
   Note: stored format code parameters are not overwritten until a new
   param of the same type is found.
*/

#if !defined(BRAILLE_IT)
#define INBUFSIZE 1024
#define BUFLEVEL 255
#else
#define INBUFSIZE 256
#define BUFLEVEL 128
#endif
#define BUFFULL (unsigned)INBUFSIZE
#define MAXFCSTORE 63


char *IT_getword (FILE *infile, char *inbuffer, char *Wbuffer, char *FCbuffer,
    struct FCparam *FCparbuffer, struct IT_FCode **FC, flag *EndOfText)
{
  static char FCstore[64], *FCp1, Wstore[256];
  struct IT_FCode *FCp;
  char tmpstr[256], *tmpptr, *storep;
  static bufleft;
  static char *bufptr;
  flag FCfound, storedFCflg;
  static flag noFCallowed;
  size_t i;
  char modulename[] = "IT_getword";

  FCfound = storedFCflg = false;
  tmpstr[0] = '\0';

  if (infile == NULL) { /* Re-init. internal settings */
    bufleft = FCparbuffer->npar1 = FCparbuffer->npar2 = 0;
    inbuffer[0] = Wstore[0] = FCstore[0] = Wbuffer[0]
      = FCparbuffer->spar[0] = '\0';
    noFCallowed = false;
    return NULL;
  }
  else
    Wbuffer[0] = '\0';

  /* Priority of extraction: stored word; stored FC; word/FC from input stream */

  /* Word in store? */
  if (Wstore[0] != '\0') {
    strcpy(Wbuffer, Wstore);
    Wstore[0] = FCbuffer[0] = '\0';
    return Wbuffer;
  }

  /* If format code left in store, use it; else if buffer low, read more */
  if (FCp1 != NULL) { /* Format code left in store */
    storedFCflg = true;
    FCp1 = strchr(FCstore+1, ITFCSTART);
    if (FCp1 != NULL) { /* More than 1 format code left */
      strncpy(tmpstr, FCstore, FCp1-FCstore);
      tmpstr[FCp1-FCstore] = '\0';
      strcpy(FCstore, FCp1);
      FCp1 = FCstore;
    }
    else {
      strcpy(tmpstr, FCstore);
      FCstore[0] = '\0';
      FCp1 = NULL;
    }
    FCfound = true;
  }
  else if (bufleft <= BUFLEVEL && !feof(infile)) { /* Fill up with more */
    if ((bufptr = IT_readin(infile, inbuffer, bufptr, &bufleft)) == NULL)
      return NULL;
  }

  /* Do we need a wordstring? */
  if (FCfound == false && tmpstr[0] == '\0') {
    /* If no more words left, set EndOftext & return */
    if (!bufleft) {
      *EndOfText = true;
      bufptr = NULL;
      inbuffer[0] = '\0';
      return NULL;
    }

    /* Before calling nxtword(), bufptr must not point to the
       terminating null char */
    if ((tmpptr = nxtword(&bufptr, &bufleft)) != NULL)
      strcpy(tmpstr, tmpptr);
    else /* Error */
      return NULL;
  }

  /* Is it a word or a format code? */
  if (tmpstr[0] == ITFCSTART
  && (_stricmp(tmpstr, ITFCstart"asc") == 0
  || !noFCallowed))
  {
    char dummyFCstore[30];
    /* If storedFCflg set, do not overwrite contents of FCstore */
    storep = (storedFCflg == false) ? FCstore : dummyFCstore;
    FCp = isFC(tmpstr, storep, FC);
    if (FCp != NULL) {
      FCfound = true;
      /* Get FC params from input FC before possibly overwriting it */
      if (islower(FCp->in[strlen(FCp->in) - 1])
      && storedFCflg == false
      ) {
    _strupr(tmpstr);
	if (IT_getFCparams(tmpstr, FCparbuffer, FCbuffer, FCp) == NULL) {
	  return NULL;
	}
      }

      /* Check on FCstore overflow */
      if (strlen(FCstore) > MAXFCSTORE) {
	IT_error(modulename, "Format code string too long:\n%s", FCstore);
	return NULL;
      }

      /* Bring down 1st element of FCstore */
      if (storedFCflg == false) {
	FCp1 = strchr(FCstore+1, ITFCSTART);
	if (FCp1 != NULL) { /* More than 1 format code left */
	  strncpy(tmpstr, FCstore, FCp1-FCstore);
	  tmpstr[FCp1-FCstore] = '\0';
	  strcpy(FCstore, FCp1);
	  FCp1 = FCstore;
	}
	else {
	  strcpy(tmpstr, FCstore);
	  FCstore[0] = '\0';
	  FCp1 = NULL;
	}
	storedFCflg = true;
      }
      /* FCp may not now point to current FC which is in tmpstr */
      storep = (storedFCflg == false) ? FCstore : dummyFCstore;
      FCp = isFC(tmpstr, storep, FC);
    }
    else { /* Invalid or undefined format code */
      IT_getword (infile, inbuffer, Wbuffer, FCbuffer,
	 FCparbuffer, FC, EndOfText);
      return (Wbuffer[0] != '\0' || FCbuffer[0] != '\0') ? Wbuffer : NULL;
    }
  }
  else if (FCfound) /* Format code already in store */
    FCp = NULL;

  if (feof(infile) && !bufleft && storedFCflg == false) {
     /* End-of-file & end-of-inbuffer & no more in FCstore */
	 *EndOfText = true;
	 bufptr = NULL;
	 inbuffer[0] = '\0';
  }

  /* Toggle noFCallowed */
  if (_stricmp(tmpstr, ITFCstart"asc") == 0) {
    noFCallowed = !noFCallowed;
    FCfound = false;
    tmpstr[0] = '\0';
  }

  /* Strip off the format code params if any */
  if (FCfound) {
    if (FCp != NULL
    && islower(FCp->in[strlen(FCp->in) - 1])
    ) {
      if (IT_getFCparams(tmpstr, FCparbuffer, FCbuffer, FCp) == NULL) {
	return NULL;
      }
    }
    else { /* param-less code; copy direct to FCbuffer */
      i = 0;
      while (tmpstr[i] != '\0' && tmpstr[i] == FCp->in[i]) { /* FC core */
	FCbuffer[i] = tmpstr[i];
	++i;
      }
      FCbuffer[i] = '\0';
      if (tmpstr[i] != '\0') /* Treat residue of tmpstr as a word */
	strcpy(Wstore, &tmpstr[i]);
    }
  }
  else {
    strcpy(Wbuffer, tmpstr);
    FCbuffer[0] = '\0';
  }

  return Wbuffer;
}

/* nxtword - Copies current word from buffer to local static, & moves
   buffer pointer to point after it. bufptr is assumed to be at the
   beginning of a word.
   Returned: pointer to isolated word or format code
*/
char *nxtword (char **bufptr, int *bufleft)
{
  static char *wstr;
  char *str, *p, *p0;
  size_t l;
  char modulename[] = "IT_getword.nxtword";

  p0 = *bufptr;
  /* Make sure we point to non-white space */
  if ((l = strspn(*bufptr, "\x20\t\r\n")) > 0)
    *bufptr += l;

  l = strcspn(*bufptr, "\x20\t\r\n\x1a"); /* Length of word bounded
					on the right by white space */

  /* Allocate temp space */
  if ((str = (char *) malloc(l+1)) == NULL) {
	 IT_error(modulename, "Word too large to fit into memory!: word length = %d", l);
    return NULL;
  }
  strncpy(str, *bufptr, l);
  str[l] = '\0';

  /* Is there a format code embedded in string? */
  if ((p = strchr(str+1, ITFCSTART)) != NULL)
    l -= strlen(p);
  else if (*str == ITFCSTART
  && (p=strstr(str, "--")) != NULL) /* leading format code followed by dash */ /*test091294*/
    l -= strlen(p); /*test091294*/

  /* Allocate wstr */
  if ((wstr = (char *) realloc(wstr, l+1)) == NULL) {
	 IT_error(modulename, "Word too large to fit into memory!: word length = %d", l);
    return NULL;
  }

  strncpy(wstr, *bufptr, l);
  wstr[l] = '\0';

  /* Move buffer pointer to 1st non_white_space char after current word */
  *bufptr += l;
  l = strspn(*bufptr, "\x20\t\n\r\x1a");
  *bufptr += l;

  /* Set bufleft */
  *bufleft -= (*bufptr-p0);
  *bufleft = max((*bufleft), 0); /* This ensures no wrap-around,
       but is hopefully never executed! */

  free(str);
  return wstr;
}

char *IT_readin (FILE *infile, char *inbuffer, char *bufptr, int *bufleft)
{
  size_t l;
  char modulename[] = "IT_getword.IT_readin";

	if (bufptr == NULL || *bufleft <= 0)
    bufptr = inbuffer;

  /* Copy any residue back to beginning of buffer */
  l = strlen(bufptr);
  if ((size_t)(bufptr-inbuffer) < strlen(inbuffer))
    strcpy(inbuffer, bufptr);
  bufptr = inbuffer + l;
  if ((*bufleft += fread(bufptr, 1, BUFFULL - 1 - l, infile)) <= 0) {
	 IT_error(modulename, "Cannot read translated file beyond position %ld",
	ftell(infile));
	 return bufptr = NULL;
  }
  inbuffer[*bufleft] = '\0';
  bufptr = inbuffer;
  return bufptr;
}

/* ITS - IT_mkfiller: makes filler string */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*27/01/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      27/01/92 */
/* (Module AGEDHBN) */
/* Returned: Pointer to filled line on success, else null */
/* Note: 1. The line sent to this function may contain data - filler is
	    appended.
	 2. The calling module is responsible for ensuring that there is
	    enough room in the line buffer for the fill string.
*/


char *IT_mkfiller(char filler, size_t n, char *line, size_t *lpos, size_t llength
#if !defined(BRAILLE_IT)
   , int *cwt
#endif
)
{
  size_t l;
  int i;

  if (*lpos + cw(filler) > llength || n == 0)
    return NULL;

  i = __min((int)n, (int)((llength - *lpos) / cw(filler)));
  if (i < 0)
    return NULL;
  l = strlen(line);
  memset(line + l, filler, i);
  line[l+i] = '\0';
  *lpos += i;
  return line;
}

/* ITS - mod Control header creation & page output */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.5*18/08/94*Amend: header_type 2, 1.4 was incomplete.         *30414*NO */
/***************************************************************************/
/*1.4*02/12/93*Amend: with header_type 2, no. was appended even  *3040?*NO */
/* . *  /  /  *where module knew there was no room (1 line moved)*     *   */
/*   *  /  /  *addnumtoline() added.                             *     *   */
/***************************************************************************/
/*1.3*30/09/92*Amend: with header_type 2, no. was at col. <l>-1  *30301*NO */
/***************************************************************************/
/*1.2*22/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*20/03/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      20/03/92 */
/* (Module AGEDHN) */
/* Returned: 0 on success, else nonzero */


extern boolean abrt;

int IT_mkpage (struct txtline line[], char *header, char *printpagestr,
   int paginat_ind, int header_ind, flag no_pg_breaks,
   size_t *pagecount, size_t *linecount, size_t *storecount, size_t linelength,
   FILE *dest)
{
  size_t l;
  char modulename[] = "IT_mkpage";

  if (header_ind != 1 && header_ind != 2 && header != NULL
  && mk_pagehdr(header, printpagestr, *pagecount,
     paginat_ind) != 0)
    return !0;
  else if (header_ind == 2 && no_pg_breaks == false) {
    /* Page-number tacked to line[0] here */
    if (addnumtoline(line[0].text, linelength, *pagecount, paginat_ind) != 0)
      return !0;
  }

  if (writepg(line, header, *linecount, *pagecount, no_pg_breaks, dest) != 0)
    return !0;

  /* Clear used part of line[] & copy the rest to the beginning */
  l = *linecount;
  while (*linecount < *storecount) {
    line[*linecount-l] = line[*linecount];
    ++*linecount;
  }

  *storecount = *linecount - l;
  *linecount = 0;
/* COMMENT OUT FOR NOW *********
    IT_print("%d\b", *pagecount);
    for (l=1; l<strlen(itoa(*pagecount, NULL, 10)); ++l)
      IT_print("\b");
  }									  ************* */
  if (no_pg_breaks == false)
    ++*pagecount;

#ifndef KevTest
  /* User abort? */
    if (_kbhit() && _getch() == ESCKEY)
	{	/* Clear keyboard buffer */
        while (_kbhit())
            _getch();
		raise(SIGINT);
	}
#endif

  if (abrt == TRUE && Confirm("Abort Formatting !", FORMBOXROWPOS + 2, 25))
  {	abrt = FALSE;
	  return(!0);
  }
  abrt = FALSE;
  return 0;
}


/***************************************************************************/
/* New module added to refine header-type 2 page-numbering:
  (a) to make pagenumber field variable instead of fixed;
  (b) to allow roman pagination
*/
static int addnumtoline(char *line, size_t linelength,
  size_t pagecount, int paginat_ind)
{
  size_t l, lbp;
  char *bp;
  char modulename[] = "addnumtoline";

    l = strlen(line);
    memset(line+l, space, linelength-l);/*test180894*/
    if (paginat_ind == ARABIC) {
      if ((bp = IToBrl(pagecount)) == NULL) {
        IT_error(modulename, "Error attaching page-number to top line of text: page %d",
            pagecount);
        return !0;
      }
    }
    else {
      if ((bp = IToBRom(pagecount)) == NULL) {
        IT_error(modulename, "Error attaching page-number to top line of text: page %d",
            pagecount);
        return !0;
      }
    }
    lbp = strlen(bp);

    if (l > linelength - lbp - 2)
#if !defined(BRAILLE_IT)
      IT_report("Page-number cannot be attached: page %d\n%s", pagecount,
          line);
#else
      ;
#endif
    else/*test180894*/
        strcpy(line + linelength - lbp, bp);/*test180894*/

    return 0;
}
/* ITS - Converts integer to braille arabic.
   Number is first converted to a string; then to a braille numberstring
   with a separate function IStrToBrl
*/
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*20/03/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      20/03/92 */
/* (Module AGEDHNBD) */
/* Returned: Pointer to static-stored brl. number */


char *IToBrl (int num)
{
/* Converts an int to a braille number *
 * 5/11/91		N. Octon
*/

  char numstr[7] = { '\0' };

  return IStrToBrl(_itoa(num, numstr, 10));
}

#define BRLMINUS ";-"
#define NUMERALSIGN '#'

char *IStrToBrl (char *numstr)
{
/* Converts an numeric string to a braille number *
 * 12/11/91		N. Octon
*/

  int i, j;
  static char bnum[9];

  i = j = 0;
  if (numstr[i] == '-') {
    strcpy(bnum, BRLMINUS);
    i += 2;
    ++j;
  }
  bnum[i++] = NUMERALSIGN;
  for (i=i, j=j; numstr[j] != '\0'; ++i, ++j)
    bnum[i] = (char) ((numstr[j] != '0')
       ? numstr[j] + 0x10
       : numstr[j] + 0x1A);
       /* Braille 0 = 'J' */
  bnum[i] = '\0';
  return bnum;
}

/* Convert integer to braille roman numeral
  N. Octon	1/10/91
  (Module lAGEDHNBD)
  Returned: Pointer to braille roman string on success, else null
*/
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*01/10/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
#define LOWERCASE 0
#define UPPERCASE 1
#define MAXROMAN 17


char *IToBRom (int n)
{
  static char broman[MAXROMAN+1], *p;

  strcpy(broman, ";");
  if ((p = IToRom(n, UPPERCASE)) != NULL) {
    strcpy(broman+1, p);
    return broman;
  }
  else
    return NULL;
}


/* Convert integer to roman number string */
/* N. Octon	20/11/91 */
/* (Module AGEDHNBB) */
/* Returned: Pointer to roman string on success, else null */
#define MAXROMAN 17
#define LOWERCASE 0
#define UPPERCASE 1

char *IToRom(int n, int mode)
{
  static char roman[MAXROMAN];
  char four;
  int i, j, k;

  i = 0;
  if ((j = n / 1000) > 0) {
    k = j;
    while (k--)
      roman[i++] = 'm';
    n -= (j * 1000);
  }
  if (n + 100 >= 1000) {
    roman[i++] = 'c';
    roman[i++] = 'm';
    n -= 900;
  }
  else if (n >= 500) {
    roman[i++] = 'd';
    n -= 500;
  }
  if ((j = n / 100) > 0) {
    four = 'd';
    k = (j == 4) ? 1 : j;
    while (k--)
      roman[i++] = 'c';
    if (j == 4)
      roman[i++] = four;
    n -= (j * 100);
  }
  if (n + 10 >= 100) {
    roman[i++] = 'x';
    roman[i++] = 'c';
    n -= 90;
  }
  else if (n >= 50) {
    roman[i++] = 'l';
    n -= 50;
  }
  if ((j = n / 10) > 0) {
    four = 'l';
    k = (j == 4) ? 1 : j;
    while (k--)
      roman[i++] = 'x';
    if (j == 4)
      roman[i++] = four;
    n -= (j * 10);
  }
  if (n == 9) {
    roman[i++] = 'i';
    roman[i++] = 'x';
    n = 0;
  }
  else if (n >= 5) {
    roman[i++] = 'v';
    n -= 5;
  }
  else if (n == 4) {
    roman[i++] = 'i';
    roman[i++] = 'v';
    n = 0;
  }
  while (n--)
    roman[i++] = 'i';

  roman[i] = '\0';

  if (i == 0)
    return NULL;
  return mode == LOWERCASE ? roman : _strupr(roman);
}

/* ITS */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*28/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* Get brl format codes (Module AGEDD) */
/* N. Octon	28/11/91 */
/* Return: pointer to table on success, null otherwise. */

char nullstr[1]; /* Null string */


struct IT_FCode **IT_GetBFormatCodes (char *FCfname, struct IT_FCode *BFC[],
   size_t linelen, size_t pagelen)
{
  FILE *f;
  char s[80];
  char temps1[10], temps2[30];
  char allocerr[60] = "Memory allocation error: record %d";
  char recorderr[60] = "Record %d field %d incorrect";
  char *p;
  size_t i, j, l1, l2;
  char modulename[] = "IT_GetBFormatCodes";

  /* Open file */
  if ((f = ITS_fopen(FCfname, "r")) == NULL
  && errno != 0) {
    return NULL;
  }

  /* Read file header */
  do {
    if (fgets(s, 80, f) == NULL) {
      return NULL;
    }
    else if (feof(f)) {
		IT_error(modulename, "Incorrect format code file: %s", FCfname);
      return NULL;
    }
  } while (s[0] != '\n'); /* File read in text mode for this condition */

  /* Allocate space for array of struct pointers */
  if ((BFC = (struct IT_FCode **) malloc(MAXFC+1)) == NULL) {
    return NULL;
  }

  /* Read format codes, 1 definition per file line */
  i = 1;
  do {
    if (fgets(s, 80, f) == NULL) {
		IT_error(modulename, "Record %d of format code file not read",
	  i);
      return NULL;
    }
    if ((BFC[i-1] = (struct IT_FCode *) malloc(sizeof(struct IT_FCode))) == NULL) {
		IT_error(modulename, allocerr, i);
      return NULL;
    }
    p = s;
    if ((j = sscanf(p, "%s %s %d %d %c %d", temps1, temps2,
	&BFC[i-1]->r1, &BFC[i-1]->r2, &BFC[i-1]->reduceto,
	&BFC[i-1]->dflt)) < 2) { /* No in field or no out field */
		IT_error(modulename, recorderr, i, j);
      return NULL;
    }
    else if (j < MAXFCFIELDS) { /* Initialise fields that were not present */
      if (j < 3)
	BFC[i-1]->r1 = 0;
      if (j < 4)
	BFC[i-1]->r2 = 0;
      if (j < 5)
	BFC[i-1]->reduceto = '\0';
      /* j == 5 */
      if (j == 5) {
      	if (BFC[i-1]->reduceto >= '0'
      	&& BFC[i-1]->reduceto <= '9') { /* reduceto not filled in but dflt present */
          BFC[i-1]->dflt = BFC[i-1]->reduceto - '0';
          BFC[i-1]->reduceto = '\0';
        }
        else
          BFC[i-1]->dflt = 0;
      }
    }
    /* Find the lengths of the in and out fields for memory allocation */
    if ((l1 = strlen(temps1)) == 0
    || (l2 = strlen(temps2)) == 0) {
		IT_error(modulename, recorderr, i, min(l1, l2));
      return NULL;
    }

    /* Allocate in and out fields - ITBFSTART char not stored */
    if ((BFC[i-1]->in = (char *) malloc(l1+1)) == NULL
    || (BFC[i-1]->out = (char *) malloc(l2+1)) == NULL) {
		IT_error(modulename, allocerr, i);
      return NULL;
    }

    /* Copy temp strings to allocated storage */
    strcpy(BFC[i-1]->in, temps1);
    strcpy(BFC[i-1]->out, temps2);

    /* Validate reduceto field & maybe reduce r2 */
    BFC[i-1]->reduceto = (char) tolower(BFC[i-1]->reduceto);
       /* Flip to lower case */
    if (BFC[i-1]->reduceto == 'm')
      BFC[i-1]->r2 = 0x7fff;
    else if (BFC[i-1]->reduceto == 'p')
      BFC[i-1]->r2 = (int) __min(BFC[i-1]->r2, (int) pagelen);
    else if (BFC[i-1]->reduceto == 'l')
      BFC[i-1]->r2 = __min(BFC[i-1]->r2, (int) linelen);

    /* Ensure dflt lies inside r1 & r2 */
    if (BFC[i-1]->dflt < BFC[i-1]->r1 && BFC[i-1]->dflt != 0)
      BFC[i-1]->dflt = BFC[i-1]->r1;
    else if (BFC[i-1]->dflt > BFC[i-1]->r2)
      BFC[i-1]->dflt = BFC[i-1]->r2;

    ++i;

    /* Check for eof character */
    if ((j = fgetc(f)) != EOFCHAR)
      ungetc(j, f);
  } while (!feof(f));

  /* BFC[i-1]->in set to null to denote end of array */
  BFC[i-1] = NULL;

  /* Close file */
  if (ITS_fclose(f) != 0) {
	 /*IT_error(modulename, nullstr);*/
    return NULL;
  }

  return BFC;
}

/* ITS - Read Character Width Table IT_readcwt (module AGEDBB) */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*15/07/93*Rewrite to accept CW file of the form m:n...\n    *30406*NO */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*21/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon	21/11/91 */
/* Return: pointer to int cwt[] on sucess, null otherwise */

#if !defined(BRAILLE_IT)
#define FULLCHARSET 256

int *IT_readcwt (char *cwtfname, int *cwt, int *min_non0width, int *max_non0width)
{
  size_t sizeofcwt, i, j;
  FILE *f;
  char s[81];
  char modulename[] = "IT_readcwt";

  /* Open CW table file */
  if ((f = ITS_fopen(cwtfname, "r")) == NULL)
    return NULL;

  /* Allocate memory for table */
  sizeofcwt = sizeof(*cwt) * FULLCHARSET;
  if ((cwt = (int *)calloc(FULLCHARSET, sizeof(*cwt))) == NULL) {
	 IT_error(modulename, "Cannot allocate memory for character width table");
    return NULL;
  }

  *min_non0width = *max_non0width = 0;

  /* Now read table - always (NumberOfChars * sizeof(CWTarg) */
  while (fgets(s, 80, f) != NULL) {
    if (sscanf(s, "%d%*c%d", &i, &j) < 2) {
      IT_error(modulename, "Incorrect character width table file: %s", cwtfname);
      ITS_fclose(f);
      return NULL;
    }
    cwt[i] = j;
    /* Assign min- and max-nonzero widths if appropriate */
    if (j > 0) {
      if (*min_non0width == 0 || *min_non0width > (int)j)
        *min_non0width = j;
      if (*max_non0width == 0 || *max_non0width < (int)j)
        *max_non0width = j;
    }
  }

  ITS_fclose(f);
  return cwt;
}
#undef FULLCHARSET
#endif /* BRAILLE_IT */
/* ITS - IT_report - Focus all report messages here to send them:-
	(a) to a report file if the report system param is set, or
	(b) to IT_error, which sends them to the screen using screen output functions.
*/
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*21/01/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon	21/1/92 */
/* Return: none. */


#if !defined(BRAILLE_IT)
void _cdecl IT_report (char *fmt, ...)
{
  char stars[] = "***";
  char *IT_reportstr, *syserrstr;
  va_list marker;
  static e;

  /*Re-init. if fmt = null */
  if (!fmt) {
    e = 0;
    return;
  }

  if ((IT_reportstr = (char *)malloc(256)) == NULL) {
	 ITS_Error("Memory allocation error", "IT_report", -1, -1);
    return;
  }

  memset(IT_reportstr, '\0', 255);
  va_start(marker, fmt);
  strcpy(IT_reportstr, stars);
  sprintf(IT_reportstr+strlen(IT_reportstr), "%5d  ", ++e);
  vsprintf(IT_reportstr + strlen(IT_reportstr), fmt, marker);
  va_end(marker);
  strcat(IT_reportstr, " ");
  strcat(IT_reportstr, stars);

  /* If system error, make system error message string */
  if (errno != 0) {
    syserrstr = _strerror(syserrstr);
    strcpy(IT_reportstr, syserrstr);
  }

  /* Call CScape function here */
#if defined(CSCAPEVER)
  /* Inhibit for present: pop_Message(IT_reportstr, 21, 1, 1, 76, 0x4F, bd_box);*test*/
#endif /* CSCAPEVER */

  free(IT_reportstr);
  return;
}
#endif /* BRAILLE_IT */
/* ITS - Set braille emboss format page number */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*09/01/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      09/01/92 */
/* (Module AGEDHBJ) */
/* Returned: New value on success, else -1 */


int IT_setpageno(int pnvalue, unsigned *pnbuf, size_t maxlen)
{
  if ((size_t)pnvalue > maxlen || pnvalue < 0)
    return -1;
  *pnbuf = pnvalue;
  return *pnbuf;
}

/* ITS - IT_set_ppstr */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*03/06/92*Correct the philos. to accept brl nos., not ASCII *30301*NO */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.			 *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*10/03/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      10/03/92 */
/* (Module AGEDHBL) */
/* Returned: pointer to print page number string on success, else null */

#define BRLNUMERALSIGN '#'
#define BRLLETTERSIGN ';'
char *IT_set_ppstr(char *ppstr, char *buf, size_t maxlen)
{
  char *p;
  char modulename[] = "IT_set_ppstr";

  p = buf;
  if (ppstr[0]) {
    if (ppstr[0] != BRLNUMERALSIGN && ppstr[0] != BRLLETTERSIGN) {
      *p++ = BRLLETTERSIGN;
      --maxlen;
    }
    strncpy(p, ppstr, maxlen);
    ppstr[maxlen] = '\0';
  }
  else
    *p = '\0';

#if !defined(BRAILLE_IT)
  if (strlen(ppstr) >= MAXLINELEN/4)
    IT_report("Print page number will be truncated: %s", buf);
#endif

  return buf;
}
#undef BRLNUMERALSIGN
#undef BRLLETTERSIGN

/* ITS - IT_splitbrlword Attempts to split a string according to braille
   splitting rules. */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.3*23/11/94*call to retranslate() now in else under issplitbar*30414*NO */
/***************************************************************************/
/*2.2*15/09/94*Move subfunctions to top-of-file & make static.   *30414*NO */
/* . *  /  /  *New retranslate() deals with prefixes/single-cell.*     *   */
/* . *  /  /  *New strcvt() inserts translated prefix/single cell*     *   */
/* . *  /  /  *issplitbarab() amended for "20-a-day" etc.        *     *   */
/***************************************************************************/
/*2.1*05/04/94*issplitbarab() corrected to include hash.         *30412*NO */
/***************************************************************************/
/*2.0*08/03/94*issplitbarab() added; split b_arab & "2-e" cured. *30412*NO */
/***************************************************************************/
/*1.9*02/02/94*Ignore leading lower signs.                       *30409*NO */
/***************************************************************************/
/*1.8*31/01/94*Amend: count not always considered & word bound   *30409*NO */
/* . *  /  /  *exceeded                                          *     *   */
/***************************************************************************/
/*1.7*26/01/94*Increase max. hyphens expected from 4-10.         *30409*NO */
/***************************************************************************/
/*1.6*20/10/93*Don't put numeral sign in e.g. 4-door             *3040?*NO */
/***************************************************************************/
/*1.5*30/09/92*Don't split before colon/dash.                    *30301*NO */
/***************************************************************************/
/*1.4*22/07/92*Fix bug whereby "COM" prefixed word could be		 *30301*NO */
/* . *	/  /  *split at "COM" instead of at hyphen			   *	  *	   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.			 *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*15/01/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      15/01/92 */
/* (Module AGEDHBP */
/* Returned: 1st part of split string on success, null on failure or on error. */
/* Notes: 1. The calling function is responsible for providing space to
     contain 2nd part of word.
   2. The string is assumed to contain no embedded spaces.
*/

static flag __near islowersigns (char *str, size_t nchars)
/* Determines whether nchars characters of str are all braille lower signs */
/* N. Octon	16/01/92 */
/* Return: true if all specified characters are lower signs */
{
  char lowersigns[] = "\"',-1234567890;."; /* "." brl italic is ignored
					       when applying the lower
					       sign rule */
  char *s = NULL;
  size_t l;
  flag retflg;
  char modulename[] = "IT_splitbrlword.islowersigns";

  if (nchars == 0 || (l = strlen(str)) == 0 || nchars > l)
    return false;

  /* Make private copy of str */
  if ((s = _strdup(str)) == NULL) {
	 IT_error(modulename, "islowersigns: memory allocation error");
    return false;
  }

  s[nchars] = '\0'; /* Exclude anything after that point */
  if (strspn(s, lowersigns) == nchars)
    retflg = true;
  else
    retflg = false;
  free(s);
  return retflg;
}

static flag __near issplitbarab(char *str, char *hyphen_p)
/* does str contain a hyphenated braille arabic number? */
/* N. Octon 08-Mar-1994 */
{
  char barabcharset[] = "ABCDEFGHIJabcdefghij#1'";
  char nonbarab[] = "KLMNOPQRSTUVWXYZklmnopqrstuvwxyz*<%?:$]\\[>@^\"_";
    /* Brl punct. not here because it is allowable after split no. */
  char *barabp, *p;
  size_t l, l2;
  int ret;

  ret = false;
  l = strspn(hyphen_p+1, barabcharset);
  l2 = strspn(hyphen_p+1+l, nonbarab);
  if ((barabp = strchr(str, '#')) != NULL && barabp < hyphen_p - 1
     /* arabic number having at least 1 digit */
  && l > 0 /* arabic char(s) after hyphen */
  && l2 == 0 /* No non-arabic char(s) immediately following */
  ) /* Hyphenated brl arabic number */
    ret = true; /* probably a hyphenated arabic .. But check beyond next hyphen */

  p = strchr(hyphen_p+1, '-');
  while(p && *(p-1) != '-') { /* That would be a dash which is irrelevant! */
    l = strspn(p+1, barabcharset);
    l2 = strspn(p+1+l, nonbarab);
    if (l == 0 /* no arabic char(s) after hyphen */
    &&  l2 != 0 ) /* non-arabic char(s) immediately following */
      ret = false;
    p = strchr(p+1, '-');
  }
  return ret;
}

static void __near strcvt(char *dest, char *src, int dif)
{
    memcpy(dest, src, strlen(src));
    memmove(dest+strlen(src), dest+dif+1, strlen(dest+dif+1)+1);
    return;
}

static void __near retranslate(char *str)
{
    static char *prefixes[] = {"-com", "4dis", "2be", "3con", "9in",
      "0WAS", "7WERE", NULL};
        /* the 1st byte of each prefix is the braille equivalent of
           the remaining bytes
        */
    char *prefix_p, *dest_p, *dest_p2;
    char prefix[2];
    flag retrans = false;
    size_t i = 0;

    /* Is retranslation needed? */
    prefix_p = *prefixes;
    dest_p = str + strspn(str, "78.");
    while (prefix_p) {
        if (_strnicmp(dest_p, prefix_p+1, strlen(prefix_p+1)) == 0) {
            dest_p2 = dest_p+strlen(prefix_p+1);
            if (_stricmp(prefix_p+1, "BE") != 0
            && *dest_p2 /* there is something after the prefix */
            && !islowersigns(dest_p2, strlen(dest_p2)) ) /* & not all lower signs */
                retrans = true;
            if (_stricmp(prefix_p+1, "BE") == 0
            && !islowersigns(dest_p2, 1) )     /* "BE" is an exception */
                retrans = true;
            if ((_stricmp(prefix_p+1, "WAS") == 0
            ||   _stricmp(prefix_p+1, "WERE") == 0)
            &&  !*dest_p2 ) /* Single-cell words; 2BE" covered in prefixes */
                retrans = true;
            break;
        }
        prefix_p = prefixes[++i];
    }
    if (retrans) {
        strncpy(prefix, prefix_p, 1);
        prefix[1] = '\0';
        strcvt(dest_p, prefix, abs((strlen(prefix_p+1) - strlen(prefix))));
    }

    return;
}

#define MAXHYPHENS 10

char *IT_splitbrlword (char *word, char *w2buf, size_t count)
{
  char *hyp[MAXHYPHENS+1] = {NULL}, *p0, *p, **pp;
  size_t l;
  char seqstr[] = "&=(!)Aa";
  char l_brlpunct[] = "78,"; /* Leading brl punctuation */
  char lowersigns[] = "\"',-1234567890;."; /* "." brl italic is ignored
					       when applying the lower
                           sign rule */
  char modulename[] = "IT_splitbrlword";
  char brl_colon = '3';
  flag barabflag;

  p = NULL;
  /* Early byebye */
  if (strlen(word) <= 1 || count </*test120994=*/ 1)
    return NULL;
  if (islowersigns(word, strlen(word)))
    return NULL;

  /* Try splitting midway through a AND-FOR-OF-THE-WITH-A sequence */
  if ((p0 = strpbrk(word, seqstr)) != NULL /* Some sequenced signs present */
  && (l = strspn(p0, seqstr)) == strlen(p0)	/* & that segment = sequenced signs*/
  && strspn(word, l_brlpunct) == (size_t)(p0-word) /* & initial segment = brl punct */
  && (size_t)(p0-word) < count) {	/* At least 1 sequenced char fits */
     /* Ignoring leading punctuation, word is entirely = seqstr & not
     beyond the scope of count */
    p0 = word + count - 1; /* Last allowable char on this line */
    strcpy(w2buf, p0+1);
    *(p0+1) = '\0';
    return word;
  }

  /* Point to hyphen/short dash */
  pp = hyp;
  if ((*pp = strchr(word + strspn(word, lowersigns), '-')) != NULL) {
    while (pp-hyp < MAXHYPHENS && *pp != NULL) {
      ++pp;
      *pp = strchr(*(pp-1)+1, '-');
    }
  }

  if (hyp[0] <= word && hyp[1] == NULL)
    return NULL;
  if (hyp[0] - word > (int)count)
    return NULL;
  /* Exit if 4 hyphens are consecutive starting at word */
  if (hyp[0] == word
  && hyp[0]+1 == hyp[1] && hyp[1]+1 == hyp[2] && hyp[2]+1 == hyp[3])
    return NULL;

  /* Don't split
     (1) if pre-hyphen/dash string is all lower signs, or
     (2) if post-hyphen/dash string is punctuation, or
     (3) if post-hyphen/dash string-length > 1 && is all lower signs, or
     (4) before colon/dash.
     This algorithm is incomplete, since the italic sign counts as a lower sign.
  */

  /* Find current hyphen cluster within the scope of <count> */
  pp = &hyp[MAXHYPHENS-1];
  while ((!*pp && pp > hyp)
  || (pp > hyp && *pp - word >= (int)count)
  || (pp < hyp+MAXHYPHENS-2 && pp > hyp && *(*pp+1) == '-'))
      /* null ptr; (hyphen beyond scope or in cluster
            beyond scope) */
    --pp;
  p = p0 = *pp;

  /* hyphen, as opposed to dash/long dash */
  if (*(p-1) == '-') { /* Find leftmost hyphen of the current group of hyphens */
    while (*p == '-')
      --p;
    ++p;
  }

  /* Now try splitting after hyphen */
  if (p == p0 /* hyphen */
  && *(p0+1) != '-') { /* dash spanning line boundary not allowed */
    if (islowersigns(word, p-word) /* lower sign(s) on left of hyphen */
    || p-word >= (int) count
    || ((l = strlen(p+1)) > 1 && islowersigns(p+1, l))
	/* Multiple lower signs on right of hyphen */
    || (l == 1 && islowersigns(p+1, l)
	/* 1 lower sign on right of hyphen */
    && *(p+1) != '9' && *(p+1) != '5')
    )
      return NULL;
  }
  else { /* dash/long dash */
    if ((l = strlen(p0+1)) > 1 && islowersigns(p0+1, l))
	/* Multiple lower signs on right of dash */
      return NULL;
    else if (((l = p-word) > 1 && islowersigns(word, l))
	/* Multiple lower signs before dash */
    || (islowersigns(word, l) && l == 1 && *(p-1) != '9' && *(p-1) != '5')
	/* No allowable single lower signs before dash */
    || (l && *(p-1) == brl_colon)
    )
      return NULL;
    if (l == 1 && *(p-1) == '9' || *(p-1) == '5')
      p0 = p - 1; /* Ensure split before dash */
    if (l + 1 >= count)
      p0 = p - 1; /* Ensure split before dash */
  }

  /* 2nd part of word, in isolation, may not be correct:
     1. if it has a prefix, it might now need to be contracted;
     2. if it is a number it must have a numeral sign
  */
  barabflag = false;
  if (p == p0 /* Hyphen */
  && (barabflag = issplitbarab(word, p0))
  && *(p0+1) != '#'
  ) { /* Braille arabic numberstring */
    strcpy(w2buf, "#");
    ++w2buf;
  }
  else
  if (!barabflag
  && getgrade() == 2
  &&  !islowersigns(p0+1+strspn(p0+1, "-.,78"), 1))
    retranslate(p0+1);

  /* Copy the 2 parts of the split word */
  strcpy(w2buf, p0+1);
  *(p0+1) = '\0';

  return word;
}
#undef MAXHYPHENS
/* ITS - Write page to file */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*11/06/93*Flush output buffer.                              *30105*NO */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*20/03/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      20/03/92 */
/* (Module AGEDHNF) */
/* Returned: 0 on success, else !0 */


int writepg(struct txtline line[], char *header, size_t linecount,
   size_t pagecount, flag no_pg_breaks, FILE *dest)
{
  size_t l;
  char modulename[] = "writepg";

  if (header != NULL
  && (fputs(header, dest) == EOF
  || putc('\n', dest) == EOF))
  {
	 IT_error(modulename, "Error writing page header to formatted file: page %d", pagecount);
    return !0;
  }

  for (l=0; l<linecount; ++l) {
    if ((fputs(line[l].text, dest) == EOF
    && strlen(line[l].text) > 0)
    || putc('\n', dest) == EOF)
    {
		IT_error(modulename, "Error writing to formatted file: page %d", pagecount);
      return !0;
    }
  }
  if (no_pg_breaks == false
  && putc('\f', dest) == EOF) {
	 IT_error(modulename, "Error writing to formatted file: page %d", pagecount);
    return !0;
  }

  fflush(dest);

  return 0;
}
/* ITS - IT_storeline - stores line in linestore */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*19/02/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* N. Octon      19/02/92 */
/* (Module AGEDHD) */
/* Returned: pointer to stored line on success, else null */
/* Note: linecount is always incremented after a line is stored;
   storecount is conditionally incremented for a non-block non-header line
*/


struct txtline *IT_storeline (struct txtline line[], struct txtline *linep,
   size_t *linecount, size_t *storecount, unsigned long *abslinecount)
{
  char modulename[] = "IT_storeline";
  /* Check for linestore overflow */
  if (*storecount > MAXPAGELEN) {
	 IT_error(modulename, "Line store overflow .. Page too big");
    return NULL;
  }

  line[*storecount] = *linep;
  ++*storecount;
  ++*abslinecount;
  if (!(linep->attribs & BLOCK))
    *linecount = *storecount;
  return &line[*storecount];
}

/* TextLength - Finds length of specified string in character width units. */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.3*29/04/93*Function CalcCW added                             *30301*MJH*/
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*??/11/91*Creation Date.                                    *30301*KJM*/
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* ITSFRM.LIB                                                              */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* K. Metcalfe		11/91 */
/* Return: int length. */


#if !defined(BRAILLE_IT)
size_t cw (int c)
{
    /* For the moment, cwt & FONTSIZE are global rather than params */
    if (cwt[c] >= 50)
        return (size_t)(cwt[c] * FONTSIZE / 50); /* twips */
    else
        return (size_t)(cwt[c]); /* no change */

}

int TextLength(char *Text, int *cwt)
{
  int TempLength;
  int loop;
  int TempStrLen;

  for (loop = 0, TempStrLen = strlen(Text), TempLength = 0;
       loop != TempStrLen; loop++)
      TempLength += cw(Text[loop]);

  return(TempLength);
} /* End of TextLength */
#endif /* BRAILLE_IT */

#if !defined(BRAILLE_IT)
/* CalcCW returns the number (int) of char width units
   calculated from the point <s> for <n> characters
*/
size_t CalcCW(char *s, int n, int *cwt)
{
    size_t cwu;
    int i;

    cwu = 0;
    for (i=0; i<n; ++i)
        cwu += cw(s[i]);
    return cwu;
}
#endif

/* ITBEMBFM.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.5*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*1.4*08/03/93*TransForm option enabled in braille menu.	     *30301*MW */
/***************************************************************************/
/*1.3*29/05/92*Standardised function naming & calling convention.*30301*MJH*/
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*01/05/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* ITBEMBFM.OBJ                                                               */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

/* ------------------------ INCLUDE FILES --------------------------------- */
/* --------------------- END OF INCLUDE FILES ----------------------------- */


/* ------------------------------------------------------------------------ */
boolean braille_emb_format(char  *inflname, char  *outflname,
                       char  *fctabname,
#if !defined(BRAILLE_IT)
                       char  *cwtname,
#endif
					   char  *reportfilename,
					   size_t linelength, size_t pagelength,
					   size_t startpage, int header_ind,
						flag header_ref, flag reportflag,
                        boolean ScreenOn
#if !defined(BRAILLE_IT)
                        , boolean SysPause
#endif
)
{	char tempfctabname[FILENAMELENGTH];
#if !defined(BRAILLE_IT)
	char tempcwtname[FILENAMELENGTH];
#endif

/* Valid source name ? */
    if (_access(inflname, 00) != 0)
	{	sprintf(errmsg, "File %s does not exist", inflname);
		ITS_Error(errmsg, "braille_emb_format", -1, -1);
		errno = 0;
		return !0;
	}

/* Valid format code table name ? */
	strcpy(tempfctabname, fctabname);
	if (LocateFile(tempfctabname) != TRUE)
		return !0;

#if !defined(BRAILLE_IT)
/* Valid character width table name ? */
	strcpy(tempcwtname, cwtname);
	if (LocateFile(tempcwtname) != TRUE)
		return !0;
#endif

	return BRLEmbForm(inflname, outflname,
                       tempfctabname,
#if !defined(BRAILLE_IT)
                       tempcwtname,
#endif
					   reportfilename,
					   linelength, pagelength, startpage,
                  header_ind, header_ref,
                  reportflag, ScreenOn
#if !defined(BRAILLE_IT)
                  ,SysPause
#endif
            	);
}
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
boolean EmbTransFormParamConfirm(	char *source,
					char *temptrans,
					char *dest,
					struct FileInfo Source,
					struct FileInfo Trans,
					struct FileInfo Dest,
					int *DefGrade,
					size_t *PLength,
					size_t *PWidth,
					size_t *PNum,
					boolean *HeadRef,
					int *HeadType)
{
	int ret = FALSE;
	sed_type sed;
	char Error[81];

/* Load screen */
	if ((sed = sfile_LoadSed(lnffile, "embtransform_cnfrm", SED_ALLOC)) == NULL)
	{	ITS_Error(" Error trying to load confirmation screen ",
						 "EmbFormParamConfirm", -1, -1);
	}
	else
	{
/* Set file information structures (used within auxiliary) */
		SourceFinfo = Source;
		TransFinfo = Trans;
		DestFinfo = Dest;
/* Initialise each field on screen */
		strcpy(Error, "\0");
		strcpy((char *) sed_GetNameVar(sed, "Error"), Error);

		strcpy((char *) sed_GetNameVar(sed, "Source"), source);
		strcpy((char *) sed_GetNameVar(sed, "TransFile"), temptrans);
		strcpy((char *) sed_GetNameVar(sed, "Destination"), dest);
        *((int *) sed_GetNameVar(sed, "DefGrade")) = *DefGrade - 1;
		*((boolean *) sed_GetNameVar(sed, "HeadReflect")) = *HeadRef;
		*((size_t *) sed_GetNameVar(sed, "PageLength")) = *PLength;
		*((size_t *) sed_GetNameVar(sed, "PageWidth")) = *PWidth;
		*((size_t *) sed_GetNameVar(sed, "StartPage")) = *PNum;
		*((int *) sed_GetNameVar(sed, "HeadType")) = *HeadType - 1;


		sed_ProtectField(sed, sed_GetNameNo(sed, "Error"));

/* Set auxiliary and special functions */
		sed_SetAux(sed, aux_sourcetransdest);
		sed_SetSpecial(sed, spc_params);

		sed_Repaint(sed);
		if (sed_Go(sed) != SED_ABORT)
		{
/* Retrieve values from each field */
			strcpy(source, (char *) sed_GetNameVar(sed, "Source"));
			strcpy(temptrans, (char *) sed_GetNameVar(sed, "TransFile"));/*new*/
			strcpy(dest, (char *) sed_GetNameVar(sed, "Destination"));
            *DefGrade = *((int *) sed_GetNameVar(sed, "DefGrade")) + 1;
			*HeadRef = *((boolean *) sed_GetNameVar(sed, "HeadReflect"));
			*PLength = *((size_t *) sed_GetNameVar(sed, "PageLength"));
			*PWidth = *((size_t *) sed_GetNameVar(sed, "PageWidth"));
			*PNum = *((size_t *) sed_GetNameVar(sed, "StartPage"));
			*HeadType = *((int *) sed_GetNameVar(sed, "HeadType")) + 1;
			ret = TRUE;
		}

		sed_Close(sed);
	}
	return(ret);
}
#endif
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
#ifdef CSCAPEVER
boolean EmbFormParamConfirm(char *source, char *dest,
					struct FileInfo Source, struct FileInfo Dest,
					size_t *PLength, size_t *PWidth, size_t *PNum,
					boolean *HeadRef, int *HeadType)
{
	int ret = FALSE;
	sed_type sed;
	char Error[81];

/* Load screen */
	if ((sed = sfile_LoadSed(lnffile, "embform_cnfrm", SED_ALLOC)) == NULL)
	{	ITS_Error(" Error trying to load confirmation screen ",
						 "EmbFormParamConfirm", -1, -1);
	}
	else
	{
/* Set file information structures (used within auxiliary) */
		SourceFinfo = Source;
		DestFinfo = Dest;
/* Initialise each field on screen */
		strcpy(Error, "\0");
		strcpy((char *) sed_GetNameVar(sed, "Error"), Error);

		strcpy((char *) sed_GetNameVar(sed, "Source"), source);
		strcpy((char *) sed_GetNameVar(sed, "Destination"), dest);
		*((boolean *) sed_GetNameVar(sed, "HeadReflect")) = *HeadRef;
		*((size_t *) sed_GetNameVar(sed, "PageLength")) = *PLength;
		*((size_t *) sed_GetNameVar(sed, "PageWidth")) = *PWidth;
		*((size_t *) sed_GetNameVar(sed, "StartPage")) = *PNum;
		*((int *) sed_GetNameVar(sed, "HeadType")) = *HeadType - 1;


		sed_ProtectField(sed, sed_GetNameNo(sed, "Error"));

/* Set auxiliary and special functions */
		sed_SetAux(sed, aux_sourcedest);
		sed_SetSpecial(sed, spc_params);

		sed_Repaint(sed);
		if (sed_Go(sed) != SED_ABORT)
		{
/* Retrieve values from each field */
			strcpy(source, (char *) sed_GetNameVar(sed, "Source"));
			strcpy(dest, (char *) sed_GetNameVar(sed, "Destination"));
			*HeadRef = *((boolean *) sed_GetNameVar(sed, "HeadReflect"));
			*HeadType = *((int *) sed_GetNameVar(sed, "HeadType")) + 1;
			*PLength = *((size_t *) sed_GetNameVar(sed, "PageLength"));
			*PWidth = *((size_t *) sed_GetNameVar(sed, "PageWidth"));
			*PNum = *((size_t *) sed_GetNameVar(sed, "StartPage"));
			ret = TRUE;
		}

		sed_Close(sed);
	}
	return(ret);
}
#endif
/* ------------------------------------------------------------------------ */



/* ---------------------------- MODULE AGKA ------------------------------- */
#ifdef CSCAPEVER
boolean ConfirmEmbForm(char *tempsource, char *tempdest,
					struct FileInfo *Source, struct FileInfo *Dest,
					size_t *PLength, size_t *PWidth, size_t *PNum,
					int *HeadType, boolean *HeadRef)
{	boolean ParamOK = FALSE;
	struct FilePath temp = { "\0", "\0", "0", "\0" };

	errmsg[0] = '\0';
	tempsource[0] = '\0';
	tempdest[0] = '\0';

/* Split source name into drive, directory, name and extension */
	_splitpath(Source->Name, temp.drv, temp.dir, temp.name, temp.ext);

/* Any source name ? */
	if (temp.name[0] != '\0')
	{
		strcpy(tempsource, Source->Name);
/* Expand source name for default/current/specified directory etc. */
		ProduceFullName(tempsource, *Source);
/* Copy base name from source to destination */
		strcpy(tempdest, temp.name);
/* Expand destination name for default/current directory etc. */
		ProduceFullName(tempdest, *Dest);
	}

	while (!ParamOK)
	{	if (!EmbFormParamConfirm(tempsource, tempdest, *Source, *Dest,
								 PLength, PWidth, PNum, HeadRef, HeadType))
			return(FALSE);
		ParamOK = TRUE;
/* Store source and destination names as parameters */
		StoreFullName(tempsource, Source);
		StoreFullName(tempdest, Dest);
	}

	return(TRUE);
}
#endif
/* ------------------------- END OF MODULE AGKA --------------------------- */


/* ------------------------ NEW MODULE A??? ------------------------------- */
#if !defined(BRAILLE_IT)
boolean ConfirmEmbTransForm(	char *tempsource,
				char *temptrans,
				char *tempdest,
				struct FileInfo *Source,
				struct FileInfo *Trans,
				struct FileInfo *Dest,
				int *DefGrade,
				size_t *PLength,
				size_t *PWidth,
				size_t *PNum,
				int *HeadType,
				boolean *HeadRef)
{	boolean ParamOK = FALSE;
	struct FilePath temp = { "\0", "\0", "0", "\0" };

	errmsg[0] = '\0';
	tempsource[0] = '\0';
	temptrans[0] = '\0';
	tempdest[0] = '\0';

/* Split source name into drive, directory, name and extension */
	_splitpath(Source->Name, temp.drv, temp.dir, temp.name, temp.ext);

/* Any source name ? */
	if (temp.name[0] != '\0')
	{
		strcpy(tempsource, Source->Name);
/* Expand source name for default/current/specified directory etc. */
		ProduceFullName(tempsource, *Source);

/* Copy base name from source to translate  */
		strcpy(temptrans, temp.name);
/* Expand translate name for default/current directory etc. */
		ProduceFullName(temptrans, *Trans);

/* Copy base name from source to destination */
		strcpy(tempdest, temp.name);
/* Expand destination name for default/current directory etc. */
		ProduceFullName(tempdest, *Dest);
	}

#ifdef CSCAPEVER
	if (!EmbTransFormParamConfirm(	tempsource,
					temptrans,
					tempdest,
					*Source,
					*Trans,
					*Dest,
					DefGrade,
					PLength,
					PWidth,
					PNum,
					HeadRef,
					HeadType))
            #endif
			return(FALSE);

	/* Store source and destination names as parameters */
	StoreFullName(tempsource, Source);
	StoreFullName(temptrans, Trans);
	StoreFullName(tempdest, Dest);

	return(TRUE);
}
#endif
/* ------------------------- END OF NEW MODULE A??? ----------------------- */


/* ---------------------------- MODULE AEACA ------------------------------ */
boolean ConfirmVitalEmbForm(char *tempsource, char *tempdest,
                    struct FileInfo *Source, struct FileInfo *Dest
#if !defined(BRAILLE_IT)
                    ,size_t *PLength, size_t *PWidth, size_t *PNum,
                    int *HeadType, boolean *HeadRef
#endif
)
{	struct FilePath temp = { "\0", "\0", "0", "\0" };

	errmsg[0] = '\0';
	tempsource[0] = '\0';
	tempdest[0] = '\0';

/* Split source name into drive, directory, name and extension */
	_splitpath(Source->Name, temp.drv, temp.dir, temp.name, temp.ext);

/* Pre-format file names */
	trimstr(temp.name);

/* Any source name ? */
	if (temp.name[0] == '\0')
	{	ITS_Error(" No source file specified for formatting ! ",
					"ConfirmVitalEmbForm", -1, -1);
		return(FALSE);
	}

	strcpy(tempsource, Source->Name);
/* Expand source name for default/current/specified directory etc. */
	ProduceFullName(tempsource, *Source);
/* Copy base name from source to destination ? */
	if (Dest->Name[0] != '\0')
		strcpy(tempdest, Dest->Name);
	else
		strcpy(tempdest, temp.name);
/* Expand destination name for default/current directory etc. */
	ProduceFullName(tempdest, *Dest);

/* Store source and destination names as parameters */
	StoreFullName(tempsource, Source);
	StoreFullName(tempdest, Dest);

	return(TRUE);
}
/* ------------------------- END OF MODULE AEACA -------------------------- */


/* End of file ITBFM.C */

/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.0*29/11/94*$G & $ASC toggle grade & asc respectively.        *30414*NO */
/***************************************************************************/
/*1.9*23/04/93*Margin-set algor. now uses c_width for space.     *30404*NO */
/* . *  /  /  *CWU converts to 1/10" units.                      *     *   */
/* . *  /  /  *IT_setmarg() now needs param. cwt for width table.*     *   */
/***************************************************************************/
/*1.8*09/07/92*Accommodate forced overwrite of p_header with $HDO*30301*NO */
/***************************************************************************/
/*1.7*16/06/92*IT_tabjump: minimum of 2 leader chars allowed	 *30301*NO */
/***************************************************************************/
/* . *	/  /  *IT_bembline: lefthand margin ignored for 	 *     *   */
/* . *	/  /  *  centring, r-just.				 *     *   */
/***************************************************************************/
/*1.7*21/10/93*R-just. new format $RF            *3040?*NO */
/***************************************************************************/
/*1.6*09/06/92*R-just. bugfix                    *30301*NO */
/***************************************************************************/
/*1.5*03/06/92*Make newline format always cause return		 *30301*NO */
/***************************************************************************/
/*1.4*26/05/92*Redundant centre, r-just. format cancelled by	 *30301*NO */
/* . *	/  /  *a following newline format			 *     *   */
/***************************************************************************/
/*1.3*22/05/92*Move line-overflow condition from 3rd to 2nd else *30301*NO */
/***************************************************************************/
/* . *	/  /  *$B condition changed to exclude return		 *     *   */
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.			 *30301*MJH*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*03/02/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* BFFUNCS.OBJ                                                             */
/*																		*/
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* ITS - Composite source file containing the functions:
	IT_bembline
	mk_pagehdr
	IT_setmargin
	IT_settab
    IT_tabjump
*/
/* ITS - IT_bembline Builds brl emboss line */
/* N. Octon      03/02/92 */
/* (Module AGEDHB) */
/* Returned: pointer to struct txtline on success, null on error or reinit. */

#define MAXPAGEVAL 0x7FFF
#define addstrtoline(s) {            \
  for (i=0; s[i] != '\0'; p->lptr+=cw(s[i]), ++i)\
    line.text[p->lptr] = s[i];\
  svspaces = 0; }
#define addspacestoline(spaces) {          \
      for (spaces=spaces, p->lptr=p->lptr; \
        spaces; --spaces, p->lptr+=cw(space)) { \
    line.text[p->lptr] = space;            \
      }                                    \
}
#define okreturn {    \
  FCbuffer[0] = '\0'; \
  return &line;       \
}
/*test120994:*/
/*** 29/7/97 comment out: int grade = 2, asc; */
int getgrade(void)
{   return _grade; /* 29/07/97: changed grade to _grade */
}
int getasc(void)
{   return _asc; /* 29/07/97: changed asc to _asc */
}
static void __near toggle_grade()
{
    _grade = !(_grade - 1) + 1; /* 29/07/97: changed grade to _grade */
	return;
}
static void __near toggle_asc(void)
{
    _asc = !_asc; /* 29/07/97: changed asc to _asc */
    return;
}
/***test120994*/

struct txtline *IT_bembline (size_t linelength, size_t pagelength,
   struct pg_settings *p,
#if !defined(BRAILLE_IT)
   int *cwt,
#endif
   struct IT_FCode **BFC, FILE *source,
   unsigned long *modeflags, flag *forcedpage, flag *EndOfText)
{
  static char inbuffer[INBUFSIZE+1];
  static struct txtline line;
  static char Wbuffer[(MAXLINELEN)*5+1], FCbuffer[11];
  char svFC[11];
  static struct FCparam FCpar;
  struct FCparam svFCpar = { 0,0,{'\0'} };
  static struct tab tabs[9];
  size_t spaces;
  static size_t svspaces;
  static flag blockflg;
  static flag EndOfFile;
  flag EndLine;
  size_t l;
  int i;
  char *s;
  char modulename[] = "IT_bembline";

  p->lptr = 0;
  EndLine = false;
  svFC[0] = svFCpar.spar[0] = '\0';
  if (!(*modeflags & P_HEADER)
  && !(p->header_ind == 2 && p->linecount == 0))
    linelength -= (size_t)p->rmarg;
  memset(line.text, '\0', linelength+1);
  line.attribs = *modeflags;

  /* Re-init. buffers & make special call to IT_getword()
     & IT_report() if source = null */
  if (source == NULL) {
    IT_getword(source, inbuffer, Wbuffer, FCbuffer, &FCpar, BFC,
       EndOfText);
#if !defined(BRAILLE_IT)
    IT_report(NULL);
#endif
    return NULL;
  }

  if ((*modeflags & (P_HEADER | CENTRE | RJUST)))
    spaces = 0;
  else
  {
    spaces = p->lmarg + svspaces;
    svspaces = 0;
  }

  /* Build main body of line */
  while (p->lptr <= linelength
  && !*EndOfText && !EndLine) {
    if (!FCbuffer[0] && !Wbuffer[0]
    && IT_getword(source, inbuffer, Wbuffer, FCbuffer, &FCpar, BFC,
    EndOfText)
    == NULL) {
      if (*EndOfText == false) {
	IT_error(modulename, "Error: cannot read any further in input file\n"
	   "file position = %ld", ftell(source));
	return NULL;
      }
    }

    /* Do we need a word as well as a format? */
    if (FCbuffer[0] == ITFCSTART
    && (strcmp(FCbuffer+1, "B") == 0
    || strcmp(FCbuffer+1, "#") == 0
    || strcmp(FCbuffer+1, "SP") == 0
    || strcmp(FCbuffer+1, "TAB") == 0)
    || *modeflags & TABMODE) /* Tabjump failed on previous line */
    { /* Minor line format code; extra word needed */
      if (
      !(*modeflags & TABMODE)
      && IT_getword(source, inbuffer, Wbuffer, svFC, &svFCpar, BFC,
      EndOfText)
	 == NULL
      && *EndOfText == false) {
	IT_error(modulename, "Error: cannot read any further in input file\n"
	   "file position = %ld", ftell(source));
	return NULL;
      }
      if (strcmp(FCbuffer+1, "B") == 0) { /* Blank spaces */
	if ((l = TextLength(Wbuffer, cwt)) + p->lptr + FCpar.npar1 <= linelength)
	  spaces = FCpar.npar1;
      }
      else if (FCbuffer[1] == '#' /* Jump to tab */
      || *modeflags & TABMODE) { /* tabjump failed on previous line */
    if (*modeflags & TABMODE) /* Write lefthand margin now,
                            as IT_tabjump is unaware of it */ /*test140395*/
    addspacestoline(spaces)
    if (
	   IT_tabjump(FCpar.npar1, tabs, line.text, &p->lptr,
	      (p->header_ind == 2 && p->linecount == 0)
	      ? linelength : p->abslinelength,
          Wbuffer
#if !defined(BRAILLE_IT)
          , cwt
#endif
    )  == NULL) {
	  if (!(*modeflags & TABMODE)) { /* Tab has failed for 1st time */
	    *modeflags |= TABMODE; /* Try again on next line */
#if !defined(BRAILLE_IT)
        IT_report("Warning: tab overflow: %s%d %s", FCbuffer, FCpar.npar1,
	       Wbuffer);
#endif
        break;
	  }
	  else {
	    *modeflags &= ~TABMODE;
	    IT_error(modulename, "Tab jump failure: %s%d %s", FCbuffer, FCpar.npar1, Wbuffer);
	    return NULL;
	  }
	}
	*modeflags &= ~TABMODE;
	/* Righthand margin is released for tab jump except for top line
	   of page when header_ind == 2 */
	if (p->header_ind != 2 || p->linecount != 0) {
	  l = strlen(line.text);
	  memset(line.text+l, '\0', p->abslinelength-l);
	  linelength += p->rmarg;
	}
	++spaces;
      }
      else if (strcmp(FCbuffer+1, "TAB") == 0) { /* 1-time tab within line */
	if ((l = TextLength(Wbuffer, cwt)) > 0
	&& (int)(FCpar.npar1 - p->lptr - 1) > 0
	&& l + (int)(FCpar.npar1 - p->lptr - 1) <= linelength) {
	  spaces = (FCpar.npar1 - p->lptr - 1);
	}
      }
      else if (strcmp(FCbuffer+1, "SP") == 0) { /* 3-space para. */
	svspaces = 2;
	spaces = svspaces + 1;
      }
    }

   /* Major format code? */
   if (*FCbuffer == ITFCSTART) {
    /* Cancel effect of HD format if immediately followed by another format */
    if (p->lptr == (size_t) p->lmarg && (line.attribs & P_HEADER))
      line.attribs &= ~P_HEADER;

    /* Select major format */
    if (strcmp(FCbuffer+1, "FILL") == 0) {
      if (IT_mkfiller(FCpar.spar[0], FCpar.npar1, line.text, &p->lptr, linelength
#if !defined(BRAILLE_IT)
        , cwt
#endif
        ) == NULL) {
	IT_error(modulename, "Line overflow while executing %s format", FCbuffer);
	return NULL;
      }
    }
    else if (strcmp(FCbuffer+1, "BLOCK") == 0) { /* Copntinuous block */
      *modeflags |= BLOCK;
      blockflg = true;
      okreturn
    }
	else if (strcmp(FCbuffer+1, "HDO") == 0) { /* Page header overwrite */
	  *modeflags = (P_HEADER_OVRWRT);
	  blockflg = false;
	  okreturn
	}
	else if (strcmp(FCbuffer+1, "HD") == 0) { /* Page header */
	  *modeflags = P_HEADER;
      blockflg = false;
      okreturn
    }
    else if (FCbuffer[1] == 'H') { /* Title header */
      *modeflags |= T_HEADER;
      *modeflags &= ~FOOTER;
      blockflg = false;
    }
    else if (strcmp(FCbuffer+1, "FT") == 0) { /* Footer */
      *modeflags |= FOOTER;
      *modeflags &= ~T_HEADER;
      blockflg = false;
    }
    else if (strcmp(FCbuffer+1, "TT") == 0) /* New title */
      *modeflags |= NEWTITLE;
    else if (strcmp(FCbuffer+1, "CS") == 0) { /* Centre */
      *modeflags |= CENTRE;
      *modeflags &= ~(P_HEADER | P_HEADER_OVRWRT | RJUST);
      okreturn
    }
    else if (FCbuffer[1] == '\\') {
      if (p->lptr > 0) {
    if (*modeflags & (CENTRE | RJUST | BLOCK | T_HEADER | FOOTER))
	  okreturn
#if !defined(BRAILLE_IT)
    else
      IT_report("Warning: backslash '\\' ignored");
#endif
      }
    }
    else if (strcmp(FCbuffer+1, "RM") == 0) { /* Right justify */
      *modeflags |= RJUST;
      *modeflags &= ~(P_HEADER | P_HEADER_OVRWRT | CENTRE);
      okreturn
    }
    else if (strcmp(FCbuffer+1, "99") == 0) { /* 3 asterisks */
      /* Module AGEDHBM */
      l = strlen(&FCbuffer[1]);
      for (i=0; i<3; i++) {
	strcat(line.text, &FCbuffer[1]);
	p->lptr += TextLength(&FCbuffer[1], cwt);
	if (i + 1 < 3) {
	  line.text[p->lptr] = space;
	  p->lptr += cw(space);
	}
      }
    }
    else if (strcmp(FCbuffer+1, "SL") == 0) { /* Blank line */
      if (FCpar.npar1 > 0) {
	p->blanklines = FCpar.npar1;
	*modeflags |= BLANK;
	okreturn
      }
    }
    else if (strcmp(FCbuffer+1, "NL") == 0) { /* Conditional newline */
      line.attribs = *modeflags;
      *modeflags = NORMAL;
      blockflg = false;
      okreturn /* If p->lptr = 0 non-effective blank line is produced */
    }
    else if (strcmp(FCbuffer+1, "LN") == 0
    && (p->lptr > 0 && p->linecount + FCpar.npar1 + 1 > pagelength
    || p->lptr == 0 && p->linecount + FCpar.npar1 > pagelength)
    ) { /* Conditional newpage */
      *modeflags = NORMAL;
      *forcedpage = true;
      blockflg = false;
      okreturn
    }
    else if (strcmp(FCbuffer+1, "PG") == 0) { /* Forced page */
      *forcedpage = true;
      blockflg = false;
      *modeflags = NORMAL;
      okreturn
    }
    else if (strcmp(FCbuffer+1, "BP") == 0) { /* Reset brl page-number */
      if (IT_setpageno(FCpar.npar1, &p->pagecount, MAXPAGEVAL) == -1) {
	IT_error(modulename, "Error while executing $BP format");
	return NULL;
      }
    }
    else if (strcmp(FCbuffer+1, "PP") == 0) { /* Reset print page-number */
      IT_set_ppstr(FCpar.spar, p->pp, MAXLINELEN / 4 - 1);
    }
    else if (strcmp(FCbuffer+1, "CPYPP") == 0) { /* Copy string from
		print page buffer to line */
      for (i=0; p->pp[i] != '\0'; p->lptr += cw(FCpar.spar[i]), ++i)
	line.text[p->lptr] = p->pp[i];
    }
    else if (strcmp(FCbuffer+1, "BPA") == 0) { /* Arabic page-numbering */
      p->paginat_ind = ARABIC;
    }
    else if (strcmp(FCbuffer+1, "BPR") == 0) { /* Roman page-numbering */
      p->paginat_ind = ROMAN;
    }
    else if (strcmp(FCbuffer+1, "MR") == 0) { /* Right margin */
      if (IT_setmargin(FCpar.npar1, &p->rmarg, linelength
#if !defined(BRAILLE_IT)
        , cwt
#endif
      ) == -1) {
	IT_error(modulename, "Margin set error while executing %s format", FCbuffer);
	return NULL;
      }
    }
    else if (FCbuffer[1] == 'M') { /* Left margin */
      if (IT_setmargin(FCpar.npar1, &p->lmarg, linelength - 5
#if !defined(BRAILLE_IT)
        , cwt
#endif
      ) == -1) {
	IT_error(modulename, "Margin set error while executing %s format: %d",
	   FCbuffer, FCpar.npar1);
	return NULL;
      }
    }
    else if (strcmp(FCbuffer+1, "STB") == 0) { /* Set tab */
      if (IT_settab((size_t)FCpar.npar1, FCpar.spar[0], (size_t)FCpar.npar2,
	 tabs, p->abslinelength) == -1) {
	IT_error(modulename, "Tab set error");
      }
    }
    /***test120994:*/
    else if (strcmp(FCbuffer+1, "G") == 0) /* Toggle grade */
      toggle_grade();
    else if (strcmp(FCbuffer+1, "ASC") == 0) /* Toggle ASCII */
      toggle_asc();
    /*test120994*/

    /* Clear FCbuffer */
    strcpy(FCbuffer, svFC); /* Refill if there is a saved FC */
    if (svFC[0] != '\0')
      FCpar = svFCpar;
    svFC[0] = '\0';
   } /* End of format codes */

    if (p->lptr > 0
    && line.text[p->lptr-1] != space
    && spaces == 0)
      ++spaces;
    /* Add word, if room; if not room, try splitting */
    l = TextLength(Wbuffer, cwt);
    if (l > 0 /* Word to add */
    && spaces + l + p->lptr <= linelength) { /* & word fits on line */
      /* Add space(s) to line */
      addspacestoline(spaces)
      addstrtoline(Wbuffer)
      Wbuffer[0] = '\0';
    }
    else if (l > 0) { /* Word does not fit on line */
       /* Try splitting current word */
      char tmpword[(MAXLINELEN)*5+1] = { '\0' };
      i = linelength - (p->lptr + spaces);
      s = NULL;
      if (i > 0) {
	if ((s = IT_splitbrlword(Wbuffer, tmpword, i)) == NULL) {
	   /* No split */
	  /* Short line? */
#if !defined(BRAILLE_IT)
      if (p->lptr > 0 && p->lptr + 1 + spaces < linelength - 10)
	    IT_report("Warning: short line: page %d line %d", p->pagecount,
	       (p->no_pg_breaks == true)
	       ? p->abslinecount
	       : (p->header_ind == 1 || p->header_ind == 2)
	       ? p->linecount + 1
           : p->linecount + 2);
#endif
      if (!*EndOfText && p->lptr > 0) {
        EndLine = true;
        break;
      }
	}
	else if (s) { /* Add 1st part of split word */
	  /* Add space(s) to line */
      /* Add space(s) to line */
      addspacestoline(spaces)
      addstrtoline(Wbuffer)
	  strcpy(Wbuffer, tmpword);
      l = TextLength(Wbuffer, cwt);
      EndLine = true;
    }
      }
      else if (!*EndOfText) {
        EndLine = true;
        break;
      }
    }
    if (p->lptr <= (size_t) p->lmarg && l > linelength - p->lmarg) {
       /* Line overflow */
      addspacestoline(spaces)
      for (i=0; p->lptr < linelength; p->lptr+=cw(Wbuffer[i]), ++i)
	line.text[p->lptr] = Wbuffer[i];
      strcpy(Wbuffer, &Wbuffer[i]);
#if !defined(BRAILLE_IT)
      IT_report("Warning: Word longer than line-length:\n %s", Wbuffer);
#endif
    }
  } /* Line completed */

  if (blockflg == true) {
    ++p->blocksize;
    line.attribs |= BLOCK;
  }
  else
    p->blocksize = 0;

  /* Last word in file needs 1 more line occasionally */
  if (*EndOfText && Wbuffer[0] != '\0') { /* 1 word remains to be processed */
    *EndOfText = false;
    EndOfFile = true;
  }
  else if (EndOfFile && Wbuffer[0] == '\0') {
    EndOfFile = false;
    *EndOfText = true;
  }


  okreturn
}

/* ITS - Make page header */
/* N. Octon      20/3/92 */
/* (Module AGEDHNB) */
/* Returned: 0 on success, else !0 */


int mk_pagehdr(char *header, char *printpagestr, size_t pagecount,
   int paginat_ind)
{
  size_t lbpn, lppn, s;
  char *bp, *p;

  /* clear away old printpagestr & brlpagestr */
  p = header;
  while (*p != space)
    *p++ = space;
  p = header + strlen(header) - 1;
  while (*p != space)
    *p-- = space;

  /* Calc. available space for printpagestr */
  for (s=0; header[s] == space; ++s)
    ;
  if ((lppn = strlen(printpagestr)) > s - 1) {
#if !defined(BRAILLE_IT)
    IT_report("Print page number truncated: page %d", pagecount);
#endif
    memcpy(header, printpagestr, s - 1);
  }
  else
    memcpy(header, printpagestr, lppn);

  /* Check that brl pagenumber does not encroach on text; if test fails,
     truncate text at righthand end */
  p = header + strlen(header) - 1;
  for (s=0; *p == space; --p)
    ;
  p = header + strlen(header);
  bp = (paginat_ind == ROMAN) ? IToBRom((int) pagecount)
     : IToBrl((int) pagecount);
  lbpn = strlen(bp);
  if (lbpn > s - 1) {
#if !defined(BRAILLE_IT)
     IT_report("Braille page number encroached on text.\n"
       "Text truncated: page %d", pagecount);
#endif
    *(p - lbpn - 1) = space;
  }
  /* Insert braille page number */
  memcpy(p - lbpn, bp, lbpn);

  return 0;
}


/* ITS - Set braille emboss format margins - left/right */
/* N. Octon      09/01/92 */
/* (Module AGEDHBF) */
/* Returned: Absolute new value on success, else -1 */

int IT_setmargin(int mvalue, int *mbuf, int maxlen
#if !defined(BRAILLE_IT)
  , int *cwt
#endif
)
{
  if (abs(mvalue) > maxlen)
    return -1;
  *mbuf = mvalue;
  return abs(*mbuf);
}

/* ITS - Set tab */
/* N. Octon      21/01/92 */
/* (Module AGEDHBH) */
/* Returned: 0 on success, -1 on error */

#define MINTAB 1
#define MAXTAB 9

int IT_settab(size_t tabnumber, char tabtype, size_t tabpos,
   struct tab tabset[], size_t linelength)
{

  if (tabnumber < MINTAB || tabnumber > MAXTAB) {
#if !defined(BRAILLE_IT)
    IT_report("Tab Set Error: tab number out of range %d-%d: %d",
	MINTAB, MAXTAB, (int)tabnumber);
#endif
    return -1;
  }
  else if (tabpos < 1 || tabpos > linelength) {
#if !defined(BRAILLE_IT)
    IT_report("Tab set error: invalid tab position: %u", tabpos);
#endif
    return -1;
  }
  else if ((tabtype &= 0xDF) != 'L' && tabtype != 'R' && tabtype != 'D') {
#if !defined(BRAILLE_IT)
    IT_report("Tab set error: invalid tab type: \'%c\'", tabtype);
#endif
    return -1;
  }
  --tabnumber;
  tabset[tabnumber].type = tabtype;
  tabset[tabnumber].pos = (tabpos * CWU);

  return 0;
}
#undef MINTAB
#undef MAXTAB

/* ITS - Jump to tab (IT_tabjmp) */
/* N. Octon      21/01/92 */
/* (Module AGEDHBD) */
/* Returned: Incremented line if nextword is not overlength, else null */

#define calcfiller(cwu, tabfiller) { \
  if (cwu >= (int)((cw(tabfiller))*2 + (cw(space)*2))) { /* 2 filler & 2 spaces */ \
    spaces = 1; \
    cwu2 = cwu - (cw(space) * 2); \
    filler = (cwu2 / cw(tabs[tabnumber].filler)); \
  } \
  else \
    spaces = (cwu / cw(space)); \
}
#define MINTAB 1
#define MAXTAB 9
#define BRLDECIMAL '1'
#define BRLNUMERAL '#'


char *IT_tabjump (size_t tabnumber,
   struct tab tabs[], char *line, size_t *lpos, size_t llength,
   char *nextword
#if !defined(BRAILLE_IT)
   , int *cwt
#endif
)
{
  int spaces, filler;
  int cwu, cwu2;
  int i;
  char modulename[] = "IT_tabjump";

	if (tabnumber < MINTAB || tabnumber > MAXTAB) {
#if !defined(BRAILLE_IT)
    IT_report("Error: Tab number out of range %d-%d: %u",
	MINTAB, MAXTAB, tabnumber);
#endif
    return NULL;
  }

	/* Tabs 8-9 have default filler chars */
	--tabnumber;
  if (tabnumber + 1 == 9)
    tabs[tabnumber].filler = '-';
  else if (tabnumber + 1 == 8)
    tabs[tabnumber].filler = '\'';
  else
    tabs[tabnumber].filler = space;

  /* Check for valid character widths for space & filler */
  if (cw(space) == 0 || cw(tabs[tabnumber].filler) == 0) {
	 IT_error(modulename, "Invalid character width value for tab filler or for space");
    return NULL;
  }

  filler = 0;
  if (tabs[tabnumber].type == 'L') {
    cwu = (tabs[tabnumber].pos - *lpos - 1);
    if (cwu < (int)cw(space))
      return NULL;
    calcfiller(cwu, tabs[tabnumber].filler);
    if (spaces == 0 || *lpos + cwu + TextLength(nextword, cwt) > llength)
      return NULL;
  }
  else if (tabs[tabnumber].type == 'R') {
    cwu = (tabs[tabnumber].pos - *lpos
	- 1 - (TextLength(nextword, cwt) - 1));
    if (cwu < (int)cw(space))
      return NULL;
    calcfiller(cwu, tabs[tabnumber].filler);
    if (spaces <= 0 || *lpos + cw(nextword[strlen(nextword)-1]) >= llength)
      return NULL;
  }
  else if (tabs[tabnumber].type == 'D') { /* Decimal-align */
    size_t diglen = 0;
    char *barabp;
    char barabcharset[] = "ABCDEFGHIJabcdefghij1'";
    /* Is there a brl decimal point in nextword, & is nextword a brl number? */
    if (strchr(nextword, BRLDECIMAL) <= (barabp = strchr(nextword, BRLNUMERAL))
    && barabp > NULL && strspn(barabp + 2, barabcharset) > 0) {
#if !defined(BRAILLE_IT)
      IT_report("Warning: Decimal-align tab jump followed by non-decimal - "
      "Tab ignored");
#endif
      spaces = 1;
    }
    else {
      /* Calc. length of string up to (not including) decimal point */
      i = 0;
      while (nextword[i] != BRLDECIMAL) /* Braille decimal point */
	diglen += cw(nextword[i++]);
      cwu = (tabs[tabnumber].pos - *lpos - 1 - diglen);
      if (cwu < (int)cw(space))
	return NULL;
      calcfiller(cwu, tabs[tabnumber].filler);
      if (spaces == 0)
	return NULL;

      /* Calc. length of string from (including) decimal point */
      diglen = 0;
      i = strcspn(nextword, "1"); /* Point to decimal point */
      while (nextword[i] != '\0')
	diglen += cw(nextword[i++]);
      if (*lpos + cwu + diglen >= llength)
	return NULL;
    }
  }
  else { /* Invalid tab type */
#if !defined(BRAILLE_IT)
    IT_report("Invalid tab type: \"%c\"\n"
       "Possible use of unset tab: %d",
       tabs[tabnumber].type, tabnumber+1);
#endif
    return NULL;
  }

  /* Augment the line */
  if (filler > 0) {
    if (*lpos > 0)
      line[*lpos] = space;
    else
      line[*lpos] = tabs[tabnumber].filler;
    ++*lpos;
    for (i=filler; i; --i) {
      line[*lpos] = tabs[tabnumber].filler;
      ++*lpos;
    }
    line[*lpos] = '\0';
  }
  else {
    --spaces; /* Don't print rightmost space; caller sees to that */
    memset(&line[(*lpos)], space, spaces);
    *lpos += spaces;
    line[*lpos] = '\0';
  }

  return line;
}
#undef calcfiller
#undef MINTAB
#undef MAXTAB
#undef BRLDECIMAL
#undef BRLNUMERAL

/* ITS - Brl Emboss Format */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.5*29/05/92*Standardised function naming & calling convention.*30301*MJH*/
/***************************************************************************/
/*1.4*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.3*20/05/92*Made check for valid source name compatible with  *30301*MJH*/
/* . *  /  /  * braille translate module.                        *     *   */
/***************************************************************************/
/*1.2*19/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.1*07/05/92*Integration work.                                 *30301*MJH*/
/***************************************************************************/
/*1.0*29/11/91*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .LIB file:                            */
/*                                                                         */
/* BRLEMBFM.OBJ                                                            */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/
/* (Module AGED) */
/* Returned: 0 on success, else nonzero */


extern boolean abrt;
const char IT_bformversno[] = "1.00";
#if !defined(BRAILLE_IT)
int *cwt, min_non0width, max_non0width;
#endif

int BRLEmbForm (char *infilename, char *outfilename, char *fctabname,
#if !defined(BRAILLE_IT)
    char *cwtname,
#endif
    char *reportfilename, size_t linelength, size_t pagelength,
	size_t startpage, int header_ind, flag header_ref, flag reportflag,
    boolean ScreenOn
#if !defined(BRAILLE_IT)
    ,boolean SysPause
#endif
)
{
  FILE *source, *dest, *reportdest;
  static struct IT_FCode **BFC;
  char modulename[] = "IT_BEmbFormat";
/* Read format codes table */
  if ((BFC = IT_GetBFormatCodes(fctabname, BFC,
     linelength, pagelength)) == NULL) {
    return FALSE;
  }

#if !defined(BRAILLE_IT)
/* Read character width table */
  if ((cwt = IT_readcwt(cwtname, cwt, &min_non0width, &max_non0width)) == NULL) {
	 errno = 0;
    return FALSE;
  }
#endif

  if ((source = ITS_fopen(infilename, "rb")) == NULL) {
     return FALSE;
  }
  if ((dest = ITS_fopen(outfilename, "w")) == NULL) {
     return FALSE;
  }

  /* Report-file or screen? */
  if (reportflag && reportfilename) {
	 if ((reportdest = ITS_fopen(reportfilename, "w")) == NULL);
  }
  else
    reportdest = stdin;

  if (
	  IT_formatexec(source, dest, infilename, outfilename,
	linelength, pagelength, startpage, header_ind, header_ref,
    BFC,
#if !defined(BRAILLE_IT)
    cwt,
#endif
	ScreenOn
#if !defined(BRAILLE_IT)
    , SysPause
#endif
    ) != 0 && (abrt == FALSE)) {
     return FALSE;
  }

  ITS_fclose(source);
  ITS_fclose(dest);
  if (reportdest != stdin)
	 ITS_fclose(reportdest);

  return TRUE;
}
