/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/***************************************************************************/
/*               UNIFIED ITS SOURCE CODE 21-Mar-1995                       */
/***************************************************************************/
/* TRANSLAT.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.4*06/11/97*Fix of 29/7/97 to make extended ASCII into punct. *60000*NO */
/* . *  /  /  *commented out, as it caused  to have letter sign.*     *   */
/***************************************************************************/
/*1.4*29/07/97*Bugfix (grade/asc): not properly communicated from*60000*NO */
/* . *  /  /  *translator to formatter.                          *     *   */
/***************************************************************************/
/*1.3*??/11/95*Bug fixes - acronyms.                             *60000*NO */
/***************************************************************************/
/*1.2*11/07/95*Installed acronym table in TRANS.C; introduced    *30420*NO */
/*   *        *ACRONYM.H; BRAILLE_IT directive controls whether  *     *   */
/*   *        *acronym processing occurs before or during        *     *   */
/*   *        *translation.                                      *     *   */
/***************************************************************************/
/*1.1*04/05/95*Binary table init.  Remove unwanted funcs./args.  *30418*NO */
/* . *  /  /  *for Braille It.                                   *     *   */
/***************************************************************************/
/*1.0*15/09/94*File split from TRANFILE.C.                       *30414*NO */
/* . *  /  /  *decis[] now a 1-dim. array of pointers to cond.   *     *   */
/***************************************************************************/
/**************************************************************************
 *                               ITTRANS                                  *
 *                    AUTOMATIC TEXT-TO-BRAILLE/MOON TRANSLATOR		  *
 *                         MS-DOS 'C' VERSION                             *
 *                             VERSION 1.02                                *
 *                                                                        *
 *                        Programmers: Mark Glover                       *
 *			  Norman Octon					 *
 *         Copyright Royal National Institute for the Blind 1990-4         *
 *                                                                        *
 *		Returns a pointer to the translated string		  *
 *		  & sets the int pointer sent to it			  *
 *		to the length of the translated string.			  *
 *		It may also modify the external flag translat_err.	  *
 **************************************************************************/
#include "its.h"
#include "itsfunc.h"
#include "ittrans.h"
#include "tranfile.h"
#include "translat.h"
#include "acronym.h"
#pragma warning(once:4057)

size_t   spcgate, scount;
int translat_err;
int _grade, _asc; /* 29/07/97 */

unsigned char buffer[BUF_SIZE+1];
unsigned char letter, tclass;
unsigned char __near *inptr;

enum { cont_test, cont_mismatch, cont_match, cont_fail, cont_found } matchind;
enum flag eosind; /*end-of-string indicator */
unsigned long nwords;

/* itinit */

struct alph  *alphabet; /*v. 2.2*/
struct cont  *contraction; /*v. 2.2*/

size_t   nat, nct, nnt, nsv, nic, ndtc;
size_t inpspaces;
unsigned   translat_flags;  /*	Bit 0 = caps enable
                            	bit 1 = alphanumsign enable
                                bit 4 = retain newlines
                        	*/
unsigned char   rcclas[MAXRCC][OUT_SIZE], nonter[MAXRCC];
unsigned char value[256], bval[BUF_SIZE];
unsigned char   __near *stvar;
unsigned char  __near *trans[NSV], __near *decis[NIC],
           __near *cond[NDTC];

unsigned char table_versno[7];
unsigned char capsign, alphanumsign, format_start;
size_t buf_size, out_size;
int zeroshftind;

/* blockio */
unsigned char inblok[BLK_BUFSIZE+2], outblok[BLK_BUFSIZE*2+2],
    *inblkptr, *endinblk,
    *outblkptr, *endoutblk;
int inbuf_full;

/***************************************************************************
 * ITS - Read translation table into memory from binary file.              *
 *     11-Apr-1995                                                         *
/***************************************************************************/

/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.0*11/04/95*Binary table init. module.                        *30418*NO */

#define NOERROR 0
#define ERROR -!NOERROR
unsigned xormask = 5;
int inc = 3;

int e_read(void *buf, size_t nbytes, FILE *f)
{
    size_t i;
    int c;
    char modulename[] = "e_read";
    static err = NOERROR;

    /* Prevent function from working if previous error */
    if (err == ERROR) /*test020595*/
        return ERROR; /*test020595*/

    for(i=0; i<nbytes; ++i) {
        if (feof(f)
        ||  (c = fgetc(f)) == EOF
        ||  ferror(f)) {
            long fp;
            sprintf(errmsg, "Cannot read from file:\npos. %ld",
              (fp=ftell(f)) );
            if (fp == _filelength(_fileno(f)))
                strcat(errmsg, " .. Attempt to read beyond end-of-file");
            ITS_Error(errmsg, modulename, -1, -1);
            err = ERROR;
            return ERROR;
        }
        ((unsigned char *)buf)[i] = (unsigned char)(c ^ xormask);
    }

    xormask = _rotl(xormask, inc);
    return NOERROR;
}

int e_putback (char c, FILE *f, boolean shift_mask)
{
    static long filepos;
    char modulename[] = "e_read";

    /* More than one consecutive char cannot be put back. Check ...  */
    if (filepos >= ftell(f) - 1) {
        ITS_Error("This function cannot be called for 2 subsequent characters",
          modulename, -1, -1);
        return ERROR;
    }

    if (shift_mask == TRUE)
        xormask = _rotr(xormask, inc);

    if (ungetc(c^xormask, f) == EOF) {
        sprintf(errmsg, "Error returning character '%c' to buffer", c);
        ITS_Error(errmsg, modulename, -1, -1);
        return ERROR;
    }

    filepos = ftell(f);
    return NOERROR;
}

static int readbin(char *binaryfilename)
{
/* N.B. Any change to the order in which elements of the file are read
   must be strictly reflected in the corresponding write routine.
*/
    char crlf[] = "\r\n", comma[] = ",";
    char commachar = ',';
    char allocerr[80] = "** Memory allocation error:\n"
		      "last successful allocation at address";
    char modulename[] = "readfrombin";
    char c;
    FILE *bin_f;
    int r;
    unsigned i, ii, e;

    if ((bin_f = ITS_fopen(binaryfilename, "rb")) == NULL) {
        return FALSE;
    }

    /* Table header */
    while (fgetc(bin_f) != '\n')
        ;
    while (fgetc(bin_f) != 'B') /* 'B' added to original v. no. for Binary */
        ;
    ungetc('B', bin_f);
    fscanf(bin_f, "%6s", table_versno);
    if (strchr((char *) table_versno, '.') == NULL) {
        sprintf(errmsg, "Invalid binary table file", binaryfilename);
        ITS_Error(errmsg, modulename, -1, -1);
        return FALSE;
    }
    while (fgetc(bin_f) != EOFCHAR)
        ;

    /* Transition table & state variables */
    r = e_read(&nsv, sizeof(nsv), bin_f);
    r |= e_read(&nic, sizeof(nic), bin_f);
    nsv = __min(nsv, NSV);
    nic = __min(nic, NIC);
    for (i=0; i<nsv; ++i) {
    	if ((trans[i] = _ncalloc(nic + 1, 1)) == NULL) {
            sprintf(errmsg, "Memory allocation error: trans%d\n", i);
        	ITS_Error(errmsg, modulename, -1, -1);
            return FALSE;
        }
    }
    for (i=0; i<nsv; ++i)
        r |= e_read(trans[i], nic, bin_f);
    if ((stvar = _ncalloc(nsv + 1, 1)) == NULL) {
        sprintf(errmsg, "Memory allocation error: state variables\n");
        ITS_Error(errmsg, modulename, -1, -1);
        return FALSE;
    }
    r |= e_read(stvar, nsv, bin_f);
    if (r == ERROR) {
        ITS_Error("Error reading transition table from binary file.",
          modulename, -1, -1);
        return FALSE;
    }

    /* Decision/condition table */
    r = e_read(&ndtc, sizeof(ndtc), bin_f);
    ndtc = __min(ndtc, NDTC);
    for ( i = 0; i < ndtc; ++i ) {
    	/* Read decision table field */
     	if (
          (cond[i] = (unsigned char __near *) _ncalloc(nsv + 1, 1)) == NULL
     	) {
            sprintf(errmsg, "Memory allocation error - decis / cond%d\n", i);
        	ITS_Error(errmsg, modulename, -1, -1);
            return FALSE;
        }
    }
    for (i=0; i<ndtc; ++i) {
        while ((r|=e_read(&c, sizeof(char), bin_f)) != ERROR && c != commachar) {
            /* Un-get it and re-read as an int */
            if (e_putback(c, bin_f, 1) == ERROR)
                return FALSE;
            r |= e_read(&ii, sizeof(int), bin_f);
            if (ii & 0x8000) /* Unconditional decision - do not assign
                                to any conditions */
                decis[(ii & ~0x8000)] = NULL;
            else
            	decis[ii] = cond[i];
        }
        if (r == ERROR)
            break;
        r |= e_read(cond[i], nsv, bin_f);
    }
    if (r == ERROR) {
        ITS_Error("Error reading decision/condition table from binary file.",
          modulename, -1, -1);
        return FALSE;
    }

    /* Miscellaneous settings */
    r = e_read(&buf_size, sizeof(int), bin_f);
    r |= e_read(&out_size, sizeof(int), bin_f);
    r |= e_read(&inpspaces, sizeof(int), bin_f);
    r |= e_read(&capsign, sizeof(char), bin_f);
    r |= e_read(&translat_flags, sizeof(int), bin_f);
    r |= e_read(&alphanumsign, sizeof(char), bin_f);
    r |= e_read(&format_start, sizeof(char), bin_f);
    if (r == ERROR) {
        ITS_Error("Error reading miscellaneous settings from binary file.",
          modulename, -1, -1);
        return FALSE;
    }

    /* RCC table */
    r = e_read(&nnt, sizeof(int), bin_f);
    nnt = __min(nnt, MAXRCC);
    for (i=0; i<nnt; ++i) {
        r |= e_read(&nonter[i], sizeof(char), bin_f);
        r |= e_read(&rcclas[i], sizeof(rcclas[i]), bin_f);
    }
    if (r == ERROR) {
        ITS_Error("Error reading rcc table from binary file.", modulename,
          -1, -1);
        return FALSE;
    }

    /* Alphabet/Contraction table */
    r = e_read(&nat, sizeof(int), bin_f);
    r |= e_read(&nct, sizeof(int), bin_f);
    nat = __min(nat, ALPHA);
    nct = __min(nct, NENTRY);

    /* Put minimal validation here (?) ... */
    if (!inpspaces || !buf_size || !out_size || !nat || !nct
    || !nsv || !nic || !ndtc) {
        ITS_Error("Binary table initialisation error", modulename, -1, -1);
        return FALSE;
    }
    r |= e_read(value, sizeof(value), bin_f);

    /* Allocate alphabet & contraction structures */
    if ((alphabet = (struct alph *) _ncalloc(nat,
      sizeof(struct alph))) == NULL) {
        ITS_Error("Error allocating memory for the Alphabet table.", modulename, -1, -1);
        return FALSE;
  	}
    if ((contraction = (struct cont *) calloc(nct,
      sizeof(struct cont))) == NULL) {
        ITS_Error("Error allocating memory for the Contraction table.", modulename, -1, -1);
        return FALSE;
    }

    for (i=0; i<nat; ++i)
        r |= e_read(&alphabet[i].extent, sizeof(int), bin_f);

    if (r == ERROR) {
        printf("Error reading alphabet table's extent or value to binary file.\n");
        return FALSE;
    }

    e = 0;
    for (i=0; i<nat; ++i) {
        r |= e_read(&alphabet[i].symbol, sizeof(char), bin_f);
        r |= e_read(&alphabet[i].cclass, sizeof(char), bin_f);
        if (i + 1 == nat)
            break; /* Remaining elements of this entry are superfluous */
        r |= e_read(&alphabet[i].altsg, sizeof(char), bin_f);
        r |= e_read(&alphabet[i].singsg, sizeof(char), bin_f);
        r |= e_read(&alphabet[i].ascsg, sizeof(char), bin_f);
        /* Contractions for this entry */
        if (alphabet[i+1].extent > 0) {
        	while (e < alphabet[i+1].extent -1) {
                r |= e_read(&contraction[e].branch, sizeof(int), bin_f);
                r |= e_read(&contraction[e].iclass, sizeof(char), bin_f);
                r |= e_read(&contraction[e].shifno, sizeof(char), bin_f);
                r |= e_read(&contraction[e].rcc, sizeof(char), bin_f);
                r |= e_read(&c, sizeof(char), bin_f);
                /* We now know how much memory to allocate for ctable */
                if ((contraction[e].ctable = calloc(c+1, 1)) == NULL) {
                    if (e > 0) --e;
                    sprintf(errmsg, "%s %p\n%s: ", allocerr, &contraction[e],
                      "contraction table error"
                    );
                	ITS_Error(errmsg, modulename, -1, -1);
                    return FALSE;
                }
                r |= e_read(contraction[e].ctable, c, bin_f);
                r |= e_read(&c, sizeof(char), bin_f);
                /* We now know how much memory to allocate for tsign */
                if ((contraction[e].tsign = calloc(c+1, 1)) == NULL) {
                    if (e > 0) --e;
                    sprintf(errmsg, "%s %p\n%s: ", allocerr, &contraction[e],
                      "contraction table error"
                    );
                	ITS_Error(errmsg, modulename, -1, -1);
                    return FALSE;
                }
                r |= e_read(contraction[e].tsign, c, bin_f);
                ++e;
            }
        }
        if (r == ERROR) {
            if (e > 0) --e;
            sprintf(errmsg, "Error reading alphabet/contraction table from binary file.\n"
              "Alphabet entry: %d (%c); contraction entry: %c%s\n",
              i+1, alphabet[i].symbol, alphabet[i].symbol, contraction[e].ctable
            );
            ITS_Error(errmsg, modulename, -1, -1);
            return FALSE;
    	}
    }

    if (ITS_fclose(bin_f) != 0) {
        return FALSE;
    }
    return TRUE;
}

struct acro acronym[NUMBER_ACRONYMS];
struct a_initl a_initial[256];

int __cdecl acro_compare(const void *ac1, const void *ac2)
{
    int cmp;
    cmp = strcmp( (*(struct acro *)ac1).in, (*(struct acro *)ac2).in );
    return cmp * -1; /* Reverse sense to cause descending sort */
}

static int __near read_acronym_table(char *acrotablename)
{
    FILE *f;
    unsigned char str[PARAMENTRYLENGTH];
    unsigned char *p, *p2;
    static const unsigned char nulstr[] = "";
    int i, l, ret;
    static unsigned char initial;
    char modulename[] = "read_acronym_table";
    int nacronyms;

    if ((f = ITS_fopen(acrotablename, "r")) == NULL)
        return FALSE;

    ret = TRUE;
    i = nacronyms = 0;
    while (fgets(str, sizeof str, f) != NULL) {
        if (i + 1 == NUMBER_ACRONYMS) {
            sprintf(errmsg, "Too many acronyms\nMax: %d", NUMBER_ACRONYMS);
            ITS_Error(errmsg, modulename, -1, -1);
            return FALSE;
        }
        l = strlen(str) - 1;
        if (str[l] == '\n')
            str[l] = '\0';
        else { /* Line too long */
            sprintf(errmsg, "Entry %d too long.\nMax. should be %d",
              i+1, sizeof str - 2); /* -2 for '\0' and '\n' */
            ITS_Error(errmsg, modulename, -1, -1);
            ret = FALSE;
            break;
        }
        if (!*str || *str == '|') { /* Ignore blank or comment line */
            continue;
        }
        /* Break up str into in- and out- (etc) elements */
        if ((p = strchr(str, '|')) != NULL) {
            /* Memory allocation for input of table */
            acronym[i].in = malloc((p-str)+1);
            if (acronym[i].in == NULL) {
                ret = FALSE;
                break;
            }
            strncpy(acronym[i].in, str, p-str);
            acronym[i].in[p-str] = '\0';
            acronym[i].case_insens = acronym[i].ignore_trailing_letters = FALSE;
            ++p;
            p2 = p;
            while ((p2 = strchr(p2, '|')) != NULL) {
                *p2 = '\0';
                ++p2;
                if (_strnicmp(p2, "ci", 2) == 0)
                    acronym[i].case_insens = TRUE;
                else
                if (_strnicmp(p2, "tl", 2) == 0)
                    acronym[i].ignore_trailing_letters = TRUE;
            }
            /* Memory allocation for output of table */
            acronym[i].out = malloc(strlen(p)+1);
            if (acronym[i].out == NULL) {
                ret = FALSE;
                break;
            }
            strcpy(acronym[i].out, p);
        }
        else {
            /* Memory allocation for input of table */
            acronym[i].in = malloc(strlen(str)+1);
            if (acronym[i].in == NULL) {
                ret = FALSE;
                break;
            }
            strcpy(acronym[i].in, str);
            acronym[i].out = (char *)nulstr;
        }

        if (acronym[i].case_insens) {
            _strlwr(acronym[i].in);
        }
        ++i;
    }
    nacronyms = i;

    if (ITS_fclose(f) != 0 || ret == FALSE)
        return FALSE;

    /* Now sort array of structures acronym[] */
    if (ret == TRUE) {
        qsort(acronym, nacronyms, sizeof(struct acro),
          acro_compare);

    	/* No. of entries for each initial */
    	for (i=0; i<nacronyms; ++i) {
        	if (*acronym[i].in == initial)
                ++a_initial[*acronym[i].in].count;
        	else {
                if (a_initial[*acronym[i].in].start == NULL) {
                    a_initial[*acronym[i].in].start = &acronym[i];
                        	/* Address-of pointer assignment
                               to mark new initial, & to relate initial's value to a
                               position in the acronym table */
                    a_initial[*acronym[i].in].count = 1;
                	initial = *acronym[i].in;
                } /* Mark start of block of initial letters */
            } /* end else */
        } /* end if */
    } /* a_initial.start pointers initialised */

    return ret;
}


static void __near close_acronym_table(void); /* Prototype */ /*test130695*/
static void __near close_acronym_table(void)
{
    int i;

    i = 0;
    while (acronym[i].in != NULL) {
        a_initial[*acronym[i].in].start = NULL; /*test191095*/
        if (*acronym[i].out)
            free(acronym[i].out);
        free(acronym[i].in);
        acronym[i].in = acronym[i].out = NULL;
        ++i;
    }

#if defined(BRAILLE_IT)
    _heapadd((void __far *)acronym, sizeof acronym);
    _heapadd((void __far *)a_initial, sizeof a_initial);
#endif
    return;
}

#if defined(BRAILLE_IT)
char acrotextfl_name[FILENAMELENGTH] = "itacrofl.tmp";/*test200695*/
      /* Try to incorporate this global more sensibley */
int preprocess_acronyms(char *source)
{
/* Use functions called by TranslateFile plus process_acronyms minus ittrans */
    return translatefile(source, NULL, 0, TRUE
#if !defined(BRAILLE_IT)
        , FALSE
#endif
        );
}
#endif

int itinit(char *transtablename, char *acrotablename
#if defined(BRAILLE_IT)
  , char *source
#endif
)
{
    int i; /* 29/07/97 */

    if (acrotablename != NULL
    &&  read_acronym_table(acrotablename) == FALSE)
        return FALSE;

/*** 29/07/97: Modify _ctype[] extended ASCII chars - accents - not punct. */
/*061197 .. Comment out:    for (i=0x80; i<=0xa8; ++i) {
        (_ctype+1)[i] |= _UPPER;
    }*test*/
    /***/

#if defined(BRAILLE_IT)
    if (preprocess_acronyms(source) == FALSE)
        return FALSE;
    close_acronym_table();
#endif
    if (readbin(transtablename) == FALSE)
        return FALSE;


    return TRUE;
}


unsigned char *ittrans(unsigned char *instr, unsigned char *outstr,
   int *len, unsigned mode)
{
  int i;
  char /*test020595__near*/ *in_p;
  char modulename[] = "ittrans";

  /* Has the translator been initialised? */
  if (nsv == 0 || nic == 0 || ndtc == 0 || nat == 0 || nct == 0
  || stvar == NULL || trans == NULL || decis == NULL)
  { ITS_Error("Translator not initialised", modulename, -1, -1);
    return NULL;
  }

  outstr[0] = '\0'; /* Clear output buffer before use */

  /* If <mode> is set, set appropriate translation flag (state variable)
     (bottom byte of <mode> */
  i = (int)(mode & 0x00ff);
  if (i >= (int)nsv)
    stvar[(i-1)%nsv] = 'N'; /* Reset */
  else if (i > 0)
    stvar[i - 1] = 'Y';
  /* Set translat_flags from upper byte of <mode> */
/****29/07/97: Transfer grade/asc info from translator to formatter via globals */
  _grade = (i & GRADE1)?1:2;
  _asc = (i & ASCCODE) ? 1 : 0; /* not operative */
/***/
  i = (int)(mode >> 8);
  if (i > 0)
    translat_flags |= (unsigned)i;

  /* get the first bufferful of characters from the input string? */
  if (buffer[0] == '\0') {
    /* Duplicate input string */
/*test020595:    if ((in_p = _nstrdup(instr)) == NULL) {
        ITS_Error("Cannot duplicate input string", modulename, -1, -1);
        return NULL;
    }*test*/
    in_p = instr;/*test020595*/
    shift (buf_size-1, in_p);
  }

  do {
    if (translat_err & 0xfff0) break;
    translate_chr (in_p, outstr, len);
  } while (letter && !(translat_err & 0xfff0));
/*test020595  _nfree(in_p);*/
  if (!(translat_err & 0xfff0)) {
    return (outstr);
  }
  else {
    *len = EOF;
    return(NULL);
  }
}

static void __near translate_chr (unsigned char __near *instr, unsigned char *outstr,
    int *outlen)
{
  int l;

  /* IDENTIFY LEFTMOST CHARACTER OF BUFFER */
  while ((letter = bval[0]) == 0) {
    if (buffer[0] == '\0') { /* Test for end */
      /*Strip out the mandatory leading space from output string */
      if (!isspace(instr[0])
      && isspace(outstr[0]))
	strcpy(outstr, outstr+1);
      /* Strip out the trailing space */
      l = strlen(outstr);
      if (!isspace(instr[strlen(instr)-1])
      && isspace(outstr[l-1]))
	outstr[--l] = '\0';
      *outlen = l;
      return;
    }
    else if (buffer[0] == '\n'
    &&  translat_flags & KEEPNEWLINES>>8) { /*test250195*/
      accept_newline(instr, outstr);
      adjust_SV ();
    }
    else {
      /* CHARACTER NOT FOUND - OUTPUT WARNING MESSAGE */
		sprintf(errmsg, "Unidentified Character in file:\n"
		    "Character     = %c\n"
		    "Decimal value = %d\n"
			 "Hex value     = %X\n",buffer[0],buffer[0],buffer[0]);
		ITS_Error(errmsg, "translate_chr", -1, -1);
      /* skip over unrecognised character */
      if (shift (1, instr) == 0)
	translat_err |= 0x10;
    }
  }
  search (instr, outstr);
		/* CHARACTER(S) ACCEPTED - BEGIN SEARCH FOR CONTRACTIONS */
  if (buffer[0] != '\0')
    adjust_SV ();
}

/* TRANSLATION TABLE SEARCH */
/* INDEX POINTS TO CONTRACTION TABLE ENTRY UNDER CONSIDERATION.
*/
static void __near search (unsigned char __near *instr, unsigned char *outstr)
{
  size_t index, pointr;
  size_t nxtlev;

  index = alphabet[letter-1].extent;
  nxtlev = 1;
  matchind = cont_test;
  do {
    /*The following conditions, except for
       cont_fail/cont_found, are independent, since matchind can be
       altered by the called functions */
    if (matchind == cont_test)
      test_match (index, nxtlev, (size_t __near *)&pointr);
    if (matchind == cont_match)
      match (index, pointr);
    if (matchind == cont_mismatch)
      mismatch ((size_t __near *)&index, pointr, (size_t __near *)&nxtlev);
  } while (matchind != cont_fail && matchind != cont_found);
  if (matchind == cont_fail || index > nct)
    no_cont (instr, outstr);
  else if (matchind == cont_found)
    accept (instr, outstr, index, pointr);
}

#pragma optimize("zax",on)
/* TEST FOR MATCH BETWEEN CURRENT TABLE ENTRY AND BUFFER */
static void __near test_match (size_t index, size_t nxtlev, size_t __near *pointr)
{
  unsigned char t;
  for ( *pointr = nxtlev; *pointr < buf_size; ++*pointr ) {
      t = (unsigned char) toupper(contraction[index-1].ctable[*pointr-1]);
      if ( t == '\0' ) {
	break;
      }
      if ((unsigned char) toupper(buffer[*pointr]) != t) {
	matchind = cont_mismatch;
	return;
      }
  }
  matchind = cont_match;
}
#pragma optimize("",on)

#pragma optimize("z",on)
/* MISMATCH; OBTAIN INDEX FOR NEXT ENTRY TO BE COMPARED */
static void __near mismatch (size_t __near *index, size_t pointr, size_t __near *nxtlev)
{
  int bri;
  bri = contraction[*index-1].branch;
  if (bri > 0 ) goto fs;
  if (pointr != *nxtlev ) goto ff;
  four: if (-bri < (int) *nxtlev) goto ff;
  ++*index;
  for (;;) {
    bri = contraction[*index-1].branch;
    if (bri <= 0 ) goto four;
    *index = bri;
  }
  ff:
  *nxtlev = -bri;
  if (*nxtlev < 1) {
    matchind = cont_fail;
    return;
  }
  ++*index;
  matchind = cont_test;
  return;

  fs:
  if (pointr != *nxtlev) {
    if (bri != (int) *index+1) ++*nxtlev;
    ++*index;
    matchind = cont_test;
    return;
  }
  *index = bri;
  matchind = cont_test;
}
#pragma optimize("z",off)

/* CHARACTER STRING MATCHES LEFT SUBSTRING OF BUFFER */
/* NOW TEST NEXT CHARACTER IN BUFFER TO CHECK THAT RIGHT CONTEXT IS OK */
static void __near match (size_t index, size_t pointr)
{
  size_t n, nn, j, t;

  t = contraction[index-1].rcc;
  if (t == ' ') {
    check_IC (index);
    return;
  }
  n = bval[pointr];
  if (n == 0) {
    matchind = cont_mismatch;
    return;
  }
  for ( nn = 0; nn < nnt; ++nn ) {
    if (t != nonter[nn]) continue;
    for ( j = 0; j <= 3; ++j ) {
    if (alphabet[n-1].cclass == rcclas[nn][j]) {
	  check_IC (index);
	  return;
	}
	if (rcclas[nn][j] == 0xFF ) {
	  matchind = cont_mismatch;
	  return;
	}
    }
    matchind = cont_mismatch;
    return;
  }
  matchind = cont_mismatch;
}

/* RIGHT CONTEXT IS OK - NOW CHECK INPUT CLASS FOR COMPATIBILITY */
/* WITH CURRENT VALUE OF STATE VARIABLES */
static void __near check_IC (size_t index)
{
  size_t k, dkt, n;

  tclass = (unsigned char) contraction[index-1].iclass;
    k = tclass-1;
    for ( n=0; n<nsv; ++n) {
      if (decis[k] == NULL) {
        n = nsv; /* Accept unconditionally */
        break;
      }
      dkt = decis[k][n];
      if (dkt != '-' && dkt != stvar[n])
	    break;
      }
  if (n == nsv) {
    matchind = cont_found;
    return;
  }
  matchind = cont_mismatch;
  return;
}

/* CONTRACTION TABLE ENTRY IS ACCEPTED */
/* SEND APPROPRIATE SIGNS TO OUTPUT BUFFER AND SHIFT INPUT BUFFER LEFT BY */
/* NUMBER OF CHARACTERS INDICATED IN CONTRACTION TABLE ENTRY */
static void __near accept (unsigned char __near *instr, unsigned char *outstr,
   size_t index, size_t pointr)
{
  unsigned char frmtstr[64] = {'\0'}, fractstr[BUF_SIZE*2+2] = {'\0'};
  unsigned char  *t;
  flag fractflg = false;

  if (_strnicmp(buffer, "$f", 2) == 0 && isdigit(buffer[2])) /* Fraction */
    fractflg = true;
  if (shift (contraction[index-1].shifno, instr)
     < (int) contraction[index-1].shifno)
      translat_err |= 0x10;

  t = (unsigned char *) contraction[index-1].tsign;
  if (*t)
    strcat(outstr, t);

  if (fractflg) { /* Fraction */
    dlr_F(instr, fractstr);
    if (translat_err & 0xFFF0)
      return;
    strcat(outstr, fractstr);
    fractflg = false;
  }

  if (contraction[index-1].shifno == 0)
	++zeroshftind;
  if (contraction[index-1].shifno == 0 && buffer[0] == format_start) {
    read_format(buffer, pointr, instr, frmtstr);
	zeroshftind = 0;
    if (*t != '\0')
      strcat(outstr, frmtstr);
  }
  else if (contraction[index-1].shifno == 0 && zeroshftind > BUF_SIZE) {
	/* Infinite loop where shiftcount == 0 caught here */
	sprintf(errmsg, "Shift count of 0 in translation table: entry %d. %c%s",
       index, alphabet[letter-1].symbol, contraction[index-1].ctable);
	ITS_Error(errmsg, "accept", -1, -1);
	translat_err |= 0x0040;
  }
  else if (contraction[index-1].shifno != 0)
	zeroshftind = 0;
}

/* ADJUST STATE VARIABLES ACCORDING TO TRANSITION TABLES */
static void __near adjust_SV (void)
{
  unsigned char tnt;
  int n;

  for (n=0; n<(int)nsv; ++n) {
      tnt = trans[n][tclass-1];
      if (tnt == '-') continue;
      if (tnt != 'S' && (tnt != 'T' || stvar[n] != 'N'))
        stvar[n] = 'N';
      else
        stvar[n] = 'Y';
  }
}

/* NO CONTRACTION IS ACCEPTABLE - OUTPUT SINGLE CHARACTER */
/* OBTAINED FROM ALPHABET TABLE                           */
static void __near no_cont (unsigned char __near *instr, unsigned char *outstr)
{
  size_t l;
  unsigned char outsig;
  if (stvar[ALTCODE-1] == 'Y')
    outsig = alphabet[letter-1].altsg;
  else if (stvar[ASCCODE-1] == 'Y')
    outsig = alphabet[letter-1].ascsg;
  else
    outsig = alphabet[letter-1].singsg;
  if (shift (1, instr) == 0)
    translat_err |= 0x10;
  zeroshftind = 0;
  tclass = (unsigned char) alphabet[letter-1].cclass;
  l = strlen(outstr);
  outstr[l] = outsig;
  outstr[l+1] = '\0';
}

/* Handle newline */
static void __near accept_newline (unsigned char __near *instr, unsigned char *outstr)
{
  size_t l;
  char space = ' ';

  if (shift (1, instr) == 0)
    translat_err |= 0x10;
  tclass = (unsigned char) alphabet[value[space-1]].cclass;
  l = strlen(outstr);
  outstr[l] = '\n';
  outstr[l+1] = '\0';
  return;
}

static int __near shift (int nch, unsigned char __near *str)
{
  int i, n;
  static unsigned char nxtchr;
  static enum flag alphanum = false, caps = false, format = false;
  static postcodeind;
         /* 1=postcode detected; 2=space in postcode; 3=insert ';' before no. */

  if (inptr == NULL) { /* Not yet initialised for new string */
    inptr = str;
    buffer[buf_size-1] = ' ';
    bval[buf_size-1] = value[buffer[buf_size-1]];
    eosind = scount = spcgate = 0;
  }
  for ( i = 1; i <= nch; ++i) {
    memcpy(buffer, buffer+1, buf_size-1);
    memcpy(bval, bval+1, buf_size-1);
    n = buf_size - 1;
    /* Get next translatable char from input stream */
    while (
       *inptr == '\f' || *inptr == '|' || *inptr == '\r' || *inptr == EOFCHAR
       || (spcgate == 0
         && (*inptr == '\t'
           || (*inptr == '\n'
             &&  !(translat_flags & KEEPNEWLINES>>8))
           || *inptr == ' '))
       || (buffer[BUF_SIZE-3] != ' ' && buffer[BUF_SIZE-2] == '-'
         && *inptr == '\n')
    ) /* All conditions for ignoring next char */
      ++inptr;
    if (*inptr == alphanumsign
    ||  *inptr == ALTERNATE_LETTERSIGN
    ||  _strnicmp(inptr, "$asc", strlen("$asc")) == 0
    )
      alphanum = true; /* Prevent further insertion of alphanumsign */
    if (*inptr == '\0')
      eosind = true;
    if (eosind == false) {
      if ((translat_flags & ALPHANUMSIGNENABLED)
      && !alphanum
	  && !format
	  && !(islower(*inptr))
      && (postcodeind == 3 /* insert letter sign after no. */
      || isalphanum(inptr, &postcodeind))) {
	/* Don't shift buffer, but insert alphanumsign at nxtchr */
	alphanum = true;
	nxtchr = alphanumsign;
    if (postcodeind == 3)
      postcodeind = 0;
      }
      else if (isalpha(*inptr) && !format
	  && caps == false
	  && (translat_flags & CAPSENABLED)) {
	nxtchr = capsign;
	caps = true;
      }
      else {
	nxtchr = *inptr;
	++inptr;
	if (nxtchr == '\t')
	  nxtchr = ' ';
    if (nxtchr == '\n' && !(translat_flags & KEEPNEWLINES>>8))
	  nxtchr = ' ';
	if (islower(nxtchr))
	  nxtchr = (unsigned char) toupper(nxtchr);
	if (isalpha(nxtchr) && caps == true)
      caps = false;
    if (postcodeind == 1 && nxtchr == ' ') /* 1st part of postcode finished */
      ++postcodeind;
    else if (postcodeind == 2 && isalpha(*(inptr+1)))
      ++postcodeind;

      }
    }
    else {
      ++scount;
      if (scount == 1 && spcgate == inpspaces && !isspace(*(inptr - 1))
      && inptr - str > 1)
	nxtchr = ' ';
      else nxtchr = '|'; /* Start filling buffer with '|' - onset of finish */
    }
    if (nxtchr == ' ') {
      if (spcgate > 0)
	--spcgate;
      alphanum = false;
    }
    else {
      spcgate = inpspaces; /* This value diminishes with each space,
			      & when = 0 no more spaces are allowed */
	  if (nxtchr == format_start && !(isdigit(*inptr))) {
	format = true;
      }
      else if (nxtchr == SLASH)
	alphanum = false;
    }
    if (!format && spcgate == inpspaces - 1) {
      ++nwords;
    }
    else if (spcgate < inpspaces)
      format = false;
    if (buffer[0] == '|') { /* End of current string */
      ++i; /* i must always be 1 more than no. of shifts on exit from loop */
      break;
    }
    buffer[n] = nxtchr; /*n should not be changed after exit from for loop */
    bval[n] = value[nxtchr];
  }

  nch = i - 1;
  if (buffer[0] == '|') { /* Reinit. buffer for next call to CURFUNC */
    inptr = NULL;
    for (i=0; i<(int)buf_size; ++i)
      buffer[i] = '\0';
  }
  return(nch);
}


#define FRACT_SEPRTR 0x2C /* Fraction separator before punctuation */
#define BRLLETTERSIGN ';' /* Translated braille letter sign */
static void __near dlr_F (unsigned char __near *shiftstr, unsigned char *outstr)
{
  unsigned num_digits, den_digits, i, ii;
  unsigned char numerator[BUF_SIZE],
	denominator[BUF_SIZE], number[BUF_SIZE];
  unsigned char *p = outstr;

  num_digits = den_digits = ii = 0;

  /* get fraction numerator from buffer */
  for (i=0; (numerator[i] = buffer[i]) != SLASH && i<buf_size; ++i) {
    if ((number[i] = (unsigned char) (numerator[i] % 16)) == 0)
      number[i] += 10; /* Account for braille zero - J */
    ++num_digits;
  }
  if (numerator[i] != SLASH)
    translat_err |= 0x1;
  else
    ++num_digits;
  for (i=0; i<num_digits-1; ++i) {
    *p++ = (unsigned char) (number[i] + 64);
  }

  /* Get fraction denominator from buffer */
  for (i=0; i<buf_size; ++i) {
    denominator[i] = buffer[num_digits+i];
    if (i == 0) {
      if (shift(num_digits, shiftstr) < (int) num_digits)
	translat_err |= 0x10;
      num_digits = 0;
    } /*Shift numerator when at least 1 denom. digit present */
    if (denominator[i] < '0' || denominator[i] > '9')
      break;
    number[i] = (unsigned char) (denominator[i] % 16);
    ++den_digits;
  }
  if (!isspace(denominator[i]) && !isalpha(denominator[i])
  && denominator[i] != '=' && denominator[i] != '-' && denominator[i] != '$') {
    number[i] = 0xFF; /*rogue value for insertion of separator */
    ii = 1; /*Extra byte for punctuation separator */
  }
  for (i=0; i<den_digits+ii; ++i) {
    if (number[i] != 0xFF)
      *p++ = denominator[i];
    else
      *p++ = FRACT_SEPRTR; /*Separator before punctuation */
  }
  if (shift (den_digits, shiftstr) < (int) den_digits)
    translat_err |= 0x10;

  if (isalpha(buffer[0])) /* Letter abutting fraction */
    *p++ = BRLLETTERSIGN;
}
#undef FRACT_SEPRTR
#undef BRLLETTERSIGN


static void __near read_format (unsigned char *in, size_t strlength,
    unsigned char __near *shiftstr, unsigned char *outstr)
{
  int i;
  static unsigned char *p;
  static unsigned char evebstr[64];
  static enum flag numalphflg; /* Exceptionally allow numeric/alpha string */
  static enum flag fillflg; /* true if $fill */

  p = outstr + strlen(outstr); /*strlen usually == 0 except when this
				function is called recursively */
  i = (p == outstr) ? 1 : 0;
  while (i<(int)strlength) {
    *p++ = in[i++];
  }
  if (_strnicmp(in, "$stb", 4) == 0)
    numalphflg = true;
  else if (_strnicmp(in, "$eb", 3) == 0 || _strnicmp(in, "$ev", 3) == 0) {
    strncpy(evebstr, in, 3);
    evebstr[3] = '\0';
    if (evebtranslat ((unsigned char __near *)evebstr, (unsigned char __near *)shiftstr, outstr,
		(stvar[GRADE1-1]=='Y')?GRADE1:
        (stvar[ALTCODE-1] == 'Y')?ALTCODE:0) == NULL)
      translat_err |= 0x2;
    return;
  }
  /* Special treatment for $FILL since the form it stringCN */
  if (_strnicmp(in, "$fill", 5) == 0)
    fillflg = true;

  /* Shift the main body of the format code */
  if (shift (strlength, shiftstr) < (int) strlength)
    translat_err |= 0x10;

  /* $FILL */
  if (fillflg) {
    fillflg = false;
    *p++ = in[0];
    /* Shift the fill char */
    if (shift(1, shiftstr) < 1)
      translat_err |= 0x10;
  }

  /* Shift the digits */
  i = 0;
  while (isdigit(in[i])) {
    *p++ = in[i];
    ++i;
  }

  if (shift (i, shiftstr) < i)
    translat_err |= 0x10;
  if (numalphflg == true) {
    numalphflg = false;
    read_format(in, 1, shiftstr, outstr);
  }
}

static unsigned char *__near evebtranslat (unsigned char __near *format,
             unsigned char __near *instr, unsigned char *outstr,
			 size_t transmode)
{
  size_t x, svspcgate, svscount;
  int tmplen;
  unsigned char tmpbuf[BUF_SIZE+1], volnumstr[4], tmpbval[BUF_SIZE];
  unsigned volnum;
  unsigned char svletter;
  unsigned char __near *svinptr, *p;
  enum flag sveosind;

  if (shift(3, instr) < 3)
    translat_err |= 0x10;
  if (_stricmp(format, "$ev") == 0) {
    if (sscanf(buffer, "%d", &volnum) != 1) {
      return NULL;
    }
    tmplen = strlen(_itoa(volnum, volnumstr, 10));
    if (shift (tmplen, instr) < tmplen)
      translat_err |= 0x10;
  }

  /* Save settings before calling CURFUNC recursively */
  strcpy(tmpbuf, buffer);
  for (x=0; x<buf_size; ++x) {
    tmpbval[x] = bval[x];
    buffer[x] = '\0';
  }
  svinptr = inptr;
  inptr = NULL;
  svspcgate = spcgate;
  svscount = scount;
  svletter = letter;
  sveosind = eosind;

  p = outstr + strlen(outstr);
  if (_stricmp(format, "$ev") == 0) {
    strcpy(format, "$H $NL $H $CS $FILL312\\End of v");
    strcat(format, volnumstr);
    CURFUNC(format, p, &tmplen, transmode);
    if (tmplen < 0)
      translat_err |= 0x20;
    strcat(outstr, "PG");
  }
  else if (_strnicmp(format, "$eb", 3) == 0) {
    CURFUNC("$H $NL $H $CS $FILL312\\THE END", p, &tmplen, transmode);
    if (tmplen < 0)
      translat_err |= 0x20;
    strcat(outstr, "PG");
  }
  /* Restore saved settings */
  for (x=0; x<buf_size; ++x)
    bval[x]=tmpbval[x];
  inptr = svinptr;
  spcgate = svspcgate;
  scount = svscount;
  letter = svletter;
  eosind = sveosind;
  strcpy(buffer, tmpbuf);

  return outstr;
}


static int __near ispostcode(unsigned char __near *str)
{
    char s1[3], s2[3], s3[3], s4[3], s5[3], s6[3];
    int i = 0;
    if ((i =
      sscanf(str, " %2[A-Z]%2[0-9]%*[ \t\n]%2[0-9]%2[A-Z]%1[.,()-:;!\"'?]",
      s1, s2, s3, s4, s5)) >= 4) /* Standard postcode */
        return true;
    else if (i == 2
    && sscanf(str, " %2[A-Z]%2[0-9]%1[A-Z]%*[ \t\n]%2[0-9]%2[A-Z]%1[.,()-:;!\"'?]",
      s1, s2, s3, s4, s5, s6) >= 5) /* London postcode */
        return true;
    return false;
}

/* Isalphanum function.
   Returns true if string argument is alpha followed immediately by numeric.
   Punctuation between the alpha & the numeric is ignored.
   N Octon		31/7/91
*/
static int __near isalphanum(unsigned char __near *str, int *postcodeind)
{
/* Mod. 24-01-94 to restrict definition of alphanum to upper case letters - NO*/
  int i, j;

  i = 0;
 do {
  while (isalpha(str[i]) && isupper(str[i]))
    ++i;
  if (i == 0)
    return(0);
  while (ispunct(str[i])) {
    if ((str[i] == '-' && str[i-1] == '-')
    || str[i] == format_start
    || str[i] == ALTERNATE_LETTERSIGN
    || str[i] == BACKSLASH)
      return(0);
    ++i;
  }
 } while (isalpha(str[i]) && isupper(str[i]));

  /* Ok so far. Now the numeric part */
  j = i;
  while (isdigit(str[i]))
    ++i;
  if (i == j)
    return(0);

  /* Alphanumeric ok.  Is it also a postcode? */
  if (ispostcode(str))
    *postcodeind = 1;
  return(i);
}

/**************************************************************************
 *				 TRANSLATOR INIT			 *
 *		      INTEGRATED TRANSLATION SYSTEM				 *
 *                         MS-DOS 'C' VERSION                             *
 *                             VERSION 1.01                                *
 *									  *
 *			PROGRAMMERS:					  *
 *			  Norman Octon					 *
 *			  Mark Glover					 *
 *         Copyright Royal National Institute for the Blind 1991          *
 *                                                                        *
 *	   Returns true on success; otherwise false.			  *
 **************************************************************************/

/* Block i/o functions.
   getstrbuf & putstrbuf point to a string representing an ASCII file line
       terminated by cr/lf.

   9/11/90		N. Octon
*/


unsigned char *__near getstrbuf(unsigned char *buf, FILE *rfileptr) {
  unsigned char *tmpptr, *tmpptr2;
  static eofind = 0;

  if (eofind) {
    eofind = 0; /* Reset for next call */
    inblkptr = NULL;
    return NULL;
  }

  if (endinblk == NULL) { /*initialise */
    endinblk = buf + BLK_BUFSIZE - 1;
  }

  tmpptr = NULL;
  if (inblkptr != NULL)
    tmpptr = strchr(inblkptr, '\0');
  if (tmpptr != NULL) ++tmpptr;
  if (tmpptr == NULL
	|| strchr(tmpptr, '\n') == NULL
	|| tmpptr > endinblk) {
    if (inblkptr != NULL && inblkptr <= endinblk)
      inblkptr = strchr(inblkptr, '\0') + 1;
    if ((tmpptr = getblok(inblok, rfileptr)) == NULL) {
      if (inblkptr != NULL) { /* No more to input,
				but residue must be cleared */
	eofind = 1;
	return inblkptr; /* getblok sees to the buffer residue */
      }
      else
	return NULL;
    }
  }
  if ((tmpptr2 = strchr(tmpptr, '\n')) <= endinblk && tmpptr2 != NULL) {
    *tmpptr2 = '\0';
    inbuf_full = 0;
  }
  else /* Newline not found within scope of buffer */
    inbuf_full = !0;

  inblkptr = tmpptr;
  return inblkptr;
}

#if !defined(BRAILLE_IT)
unsigned char *__near putstrbuf(unsigned char *buf,
   unsigned char *str, FILE *wfileptr, int crlfmode)
{
  static prevline;

  if (endoutblk == NULL) endoutblk = buf + (BLK_BUFSIZE*2) - 1;
  if (outblkptr == NULL) {
    outblkptr = buf;
    prevline = 0;
  }

  /*advance output buffer pointer to beginning of next line */
  outblkptr += prevline;

  if (outblkptr + strlen(str) > endoutblk) {
    putblok(buf, wfileptr);
    outblkptr = buf;
  }
  strcpy(outblkptr, str);
  if (str[strlen(str)-1] != '\n' && crlfmode == CRLF)
    strcat(outblkptr, "\n");
  prevline = strlen(outblkptr);
  return outblkptr;
}
#endif

unsigned char *getblok(unsigned char *buf, FILE *rfileptr) {
  int l, r;

  if (inblkptr == NULL || inblkptr > endinblk)
    inblkptr = buf;
  /* Copy residue of buffer to buffer start & point after it */
  if (inblkptr > buf) {
    strcpy(buf, inblkptr);
    l = strlen(buf);
    inblkptr = buf + l;
  }
  else
    l = 0;


  /*Now read from file */
  if (!feof(rfileptr) && (r = fread(inblkptr, 1, BLK_BUFSIZE - l, rfileptr)) > 0
  && !ferror(rfileptr)) {
    inblkptr = buf;
    endinblk = buf + r + l - 1;
    /* Reinit. buffer from end of input string to buffer end */
    blk_clrbuf(buf+r+l, BLK_BUFSIZE - r - l);
    return buf;
  }
  else if (feof(rfileptr)) {
    if (l == 0)
      inblkptr = NULL;
    else
      inblkptr = buf;
    return NULL;
  }
  else
    return inblkptr = NULL; /* Error while reading */
}

unsigned char *putblok (unsigned char *buf, FILE *wfileptr) {

  if (endoutblk == NULL) endoutblk = buf + (BLK_BUFSIZE*2) - 1;
  if ( strlen(outblok) > 0
  && fwrite(buf, 1, strlen(outblok), wfileptr) == 0
  || ferror(wfileptr))
    return outblkptr = NULL;
  else {
    outblkptr = buf;
	 blk_clrbuf(buf, (BLK_BUFSIZE*2));
    return buf;
  }
}

void __near blk_clrbuf(unsigned char *buf, int length) {
  int i;

  for (i=0; i<length; ++i) /*Element buf[length] is always NULL */
    buf[i] = '\0';
}

void itclose ()
{
  /* Free up all the allocated memory of itinit.
     N. Octon   12/12/91
     Mod. 13/12/93 to take account of whole block allocation in itinit()
  */

  struct cont  *c = contraction;
  size_t i;

#if !defined(BRAILLE_IT)
    close_acronym_table(); /*test050795*/
#endif
    for (i=0; i < nct; ++i) {   /* Ver 2.0 changed "<=" to "<" */
    free((unsigned char  *) c[i].tsign);
    free((unsigned char  *) c[i].ctable);
  }
  free(alphabet);
  free(contraction);

  /* Transition, decision & condition tables, & stvar */
  for (i=0; i<nsv; ++i)
    _nfree((unsigned char __near *) trans[i]);
  for (i=0; i<nic; ++i)
    _nfree((unsigned char __near *) decis[i]);
  for (i=0; i<ndtc; ++i) {
    _nfree((unsigned char __near *) cond[i]);
  }
  _nfree((unsigned char __near *) stvar);

  /* Reinit. buffer pointers */
  inblkptr = NULL;
  outblkptr = NULL;

  return;
}

/* TRANFILE.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.5*27/01/95*Don't let tranfile() set/reset grade when trnslate*30417*NO */
/***************************************************************************/
/*2.4*21/09/94*check_last_word(): dash is word boundary.         *30414*NO */
/* . *  /  /  *ismidsequence(): "--" added.                      *     *   */
/***************************************************************************/
/*2.3*12/01/94*spaceflg handles insertion of interblock space.   *30410*NO */
/***************************************************************************/
/*2.2*09/12/93*Allocate alphabet & contraction table in one go. *BRL_IT*KJM*/
/***************************************************************************/
/*2.1*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*2.0*08/03/93*Amended itclose for free error                    *     *MJH*/
/***************************************************************************/
/*1.9*19/02/93*Strip multi. lead/trail spaces instead of 1       *30301*NO */
/* . *  /  /  *                                                  *     *   */
/* . *  /  /  *Amend to allow unsetting stvars on calling.       *     *   */
/***************************************************************************/
/*1.9*18/03/93* /B option now performs Transform		 		 *30301*MW */
/***************************************************************************/
/*1.8*10/07/92*Amend v.1.6: space before '-' makes \n -> ' '	 *30301*NO */
/***************************************************************************/
/*1.7*08/06/92*Fraction routine & caller bug fix   		 		 *30301*NO */
/***************************************************************************/
/*1.6*03/06/92*Hyphen / newline: newline does not -> space	     *30301*NO */
/***************************************************************************/
/*1.5*01/06/92*Moon translation added. (ITTRANS.H -> TRANFILE.H) *30301*MJH*/
/***************************************************************************/
/*1.4*29/05/92*Standardised function naming & calling convention.*30301*MJH*/
/***************************************************************************/
/*1.3*22/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.2*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.1*11/05/92*C-scape screen handling facilities introduced.    *30301*MJH*/
/***************************************************************************/
/*1.0*02/01/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* TRANFILE.OBJ                                                            */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

#pragma warning(once:4057)

#if !defined(_MSC_VER)
  #include <dir.h>
#endif

char *tran_ver = "1.8";
extern unsigned long nwords;
extern unsigned char *endinblk;

static unsigned char * __near strip_lead_trail(unsigned char *string) {
  int l;

  l = strlen(string) - 1;
  /*Strip trailing spaces */
  while (isspace(string[l]))
    string[l--] = '\0';
  /* Strip out leading spaces */
  while (isspace(*string))
    ++string;

  return string;
}

static void __near findrchr(unsigned char *buf, unsigned char chr, FILE *fp,
    unsigned char *endbufp)
{
/* Find the rightmost character in buf &, if found, adjust fp downward. */
/* N. Octon	02/01/92 */
  unsigned char *p, *p2;
  unsigned l;

  if ((p = strrchr(buf, chr)) != NULL) {
    p2 = buf + strlen(buf) - 1;
    l = p2 - p;
    if (l > 0) {
      if (fseek(fp, -(int)l, SEEK_CUR) != 0) {
	ITS_Error("File pointer manipulation error!", "findrchr", -1, -1);
	return;
      }
      *(p+1) = '\0';
      endbufp -= l;
    }
  }
  return;
}

static int __near ismidsequence(unsigned char *word)
{
/* Find whether the 1st word in a given string is in the middle of or the
   beginning of a braille sequence.  Such a sequence must not be broken in
   translation.  The string <word> must be an integral number of words,
   but only the 1st word is checked.
   ** Mod. 17/12/93 N. Octon: include spaced hyphen.
   Return: true if at beginning or middle of sequence; else false.
   N. Octon	07/01/92
*/
/*** Added 15/07/94 N. Octon to skip over leading punctuation */
  while (ispunct(*word) && *word != '-') /* except hyphen/dash */
    ++word;
/***                                                          */

  if (_strnicmp(word, "the ", 4) == 0
  || _strnicmp(word, "of ", 3) == 0
  || _strnicmp(word, "and ", 4) == 0
  || _strnicmp(word, "for ", 4) == 0
  || _strnicmp(word, "with ", 5) == 0
  || _strnicmp(word, "a ", 2) == 0
  || _strnicmp(word, "in ", 3) == 0
  || _strnicmp(word, "in--", 4) == 0
  || _strnicmp(word, "be ", 3) == 0
  || _strnicmp(word, "be--", 4) == 0
  || _strnicmp(word, "was ", 4) == 0
  || _strnicmp(word, "was--", 5) == 0
  || _strnicmp(word, "were ", 5) == 0
  || _strnicmp(word, "were--", 6) == 0
  || _strnicmp(word, "his ", 4) == 0
  || _strnicmp(word, "his--", 5) == 0
  || _strnicmp(word, "enough ", 7) == 0
  || _strnicmp(word, "enough--", 8) == 0
  || _strnicmp(word, "to ", 3) == 0
  || _strnicmp(word, "into ", 5) == 0
  || _strnicmp(word, "by ", 3) == 0
  || _strnicmp(word, "- ", 2) == 0
  || _strnicmp(word, "--", 2) == 0
  )
    return true;
  return false;
}

static void __near check_last_word(unsigned char *buf, FILE *fp)
{
  unsigned char *p = NULL, *p2, *p3;
  size_t l;

  /* First find rightmost nonspace */
  p = buf + strlen(buf);
  if (p > buf)
    --p;
  while (isspace(*p)) {
    *p = ' '; /* Convert white space to actual space */
    --p;
  }

  /* Find beginning of "word" */
  while (!isspace(*p)) {
    --p;
    if (*(p+1) == '-' && *(p+2) == '-')
      break;
  }
  ++p; /* At start of rightmost word */

  if (p <= buf)
    return;
  if (ismidsequence(p) == false)
    return;
  if (_stricmp(p, "a ") == 0
  || _stricmp(p, "the ") == 0) /*rightmost word is assumed to be end-of-sequence */
    return;

  /* We are in midsequence & must go back 1 word at a time */
  p2 = p;
  do {
    /* Find beginning of previous word */
    p3 = p;
    --p;
    while (isspace(*p)) {
      *p = ' '; /* Convert white space to actual space */
      --p;
    }
    /* Find beginning of word */
    while (!isspace(*p)) {
      --p;
      if (*(p+1) == '-' && *(p+2) == '-')
        break;
    }
    ++p;
  } while (p > buf && ismidsequence(p));

  /* update file pointer */
  if (p <= buf)
    p = p2;
  p2 = buf + strlen(buf);
  l = p2 - p3;
  if (l > 0 && fseek(fp, -(long)l, SEEK_CUR) != 0) {
	 ITS_Error("ITS translator - input file-pointer error",
						"check_last_word", -1, -1);
    return;
  }
  /* Terminate buffer */
  *p3 = '\0';

  return;
}

/* test260595: Design 2: search every "word" in every <str> to see if an
   acronym exists for it; start search at correct point in acronym table,
   & continue only for as many acronyms as there are with that initial.
*/
static void __near process_acronyms(unsigned char *str)
{
    unsigned char *p;

    p = str;
    while (*p && p != NULL) {
        /* Check leading punct. & initial of word for acronym */
        do {
            enum {nomore, common, uppercase} match = uppercase;
            boolean match_found = FALSE;
            struct acro *a_p;
            /* Any acronyms for this char? */
            a_p = a_initial[*p].start; /*test070795*/
          do {
            if (a_p) {
                int e;
                e = (match == common) ? a_initial[tolower(*p)].count
                  : a_initial[*p].count;
                    	/* Control to be decremented while nonzero */
                /* Process each acronym until match found/matches exhausted */
                while (e--) {
                    boolean a_fail = FALSE;
                    int l1, l2;
                    l1 = strlen(a_p->in);

                    /* The following code eliminates some acronyms */
                    if (!a_p->case_insens && strncmp(p, a_p->in, l1) != 0)
                        a_fail = TRUE;
                    else if (a_p->case_insens && _strnicmp(p, a_p->in, l1) != 0)
                        a_fail = TRUE;
                    if (a_fail == FALSE
                    &&  ( p > str
                      &&  (ispunct(*p)
                        && !ispunct(*(a_p->in)) )
                      || *(p-1) == '$'
                      || *(p-1) == '`'
                      || *(p-1) == '=' )
                    )
                        a_fail = TRUE;
                    if (!a_fail
                    &&  !a_p->ignore_trailing_letters
                    &&  (isalnum(p[l1])
                      || p[l1] == '$'
                      || (l1 == 1
                        && p[l1] == '.' )
                    ) )
                        a_fail = TRUE;
                    if (a_fail == TRUE) {
                        ++a_p;
                    	continue;
                    }

                    /* Acronym accepted */
                    match = nomore;
                    l2 = strlen(a_p->out);
                    if (l1 > l2) /* more room than needed - close up the gap */
                    	memmove(p+l2, p+l1, strlen(p+l2)+1);
                    else if (l1 < l2)  /* less room than needed - open up the gap */
                    	memmove(p+l2, p+l1, strlen(p)+1);
                    memcpy(p, a_p->out, l2);
                    p += l2;
                    match_found = TRUE;
                    e = 0;
                } /* endwhile - search exhausted/completed */
            } /* endif - a_p pointed to the start of a block of acronyms */
            if (match != nomore) {
                --match;
                if (match == common) {
                    /*test070795if (*p >= ' ')*/
                        a_p = a_initial[tolower(*p)].start;
                }/*test300695*/
                            /* Lowercase area includes case-insens. acronyms */
            }
          } while (match); /* Tried uppercase, then lowercase/common entries */
            if (!match_found)
            	++p;
        } while (*p
          && ( (ispunct(*(p-1)) || isspace(*(p-1)))
        	&& *(p-1) != '$' && *(p-1) != '`' && *(p-1) != '=' ) );
                /* Try each leading char till (incl.) 1st alphanumeric */

        /* Find next "word" (include leading punctuation in case needed;) */
        /* hyphen counts as word boundary */
        p += strcspn(p, " \t\n\r-");
        p += strspn(p, " \n\r"); /*test070795 exclude tab*/
    } /* String exhausted */

    return;
} /* End process_acronyms */


int translatefile(char *infl_name, char *outfl_name, int mode,
                    boolean ScreenOn
#if !defined(BRAILLE_IT)
                    ,boolean SysPause
#endif
)
{
  FILE *fp1 = NULL, *fp2 = NULL;
  int slength;
  char errorloc[] = "ITS translator - ";
  char modulename[] = "translatefile";
  unsigned char *tmpptr, *inptr, *storinstr = NULL;
  unsigned char *outstr = NULL, *transptr;
  flag notfirstblock = false;
    #ifdef CSCAPEVER
	char transstr[TRANBOXWIDTH + 1];
	char tostr[TRANBOXWIDTH + 1];
	char titlestr[TRANBOXWIDTH + 1];
	char transource[FILENAMELENGTH];
	char trandest[FILENAMELENGTH];

	menu_type process_menu;
	sed_type process_sed;
	#endif

/* Initialise for translate */
	nwords = 0;
	abrt = FALSE;
#ifndef KevTest
/* Clear keyboard buffer */
    while (_kbhit())
        _getch();
#endif

  if ((fp1 = ITS_fopen(infl_name, "rb")) == NULL) {
     return (FALSE);
  }
#if defined(BRAILLE_IT)
  if (outfl_name == NULL) { /* the test for acronym preprocessing */
    if ((fp2 = ITS_fopen(acrotextfl_name, "w")) == NULL) {
       return (FALSE);
    }
  }
#endif
  else {
    if ((fp2 = ITS_fopen(outfl_name, "w")) == NULL) {
       return (FALSE);
    }
  }

  /* Allocate storage for the output string */
  if ((outstr = (unsigned char *) malloc(BLK_BUFSIZE*2+4)) == NULL) {
     ITS_Error("Output string allocation error", modulename, -1, -1);
	 return(FALSE);
  }
	#ifdef CSCAPEVER
/* Setup screen */
	if (ScreenOn)
	{	process_menu = menu_Open();
		process_sed = sed_Open(process_menu);

		sed_SetBorder(process_sed, bd_null);
		sed_SetBorderColor(process_sed, TRANBOXPROMPTCLR);

		sed_SetColors(process_sed, TRANBOXREGCLR, TRANBOXBACKCLR, TRANBOXHICLR);
		sed_SetPosition(process_sed, TRANBOXROWPOS, TRANBOXCOLPOS);
		sed_SetShadow(process_sed, 1);
		sed_SetShadowAttr(process_sed, 0x07);
		sed_SetHeight(process_sed, TRANBOXHEIGHT);
		sed_SetWidth(process_sed, TRANBOXWIDTH);

		sprintf(titlestr, "ITS Translate Process  Version %s", tran_ver);
		strcenter(titlestr, TRANBOXWIDTH);
                menu_Printf(process_menu, "@p[1,0]%s", titlestr);

		strcpy(transstr, "Translating the following source file:");
		strcenter(transstr, TRANBOXWIDTH);
                menu_Printf(process_menu, "@p[3,0]%s", transstr);

		strcpy(transource, infl_name);
		strcenter(transource, TRANBOXWIDTH);
                menu_Printf(process_menu, "@p[4,0]%s", transource);

		strcpy(tostr, "To the following destination file:");
		strcenter(tostr, TRANBOXWIDTH);
                menu_Printf(process_menu, "@p[5,0]%s", tostr);

        strcpy(trandest,
#if defined(BRAILLE_IT)
        (outfl_name == NULL) ? acrotextfl_name :
#endif
        outfl_name);/*test220695*/
		strcenter(trandest, TRANBOXWIDTH);
                menu_Printf(process_menu, "@p[6,0]%s", trandest);

                menu_Printf(process_menu, "@p[8,20]Word Count: %6ld", nwords);
		movecursor(TRANBOXROWPOS + 8, TRANBOXCOLPOS + 33);
		sed_Repaint(process_sed);
		sed_BorderPrompt(process_sed, " Press ESCAPE to abort process");
		sed_Repaint(process_sed);
	}
#else
	if(ScreenOn)
		{
		printf("\n\nITS Translate Process   Version %s", tran_ver);
		printf("\nTranslating the following source file: %s",infl_name);
        printf("\nTo the following destination file: %s\n",
#if defined(BRAILLE_IT)
                (outfl_name == NULL) ? acrotextfl_name :
#endif
            outfl_name);/*test220695*/
		}
#endif

/* Now translate block-by-block till eof */
  while ((tmpptr = getblok(inblok, fp1)) != NULL) {

#ifndef KevTest
     /* User abort? */
     if (_kbhit() && _getch() == ESCKEY)
	 {	/* Clear keyboard buffer */
        while (_kbhit())
            _getch();
		raise(SIGINT);
	 }
#endif
	 
/* If abort required then clean up mess ! (files will sort themselves out)*/
	 if (abrt == TRUE &&
				Confirm("Abort Translation !", TRANBOXROWPOS + 2, 25))
     {
			free (storinstr);
			free(outstr);
			ITS_fclose(fp1);
			ITS_fclose(fp2);
			#ifdef CSCAPEVER
			if (ScreenOn)
				sed_Close(process_sed);
			#endif
			return(FALSE);
	 }
	 abrt = FALSE;

    /* Adjust file pointer & input buffer end to coincide with rightmost space */
    if (!feof(fp1))
      findrchr(inblkptr, ' ', fp1, endinblk);
    /* Exclude rightmost word if it is a mid-sequence word */
    if (!feof(fp1))
      check_last_word(inblok, fp1);

    /* Allocate storage for temp. input string */
    if ((storinstr = malloc(strlen(inblkptr)*2)) == NULL) {
		sprintf(errmsg, "Input string allocation error:-\n  %s", inblkptr);
        ITS_Error(errmsg, modulename, -1, -1);
		free(outstr);
		ITS_fclose(fp1);
		ITS_fclose(fp2);
		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(process_sed);
		#endif
		return(FALSE);
    }
    strcpy(storinstr, inblok);

    if (outfl_name != NULL) /* NULL would indicate acronyn preprocessing *//*test200695*/
      inptr = strip_lead_trail(storinstr);
    else/*test200695*/
      inptr = storinstr;/*test200695*/

    /* Translate or preprocess */
	if (strlen(inptr) > 0) {
#if !defined(BRAILLE_IT)
      /* Process acronyms here */
      process_acronyms(inptr);
#else
      if (outfl_name == NULL)
        transptr = outstr; /* Fix to prevent error during acronym preprocessing */
      if (outfl_name == NULL) { /* special case to cause acronym processing */ /*test130695*/
        strcpy(outstr, inptr); /*test130695*/
        process_acronyms(outstr); /*test130695*/
      }
      else { /*test130695*/
#endif

        transptr = ittrans(inptr, outstr, &slength,
           (mode == 1)?(unsigned)GRADE1:0);
#if defined(BRAILLE_IT)
      }
#endif
      if (notfirstblock) {
        flag spaceflg = true;
        if (outstr[0] == '-') {
          if (outstr[1] == '-' || isspace(outstr[1]))
            spaceflg = false;
        }
        else if (isspace(outstr[0]))
          spaceflg = false;
        if (spaceflg == true) {
          memmove(outstr+1, outstr, strlen(outstr)+1); /* Move up by 1 */
          outstr[0] = ' ';
        }
      }
      notfirstblock = true;
      if (transptr == NULL || slength == EOF) {
        ITS_Error("Translation error!", modulename, -1, -1);
        free (storinstr);
        free(outstr);
        ITS_fclose(fp1);
        ITS_fclose(fp2);
        #ifdef CSCAPEVER
        if (ScreenOn)
          sed_Close(process_sed);
        #endif
        return(FALSE);
      }

	  strcpy(outblok, outstr);
	  if (putblok(outblok, fp2) == NULL)
		break;

/* Update word count on screen */
	#ifdef CSCAPEVER
	if (ScreenOn)
		{	menu_Printf(process_menu, "@p[8,33]%6ld", nwords);
			movecursor(TRANBOXROWPOS + 8, TRANBOXCOLPOS + 33);
			sed_Repaint(process_sed);
		}
	#else
	if(ScreenOn)
		{
		printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bWord Count: %6ld", nwords);
		}
	#endif

	 }
	 /* Free up temp. store */
    free (storinstr);
  }

   if (tmpptr != NULL)
			free (storinstr);
	free(outstr);

	if (ferror(fp1)) { /* Error */
        ITS_Error("Error while reading input file!", modulename, -1, -1);
		ITS_fclose(fp1);
		ITS_fclose(fp2);
		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(process_sed);
		#endif
		return(FALSE);
	}
	else
	if (ferror(fp2)) { /* Error */
        ITS_Error("Error while writing to output file!", modulename, -1, -1);
		ITS_fclose(fp1);
		ITS_fclose(fp2);
		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(process_sed);
		#endif
		return(FALSE);
	}

  /* Closedown */
	if (ITS_fclose(fp1) != 0) {
		ITS_fclose(fp2);
		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(process_sed);
		#endif
		return(FALSE);
	}
	if (ITS_fclose(fp2) != 0) {
		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(process_sed);
		#endif
		return(FALSE);
    }

#ifdef BRAILLE_IT
	/* v2.2*/ itclose(); /* Clear the allocations needed in Translate when running
				Braille It - not needed in Format, and Braille It is a one-pass
				program.*/
#endif
/*	Comment out for now --> itclose(); */

	#ifdef CSCAPEVER
/* Provide closing message and wait */
	if (ScreenOn)
	{	if (SysPause)
		{	menu_Printf(process_menu, "@p[8,14]Final");
			menu_Printf(process_menu, "@p[8,41]@fd1[ OK ]", NULL, &nowrite_funcs,
							" Process completed ... Press ESCAPE or ENTER to continue.");
			movecursor(TRANBOXROWPOS + 8, TRANBOXCOLPOS + 33);
			tone();
			sed_Repaint(process_sed);
			sed_Go(process_sed);
		}
		sed_Close(process_sed);
	}
    #endif

#if defined(BRAILLE_IT)
  if (outfl_name != NULL) /* Not preprocessing acronyms */ /*test200695*/
    remove(acrotextfl_name); /* No longer needed */
#endif

  return(TRUE);
}
/* ITTRANS.C */
/***************************************************************************/
/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*2.0*13/10/93*BRAILLE_IT compiler directive now comments out    *?????*KJM*/
/*   *        *code not specifically needed in 'ItsLite'.        *     *   */
/***************************************************************************/
/*1.7*19/04/93*ITSlite preprocessor commands "CSCAPEVER=1"       *30301*MW */
/***************************************************************************/
/*1.6*01/07/92*Adjusted colours for screens.                     *30301*MJH*/
/***************************************************************************/
/*1.5*08/06/92*Set auxiliary to move cursor in initialise screen.*30301*MJH*/
/***************************************************************************/
/*1.4*01/06/92*Moon translation added. ITTRANS.H -> TRANFILE.H   *30301*MJH*/
/***************************************************************************/
/*1.3*29/05/92*Standardised function naming & calling convention.*30301*MJH*/
/***************************************************************************/
/*1.2*20/05/92*Rationalised header files.                        *30301*MJH*/
/***************************************************************************/
/*1.1*15/05/92*Version Notes added.                              *30301*KJM*/
/***************************************************************************/
/*1.0*07/05/92*Creation Date.                                    *30301*NO */
/***************************************************************************/
/*This File is part of the following .OBJ files:                           */
/*                                                                         */
/* ITTRANS.OBJ                                                             */
/*                                                                         */
/*Any change to this file means that the above programs have to be         */
/*recompiled with a new version number.                                    */
/***************************************************************************/

/* ------------------------ INCLUDE FILES --------------------------------- */
/* --------------------- END OF INCLUDE FILES ----------------------------- */


#if !defined(BRAILLE_IT)
/* ------------------------------------------------------------------------ */
boolean moon_translate(char *source, char *dest, char *Table,
			boolean *TransInit, boolean *BrlInit, int grade,
			boolean ScreenOn, boolean SysPause)
{
	#ifdef CSCAPEVER
	sed_type pop_sed;
	#endif
	char TempTableName[FILENAMELENGTH];

/* Valid source name ? */
    if (_access(source, 00) != 0)
	{	sprintf(errmsg, "File %s does not exist", source);
		ITS_Error(errmsg,
					"moon_translate", -1, -1);
		errno = 0;
		return(FALSE);
	}

/* Valid table name ? */
/* Copy to temporary storage which holds full path name */
	strcpy(TempTableName, Table);
	if (LocateFile(TempTableName) != TRUE)
	{	errno = 0;
		return(FALSE);
	}


/* Initialise table ? */
	if (!(*TransInit))
	{
		#ifdef CSCAPEVER
		if (ScreenOn)
		{	pop_sed = pop_Text("\n  Initialising Translation Table ", 11, 23, 3, -1,
									0x3f, bd_null);
			sed_SetShadow(pop_sed, 1);
			sed_SetShadowAttr(pop_sed, 0x07);
			sed_SetBorderColor(pop_sed, 0x3f);
			sed_BorderPrompt(pop_sed, " Please wait ...");
         	movecursor(12,23);
			sed_Repaint(pop_sed);
			sed_Go(pop_sed);
		}
		#endif

/* Need to close braille ? */
		if (*BrlInit)
		{	itclose();
			*BrlInit = FALSE;
		}

        if (itinit(TempTableName, NULL
#if defined(BRAILLE_IT)
          , source/*test130695*/
#endif
        ))
			*TransInit = TRUE;

		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(pop_sed);
		#endif
	}

/* Translate the file */
	if (*TransInit)
        return(translatefile(
#if defined(BRAILLE_IT)
          acrotextfl_name/*test130695*/
#else
          source
#endif
          , dest, grade, ScreenOn, SysPause));

	return(FALSE);
}
/* ------------------------------------------------------------------------ */
#endif


/* ------------------------------------------------------------------------ */
boolean braille_translate(char *source, char *dest, char *TransTable,
            boolean *TransInit,
#if !defined(BRAILLE_IT)
            boolean *MoonInit,
#endif
            int grade,
            boolean ScreenOn
#if !defined(BRAILLE_IT)
            ,boolean SysPause
#endif
)
{
#ifdef CSCAPEVER
	sed_type pop_sed;
#endif
    char TempTransTableName[FILENAMELENGTH];
    char TempAcroTableName[FILENAMELENGTH];

/* Valid source name ? */
    if (_access(source, 00) != 0)
	{	sprintf(errmsg, "File %s does not exist", source);
		ITS_Error(errmsg,
					"braille_translate", -1, -1);
		errno = 0;
		return(FALSE);
	}

/* Valid trans. table name ? */
/* Copy to temporary storage which holds full path name */
    strcpy(TempTransTableName, TransTable);
    if (LocateFile(TempTransTableName) != TRUE)
	{	errno = 0;
		return(FALSE);
	}

/* Valid acronym table name ? */
/* Copy to temporary storage which holds full path name */
    strcpy(TempAcroTableName, BrlParam.AcronymTable);
    if (LocateFile(TempAcroTableName) != TRUE)
	{	errno = 0;
		return(FALSE);
	}

/* Initialise table ? */
	if (!(*TransInit))
	{
		#ifdef CSCAPEVER
		if (ScreenOn)
		{	pop_sed = pop_Text("\n  Initialising Translation Table ", 11, 23, 3, -1,
									0x3f, bd_null);
			sed_SetShadow(pop_sed, 1);
			sed_SetShadowAttr(pop_sed, 0x07);
			sed_SetBorderColor(pop_sed, 0x3f);
			sed_BorderPrompt(pop_sed, " Please wait ...");
			movecursor(12,23);
			sed_Repaint(pop_sed);
			sed_Go(pop_sed);
		}
		#endif

#if !defined(BRAILLE_IT)
/* Need to close Moon ? */
		if (*MoonInit)
		{	itclose();
			*MoonInit = FALSE;
		}
#endif                  
                  
        if (itinit(TempTransTableName, TempAcroTableName
#if defined(BRAILLE_IT)
          , source/*test130695*/
#endif
        ))
			*TransInit = TRUE;

		#ifdef CSCAPEVER
		if (ScreenOn)
			sed_Close(pop_sed);
		#endif
	}

/* Translate the file */
	if (*TransInit)
        return(translatefile(
#if defined(BRAILLE_IT)
          acrotextfl_name/*test130695*/
#else
          source
#endif
          , dest, grade, ScreenOn
#if !defined(BRAILLE_IT)
          ,SysPause
#endif
        ));

	return(FALSE);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */

#if !defined(BRAILLE_IT)
boolean TransParamConfirm(
                          char *source, char *dest, int *TransGrade,
                    struct FileInfo Source, struct FileInfo Dest
)
{
	int ret = FALSE;
	#ifdef CSCAPEVER
	sed_type sed;
	char Error[81];

/* Load screen */
	if ((sed = sfile_LoadSed(lnffile, "trans_cnfrm", SED_ALLOC)) == NULL)
	{	ITS_Error(" Error trying to load confirmation screen ",
				 "TransParamConfirm", -1, -1);
	}
	else
	{
/* Set file information structures (used within auxiliary) */
		SourceFinfo = Source;
		DestFinfo = Dest;
/* Initialise each field on screen */
		strcpy(Error, "\0");
		strcpy((char *) sed_GetNameVar(sed, "Error"), Error);

		strcpy((char *) sed_GetNameVar(sed, "Source"), source);
		strcpy((char *) sed_GetNameVar(sed, "Destination"), dest);
		*((int *) sed_GetNameVar(sed, "Grade")) = *(TransGrade) - 1;

		sed_ProtectField(sed, sed_GetNameNo(sed, "Error"));

/* Set auxiliary and special functions */
		sed_SetAux(sed, aux_sourcedest);
		sed_SetSpecial(sed, spc_params);

		sed_Repaint(sed);
		if (sed_Go(sed) != SED_ABORT)
		{
/* Retrieve values from each field */
			strcpy(source, (char *) sed_GetNameVar(sed, "Source"));
			strcpy(dest, (char *) sed_GetNameVar(sed, "Destination"));
			*(TransGrade) = *((int *) sed_GetNameVar(sed, "Grade")) + 1;
			ret = TRUE;
		}

		sed_Close(sed);
	}
	#endif
	return(ret);

}
#endif

/* ------------------------------------------------------------------------ */



/* --------------------------- MODULE AGJA -------------------------------- */
#ifdef CSCAPEVER
boolean ConfirmTrans(char *tempsource, char *tempdest,
			struct FileInfo *Source, struct FileInfo *Dest,
			int *TransGrade)
{	boolean ParamOK = FALSE;
	struct FilePath temp = { "\0", "\0", "0", "\0" };

	errmsg[0] = '\0';
	tempsource[0] = '\0';
	tempdest[0] = '\0';

/* Split source name into drive, directory, name and extension */
	_splitpath(Source->Name, temp.drv, temp.dir, temp.name, temp.ext);

/* Any source name ? */
	if (temp.name[0] != '\0')
	{
		strcpy(tempsource, Source->Name);
/* Expand source name for default/current/specified directory etc. */
		ProduceFullName(tempsource, *Source);
/* Copy base name from source to destination */
		strcpy(tempdest, temp.name);
/* Expand destination name for default/current directory etc. */
		ProduceFullName(tempdest, *Dest);
	}

	while (!ParamOK)
	{	if (!TransParamConfirm(tempsource, tempdest, TransGrade, *Source, *Dest))
			return(FALSE);
		ParamOK = TRUE;
/* Store source and destination names as parameters */
		StoreFullName(tempsource, Source);
		StoreFullName(tempdest, Dest);
	}

	return(TRUE);
}
#endif
/* ------------------------ END OF MODULE AGJA ---------------------------- */



/* --------------------------- MODULE AEAAA ------------------------------- */
boolean ConfirmVitalTrans(char *tempsource, char *tempdest,
            struct FileInfo *Source, struct FileInfo *Dest
#if !defined(BRAILLE_IT)
            ,int *TransGrade, char *TransTable
#endif
)
{	struct FilePath temp = { "\0", "\0", "0", "\0" };

	errmsg[0] = '\0';
	tempsource[0] = '\0';
	tempdest[0] = '\0';

/* Split source name into drive, directory, name and extension */
	_splitpath(Source->Name, temp.drv, temp.dir, temp.name, temp.ext);

/* Pre-format file names */
	trimstr(temp.name);

/* Any source name ? */
	if (temp.name[0] == '\0')
	{	ITS_Error(" No source file specified for translation ! ",
					"ConfirmVitalTrans", -1, -1);
		return(FALSE);
	}

	strcpy(tempsource, Source->Name);
/* Expand source name for default/current/specified directory etc. */
	ProduceFullName(tempsource, *Source);
/* Copy base name from source to destination ? */
	if (Dest->Name[0] != '\0')
		strcpy(tempdest, Dest->Name);
	else
		strcpy(tempdest, temp.name);
/* Expand destination name for default/current directory etc. */
	ProduceFullName(tempdest, *Dest);

/* Store source and destination names as parameters */
	StoreFullName(tempsource, Source);
	StoreFullName(tempdest, Dest);

	return(TRUE);
}
/* ------------------------ END OF MODULE AEAAA --------------------------- */


/* End of file ITBTRANS.C */
