/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/***************************************************************************
 * ITS - Read translation table into memory & write to binary file.        *
 *     11-Apr-1995                                                         *
/***************************************************************************/


/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.0*06/04/95*Binary table init. module.                        *30418*NO */
/***************************************************************************/
/**************************************************************************
 *                                                                        *
/**************************************************************************/
#include "its.h"
#include "itsfunc.h"
/*test#include "tranfile.h"*/
#include "translat.h"
#pragma warning(once:4057)
#define NOERROR 0
#define ERROR -!NOERROR

extern unsigned xormask;
extern inc;
extern char errmsg[];
extern struct alph  *alphabet; /*v. 2.2*/
extern struct cont  *contraction; /*v. 2.2*/

extern size_t   nat, nct, nnt, nsv, nic, ndtc;
extern size_t inpspaces;
extern unsigned   translat_flags;  /*  Bit 0 = caps enable
                            	bit 1 = alphanumsign enable
                                bit 4 = retain newlines
                        	*/
extern unsigned char   rcclas[MAXRCC][OUT_SIZE], nonter[MAXRCC];
extern unsigned char value[256], bval[BUF_SIZE];
extern unsigned char   __near *stvar;
extern unsigned char  __near *trans[NSV], __near *decis[NIC], __near *cond[NDTC];

extern unsigned char table_versno[];
extern unsigned char capsign, alphanumsign, format_start;
extern size_t buf_size, out_size;

int e_write(void *buf, size_t nbytes, FILE *f)
{
    size_t i;
    char modulename[] = "e_write";
    static err = NOERROR;

    /* Prevent function from working if previous error */
    if (err == ERROR) /*test020595*/
        return ERROR; /*test020595*/

    for(i=0; i<nbytes; ++i)
        if (fputc(((unsigned char *)buf)[i]^xormask, f) == EOF) {
            sprintf(errmsg, "Cannot write to file:\npos. %ld",
              ftell(f));
            ITS_Error(errmsg, modulename, -1, -1);
            err = ERROR;
            return ERROR;
        }

    xormask = _rotl(xormask, inc);
    return NOERROR;
}

int writetobin(char *binfilename)
/* N.B. Any change to the order in which elements of the file are written
   must be strictly reflected in the corresponding read routine.
*/
{
#define TABLE_HEADER "ITS Translation Tables\r\n   Version B"
    FILE *bin_f;
    int w;
    unsigned i, ii, jj, e;
    char crlf[] = "\r\n", comma[] = ",";
    char modulename[] = "writetobin";

    if ((bin_f = fopen(binfilename, "wb")) == NULL) {
        sprintf(errmsg, "Cannot open binary file: %s", binfilename);
        ITS_Error(errmsg, modulename, -1, -1);
        return ERROR;
    }

/* Dump the tables to binary file */
    /* Table header */
    w = fwrite(TABLE_HEADER, 1, strlen(TABLE_HEADER), bin_f);
    w |= fwrite(table_versno, 1, strlen(table_versno), bin_f);
    w |= fwrite(crlf, 1, strlen(crlf), bin_f);
    w |= fprintf(bin_f, "Alphabet: %03d; contraction: %04d; classes: %02d;\r\n"
      "   state table: %02d; condition lines: %02d; right context classes: %1d",
      nat, nct, nic, nsv, ndtc, nnt);
    w |= fwrite(crlf, 1, strlen(crlf), bin_f);
    w |= fputc(EOFCHAR, bin_f);
    if (w < 0 || ferror(bin_f)) { /* Cannot use ERROR - system routines use EOF */
        printf("Error writing header to binary file.\n");
        return ERROR;
    }

    /* Transition table & state variables */
    w = e_write(&nsv, sizeof(nsv), bin_f);
    w |= e_write(&nic, sizeof(nic), bin_f);
    for (i=0; i<nsv; ++i)
        w |= e_write(trans[i], nic, bin_f);
    w |= e_write(stvar, nsv, bin_f);
    if (w == ERROR) {
        printf("Error writing transition table to binary file.\n");
        return ERROR;
    }

    /* Decision/condition table */
    w = e_write(&ndtc, sizeof(ndtc), bin_f);
    /* Decisions point to cond[x], one decision for each input class */
    for (i=0; i<ndtc; ++i) {
        for (ii=0; ii < nic; ++ii) {
            if (decis[ii] == cond[i]) {
                w |= e_write(&ii, sizeof(int), bin_f);
            }
            else if (decis[ii] == NULL) { /* Unconditional acceptance */
                jj = ii | 0x8000; /* Set sign bit to show that
                                     this decision is unconditional */
                decis[ii] = cond[i]; /* Prevent it being written again */
                w |= e_write(&jj, sizeof(int), bin_f);
            }
        }
        w |= e_write(comma, sizeof(char), bin_f);
        w |= e_write(cond[i], nsv, bin_f);
    }
    if (w == ERROR) {
        printf("Error writing decision/condition table to binary file.\n");
        return ERROR;
    }

    /* Miscellaneous settings */
    w = e_write(&buf_size, sizeof(int), bin_f);
    w |= e_write(&out_size, sizeof(int), bin_f);
    w |= e_write(&inpspaces, sizeof(int), bin_f);
    w |= e_write(&capsign, sizeof(char), bin_f);
    w |= e_write(&translat_flags, sizeof(int), bin_f);
    w |= e_write(&alphanumsign, sizeof(char), bin_f);
    w |= e_write(&format_start, sizeof(char), bin_f);
    if (w == ERROR) {
        printf("Error writing miscellaneous settings to binary file.\n");
        return ERROR;
    }

    /* RCC table */
    w = e_write(&nnt, sizeof(int), bin_f);
    for (i=0; i<nnt; ++i) {
        w |= e_write(&nonter[i], sizeof(char), bin_f);
        w |= e_write(&rcclas[i], sizeof(rcclas[i]), bin_f);
    }
    if (w == ERROR) {
        printf("Error writing rcc table to binary file.\n");
        return ERROR;
    }

    /* Alphabet/Contraction table */
    w = e_write(&nat, sizeof(int), bin_f);
    w |= e_write(&nct, sizeof(int), bin_f);
    /* Put minimal validation here (?) ... */
    if (!inpspaces || !buf_size || !out_size || !nat || !nct
    || !nsv || !nic || !ndtc) {
        printf("Text table initialisation error\n");
        return ERROR;
    }

    w |= e_write(value, sizeof(value), bin_f);
    for (i=0; i<nat; ++i)
        w |= e_write(&alphabet[i].extent, sizeof(int), bin_f);

    if (w == ERROR) {
        printf("Error writing alphabet table's extent or value to binary file.\n");
        return ERROR;
    }

    e = alphabet[0].extent - 1;
    for (i=0; i<nat; ++i) {
        w |= e_write(&alphabet[i].symbol, sizeof(char), bin_f);
        w |= e_write(&alphabet[i].cclass, sizeof(char), bin_f);
        if (i + 1 == nat)
            break; /* Remaining elements of this entry are superfluous */
        w |= e_write(&alphabet[i].altsg, sizeof(char), bin_f);
        w |= e_write(&alphabet[i].singsg, sizeof(char), bin_f);
        w |= e_write(&alphabet[i].ascsg, sizeof(char), bin_f);
        /* Contractions for this entry */
        if (alphabet[i+1].extent > 0) {
        	while (e < alphabet[i+1].extent -1) {
            	w |= e_write(&contraction[e].branch, sizeof(int), bin_f);
            	w |= e_write(&contraction[e].iclass, sizeof(char), bin_f);
            	w |= e_write(&contraction[e].shifno, sizeof(char), bin_f);
            	w |= e_write(&contraction[e].rcc, sizeof(char), bin_f);
            	jj = strlen(contraction[e].ctable);
            	w |= e_write(&jj, sizeof(char), bin_f);
            	w |= e_write(contraction[e].ctable, jj, bin_f);
            	jj = strlen(contraction[e].tsign);
            	w |= e_write(&jj, sizeof(char), bin_f);
            	w |= e_write(contraction[e].tsign, jj, bin_f);
            	++e;
            }
        }
        if (w == ERROR) {
            sprintf(errmsg, "Error writing alphabet/contraction table to binary file.\n"
              "Alphabet entry: %d (%c); contraction entry: %c%s\n",
              i+1, alphabet[i].symbol, alphabet[i].symbol, contraction[i].ctable
            );
            ITS_Error(errmsg, modulename, -1, -1);
        	return ERROR;
    	}
    }

    if (fclose(bin_f) != 0) {
        perror("Error while closing binary file");
        return ERROR;
    }
    return NOERROR;
#undef TABLE_HEADER
}
