/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/* Two of the three functions in this file - ITS_Error & LocateFile - */
/*    are taken from ITS (COMMON.C).                                  */
#include "its.h"
/* ------------------------------------------------------------------------ */
/* Provide ITS format error message */
void ITS_Error(char *msg, char *Module, int row, int col)
{
#ifdef CSCAPEVER
	menu_type errormenu;
	sed_type error_screen;
	char TmpMsg[80];
	char ModMsg[80];
	int msglines, msgwidth, tmpwidth, tmplines;
	int maxwidth = 80;
	int maxlines = 25;
	int borderwidth = 2;
	int titlelines = 3;
   char *p, *formstr;
	char Title[] = "**** ITS Error Message ****";

/* Calculate maximum area for message */
	if (row > 0)
	   maxlines = maxlines - (2 * row);
	if (col > 0)
		maxwidth = maxwidth - (2 * col);

/* Find largest width out of title, module name and max message width*/
	sprintf(ModMsg, "Module: %s", Module);
	msgwidth = max(strlen(Title), strlen(ModMsg));
   msgwidth = max(msgwidth, (maxwidth - borderwidth - 2));

/* Format message string according to maximum width possible */
   if ((formstr = strwrap(msg, &msglines, msgwidth)) == NULL)
    	return;

/* Improve format by using greatest width - each line is then centred. */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
	{	if (*p == '\n')
		{	if (tmpwidth > msgwidth)
				msgwidth = tmpwidth;
			tmpwidth = 0;
			++tmplines;
		}
		else
			++tmpwidth;
	}
	if ((tmpwidth > 0) && (tmpwidth > msgwidth))
		msgwidth = tmpwidth;

	if ((errormenu = menu_Open()) == NULL)
      return;
/* Format the title */
	strcpy(TmpMsg, Title);
	strcenter(TmpMsg, msgwidth);
	menu_Printf(errormenu, "@p[1]%s@f[#]\n", TmpMsg, NULL, &nowrite_funcs);

/* Format the module name */
	strcenter(ModMsg, msgwidth);
	menu_Printf(errormenu, "@p[1]%s\n\n", ModMsg);

/* Copy and format each line in message */
	for (p = formstr, tmplines = 0, tmpwidth = 0;
    					*p && (tmplines < msglines); p++)
  {   if (*p == '\n')
		{	TmpMsg[tmpwidth] = '\0';
			strcenter(TmpMsg, msgwidth);
			menu_Printf(errormenu, "@p[1]%s\n", TmpMsg);
			tmpwidth = 0;
         ++tmplines;
		}
		else
		{	if (*p == '\t')
				TmpMsg[tmpwidth] = ' ';
			else
				TmpMsg[tmpwidth] = *p;
			++tmpwidth;
		}
	}
	if (tmpwidth > 0)
	{	TmpMsg[tmpwidth] = '\0';
		strcenter(TmpMsg, msgwidth);
		menu_Printf(errormenu, "@p[1]%s\n", TmpMsg);
		tmpwidth = 0;
	}

	if ((error_screen = sed_Open(errormenu)) == NULL)
	{	menu_Flush(errormenu);
		return;
	}

/* Calculate 'centred' position (include title lines and border width) */
	if (row < 0)
		row = (int) (maxlines - (msglines + titlelines)) / 2;
   if (col < 0)
		col = (int) (maxwidth - (msgwidth + borderwidth + 1)) / 2;

   sed_SetLabel(error_screen, 6);
	sed_SetBorder(error_screen, bd_prompt);
	sed_SetPosition(error_screen, row, col);
	sed_SetHeight(error_screen, msglines + titlelines);
	sed_SetWidth(error_screen, msgwidth + borderwidth);

	sed_SetBorderColor(error_screen, ERRORBACK_CLR);
	sed_SetColors(error_screen, ERRORREG_CLR, ERRORBACK_CLR, ERRORHI_CLR);
	sed_SetShadow(error_screen, 1);
	sed_SetShadowAttr(error_screen, 0x08);

	sed_SetSpecial(error_screen, spc_Abort);
	sed_SetCursorType(error_screen, CURSOR_NONE);

	tone();
	sed_Repaint(error_screen);
	movecursor(row + 1, col + 1);
	sed_Go(error_screen);
	sed_Close(error_screen);
#else
    printf("\n **** ITS Error Message ****");
    printf("\n in function %s",Module);
    printf("\n error : \n");
    perror(msg);/*test160595*/
#endif
	return;
}
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------ */
boolean LocateFile(char *Name)
{   char tmp[FILENAMELENGTH];
    char modulename[] = "LocateFile";
    static struct FilePath prog;
    static struct FilePath current;
    struct FilePath temp = { {""}, {""}, {""}, {""} };
    char currentdrvdir[FILENAMELENGTH];

    _splitpath(Name, temp.drv, temp.dir, temp.name, temp.ext);

/* If prog is not initialised, location of this program has not been determined */
    if (prog.drv[0] == '\0' || prog.name[0] == '\0' || prog.ext[0] == '\0') {
        /* Is this call the call to initialise prog ?
           If so, all elements of Name will be present. */
        if (temp.drv[0] == '\0' || temp.dir[0] == '\0' || temp.name[0] == '\0'
        ||  ( _stricmp(temp.ext, ".exe") != 0
          &&  _stricmp(temp.ext, ".com") != 0 )
        ) {
            strcpy(errmsg, "Location of this program not known");
            ITS_Error(errmsg, modulename, -1, -1);
            return FALSE;
        }
        prog = temp;
        return TRUE;
    }

/* Determine current drive and directory, if not already done */
    if (current.drv[0] == '\0') {
        _getdcwd(_getdrive(), currentdrvdir, FILENAMELENGTH);
        if ((currentdrvdir[0] != '\0') && (currentdrvdir[1] == ':'))
        {       strncpy(current.drv, currentdrvdir, 2);
            strcpy(current.dir, &(currentdrvdir[2]));
        }
        else
            strcpy(current.dir, currentdrvdir);
    }

/* Determine if supplied name has drive &/or directory */
    if ((temp.drv[0] != '\0') || (temp.dir[0] != '\0'))
    {
/* Is file in specified drive &/or directory ? */
        if (_access(Name, 0) != 0)
        {   errno = 0;
            sprintf(errmsg," Cannot find file %s", Name);
            ITS_Error(errmsg, modulename, -1, -1);
            return(FALSE);
        }
        return(TRUE);
    }

/* Is file in current directory ? */
    strcpy(tmp, current.drv);
    strcat(tmp, current.dir);
    strcat(tmp, "\\");
    strcat(tmp, Name);

    if (_access(tmp, (int) 0) != 0)
    {   errno = 0;
/* Is file in program directory ? */
        strcpy(tmp, prog.drv);
        strcat(tmp, prog.dir);
        strcat(tmp, Name);

        if (_access(tmp, 0) != 0)
        {   errno = 0;
            sprintf(errmsg," Cannot find file %s", Name);
            ITS_Error(errmsg, modulename, -1, -1);
            return(FALSE);
        }
    }

    strcpy(Name, tmp);
    return(TRUE);
}
/* ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------ */
boolean yorn(const char *question)
{
    int yornchar;

    do  {
        printf("%s (y\\n) ? ", question);
        yornchar = _getche();
        yornchar = toupper(yornchar);
    } while(yornchar != 'Y' &&
            yornchar != 'N');
    if (yornchar == 'Y')
        return TRUE;
	else
        return FALSE;
}
/* ------------------------------------------------------------------------ */
