/* This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version. */

/* This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details. */

/* You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA 02111-1307, USA. */

/***************************************************************************
 * ITS - Read text translation table into memory.                          *
 *     11-Apr-1995                                                         *
/***************************************************************************/


/*Ver*  Date  *                Version notes                     *Projt*INS*/
/***************************************************************************/
/* . *  /  /  *                                                  *     *   */
/***************************************************************************/
/*1.0*06/04/95*Text table init. module.                          *30418*NO */
/* . *  /  /  *decis[] now a 1-dim. array of pointers to cond.   *     *   */
/***************************************************************************/
/**************************************************************************
 *                                                                        *
 **************************************************************************/
#include "its.h"
#include "itsfunc.h"
#include "tranfile.h"
#include "translat.h"
#pragma warning(once:4057)
#define NOERROR 0
#define ERROR -!NOERROR

char errmsg[ERRMSGSIZE];


struct alph  *alphabet; /*v. 2.2*/
struct cont  *contraction; /*v. 2.2*/

size_t   nat, nct, nnt, nsv, nic, ndtc;
size_t inpspaces;
unsigned   translat_flags;  /*	Bit 0 = caps enable
                            	bit 1 = alphanumsign enable
                                bit 2 = doulbe capsign enable*test010895*
                                bit 4 = retain newlines
                        	*/
unsigned char   rcclas[MAXRCC][OUT_SIZE], 
               nonter[MAXRCC];
unsigned char value[256], bval[BUF_SIZE];
unsigned char   *stvar;
unsigned char  *trans[NSV], *decis[NIC], *cond[NDTC];

unsigned char table_versno[7];
unsigned char capsign, alphanumsign, format_start;
size_t buf_size, out_size;

/* blockio */
unsigned char inblok[BLK_BUFSIZE+2], outblok[BLK_BUFSIZE*2+2],
    *inblkptr, *endinblk,
    *outblkptr, *endoutblk;
int inbuf_full;
size_t maxext;

static void __near makebranch(void)
{
  size_t i, j, n;
  int larray[ALPHA];
  int lowno, highno, vhigh, ii;
  unsigned char izzz;

      for (i=0; i < maxext; ++i) {
	      n = 1;
          lowno = alphabet[i].extent;
	      j = i + 1;
          highno = alphabet[j].extent - 1;
	      vhigh = highno;
	   h: ii = lowno + 1;
	      if (ii > highno) goto d;
          izzz = contraction[lowno-1].ctable[n-1];
	      if (izzz == '\0') goto e;
	   f: if (ii > highno) goto d;
          if (contraction[lowno-1].ctable[n-1]
         != contraction[ii-1].ctable[n-1]) goto e;
	      ++ii;
	      goto f;
	   e: larray[n-1] = ii;
          contraction[lowno-1].branch = ii;
          contraction[ii-1].branch = n;
	      ++n;
	      ++lowno;
	      highno = ii - 1;
	   k: if (lowno <= highno) goto h;
	      --n;
       i: if (n == 0)
          continue;
	      if (n <= 1) goto j;
	      highno = larray[n-2] - 1;
	      goto k;
	   j: highno = vhigh;
	      goto k;
           d: if (ii != lowno + 1) goto l;
          contraction[lowno-1].branch = -contraction[ii-1].branch;
	      ++lowno;
          contraction[ii-1].branch = n;
	      goto i;
	   l: larray[n-1] = ii;
	      ++n;
          contraction[lowno-1].branch = -(int)n;
	      ++lowno;
	      goto h;
    }
}



#define RCCFLD 12

int txinit (char *table_txt)
{
  size_t n, nn, nnn, i, j, l, t;
  unsigned char let1, templ;
  unsigned char tempstr[BUF_SIZE]; /*BUF_SIZE = max. input string in table (incl. null) */
  unsigned char capsstr[18] = {'\0'}, alphanumstr[18] = {'\0'},
	formatstr[18] = {'\0'};
  unsigned char tempsign[OUT_SIZE+1]; /*Temp storage of output signs */
  unsigned char tablhdr[81] = { '\0' }, *hp;
  char allocerr[80] = "** Memory allocation error:\n"
		      "last successful allocation at address";
  char modulename[] = "itinit";
  FILE  *table_file;

  if ((table_file = fopen (table_txt, "r")) == NULL) {
	  sprintf(errmsg, "Unable to open tables file: \n%s", table_txt);
      ITS_Error(errmsg, modulename, -1, -1);
     return (false);
  }

  templ = 0;
  translat_flags = 0;

/* READ ALPHABET TABLE */
/* First read header comment & table version number */
  strcpy(tablhdr, getstrbuf(inblok, table_file));

  /* Allocate alphabet table */
/* v. 2.2*/  if ((alphabet = (struct alph *) calloc(ALPHA,
    sizeof(struct alph))) == NULL) {
        ITS_Error("Error allocating memory for the Alphabet table.", modulename, -1, -1);
      return(false);
  }

  if ( sscanf (tablhdr, "%c %x %x %x %x", &alphabet[0].symbol,
         &alphabet[0].cclass, &alphabet[0].altsg,
         &alphabet[0].singsg, &alphabet[0].ascsg) > 1 ) /* Normal record */
  {
     ITS_Error("Table header missing", modulename, -1, -1);
    return(false);
  }
  else { /* Read table version no. */
    if ((hp = strstr(tablhdr, "v.")) == NULL
    && (hp = strstr(tablhdr, "V.")) == NULL) { /* No version number */
      strcpy(table_versno, "xx.yy");
    }
    else
      sscanf(hp + 2, "%s", table_versno);
  }

  n = 0;
  do {
     getstrbuf(inblok, table_file);
     sscanf (inblkptr, "%c %x %x %x %x", &alphabet[n].symbol,
         &alphabet[n].cclass, &alphabet[n].altsg,
         &alphabet[n].singsg, &alphabet[n].ascsg);
       t = alphabet[n].symbol;
     value[t] = (unsigned char) (n + 1);
     ++n;
  } while ( alphabet[n-1].cclass != 0xFF );

  /*v. 2.2alphabet[n] = NULL; */
  nat = n; /*v. 2.2*/
  if (n > ALPHA) { /*v. 2.2*/
     sprintf(errmsg, "Too many alphabet table entries: %d\n"
         "   Max: %d\n", n, ALPHA);
     ITS_Error(errmsg, modulename, -1, -1);
    return (false);
  }
  if ((alphabet = _expand(alphabet, sizeof(struct alph) * nat)) == NULL) {
    sprintf(errmsg, "Cannot reallocate alphabet table");
    return(false);
  }/*v. 2.2*/

  /* Read maxin-, maxout- buffer sizes */
  getstrbuf(inblok, table_file);
  sscanf (inblkptr, "%*17c%x", &buf_size);
  getstrbuf(inblok, table_file);
  sscanf (inblkptr, "%*17c%x", &out_size);
  if (buf_size > BUF_SIZE || out_size > OUT_SIZE) {
	 ITS_Error("Contraction table error: input or output buffer size too large!",
                     modulename, -1, -1);
    return(false);
  }

  /* Read max allowed spaces in input */
  getstrbuf(inblok, table_file);
  if (sscanf(inblkptr, "%*17c%x", &inpspaces) == 0
      || inpspaces >= buf_size
      || inpspaces == 0)
  {
	 sprintf(errmsg, "Error while reading max. input spaces field: %d\n", inpspaces);
     ITS_Error(errmsg, modulename, -1, -1);
    return(false);
  }

  /* Read CAPSENABLED, capsign, ALPHANUMSIGNENABLED, alphanumsign */
  getstrbuf(inblok, table_file);
  if (sscanf(inblkptr, "%s %2x %2x", capsstr, &n, &nn) == 3)
  {
    if (n)
      translat_flags |= CAPSENABLED | n/*test010895*/;
    capsign = (unsigned char) nn;
  }
  getstrbuf(inblok, table_file);
  if (sscanf(inblkptr, "%s %2x %2x", alphanumstr, &n, &nn) == 3)
  {
    if (n)
      translat_flags |= ALPHANUMSIGNENABLED | n/*test010895*/;
    alphanumsign = (unsigned char) nn;
  }
  if (_strnicmp(capsstr, "caps", 4) != 0) {
	 sprintf(errmsg, "Invalid CAPSENABLE switch or capsign: %02d, %c\n", n, nn);
     ITS_Error(errmsg, modulename, -1, -1);
    return(false);
  }
  if (_strnicmp(alphanumstr, "alphanum", 8) != 0) {
	 sprintf(errmsg, "Invalid ALPHANUMSIGNENABLE switch or alphanumsign: %02d, %c\n",
	n, nn);
     ITS_Error(errmsg, modulename, -1, -1);
    return(false);
  }

  /* Read format_start definition */
  getstrbuf(inblok, table_file);
  if (sscanf(inblkptr, "%s%x", formatstr, &i) != 2
  || _strnicmp(formatstr, "format", 6) != 0) {
	sprintf(errmsg, "Invalid FORMAT_START entry: %s, %02d\n"
	   "Default of \'%c\' being used",
	   formatstr, format_start, FORMAT_START);
    ITS_Error(errmsg, modulename, -1, -1);
    format_start = FORMAT_START;
  }
  else
	format_start = (unsigned char) i;

  /* FILL CONTRACTION TABLE */
  i = 0;
  
/* V2.2*/    if ((contraction = (struct cont *) calloc(NENTRY,
    sizeof(struct cont))) == NULL) {
        ITS_Error("Error allocating memory for the Contraction table.", modulename, -1, -1);
      return(false);
    }

  for ( n = 0; n < NENTRY; ++n ) {

    getstrbuf(inblok, table_file);
    sscanf (inblkptr, "%c", &let1);
    for ( nn = 0; nn < buf_size-1; ++nn ) {
    sscanf (&inblkptr[nn+1], "%c", &tempstr[nn]);
	if (tempstr[nn] == '|')
	  break;
    }
    tempstr[nn] = '\0';
    l = strlen(tempstr);
    strcpy(contraction[n].ctable, tempstr);
    nn = RCCFLD;
    sscanf(&inblkptr[nn], "%c", &contraction[n].rcc);
    sscanf (&inblkptr[nn+1], "%x %x", &contraction[n].iclass, &contraction[n].shifno);
    if (contraction[n].iclass == 0xFF) {
      for (j=i; alphabet[j].cclass != 0xFF; ++j)
    alphabet[j].extent = 0;
      break;
    }

    if (l > buf_size-1) { /*test181295:*/
        sprintf(errmsg, "Input string for entry %u too long: %u", i+1, l);
        ITS_Error(errmsg, modulename, -1, -1);
        return false;
    }/*test151295*/

    for ( nn = 0, nnn = DIGSTRFLD;
	nn < out_size
	&& inblkptr[DIGSTRFLD-1] != '\0'
    && sscanf (&inblkptr[nnn], "%2x", &tempsign[nn]) == 1
	&& tempsign[nn] != '\0';
	++nn, nnn +=2)
      ;
    tempsign[nn] = '\0';
    l = strlen(tempsign);
    if (l > out_size) { /*test151295:*/
        sprintf(errmsg, "Output string for entry %u too long: %u", i+1, l);
        ITS_Error(errmsg, modulename, -1, -1);
        return false;
    }/*test151295*/
    strcpy(contraction[n].tsign, tempsign);

    contraction[n].branch = 0;
    if (let1 != templ) {
      templ = let1;
      alphabet[i].extent = n+1;
      if (alphabet[i].symbol != templ && contraction[n].iclass != 0xFF) {
	sprintf(errmsg, "Table sequence error ..\nAlphabet symbol: '%c'.\n"
	    "Contraction table entry %d:-\n   %s **\n",
          alphabet[i].symbol, n+1, inblkptr);
    ITS_Error(errmsg, modulename, -1, -1);
	return(false);
      }
      ++i;
    }
  }

  free (contraction[n].ctable);
  nct = n;

  if (n > NENTRY) { /*v. 2.2*/
	 sprintf(errmsg, "Too many translation table entries: %d\n"
		 "   Max: %d\n", n, NENTRY);
     ITS_Error(errmsg, modulename, -1, -1);
    return (false);
  }
  if ((contraction = _expand(contraction, sizeof(struct cont) * nct)) == NULL) {
    sprintf(errmsg, "Cannot reallocate contraction table");
    return(false);
  }/*v. 2.2*/

      j = i - 1;
      alphabet[i].extent = alphabet[j].extent + 1;
	  maxext = j;

/* CONTRACTION TABLE SEARCH SET-UP ALGORITHM */
      makebranch();

/* READ RIGHT CONTEXT TABLE */

     getstrbuf(inblok, table_file);
     if (sscanf (inblkptr, "%x", &nnt) == 0 || nnt > MAXRCC) {
		 sprintf(errmsg, "Error while reading right context table ..\n"
		"   MAXRCC = %d; rcc = %d\n", MAXRCC, nnt);
         ITS_Error(errmsg, modulename, -1, -1);
       return(false);
     }
     for ( n = 0; n < nnt; ++n ) {
       getstrbuf(inblok, table_file);
       sscanf (inblkptr, "%*12c %c%2x%2x%2x%2x", &nonter[n],
	   &rcclas[n][0], &rcclas[n][1], &rcclas[n][2], &rcclas[n][3]);
     }

/* READ STATE TABLES */
     getstrbuf(inblok, table_file);
     sscanf (inblkptr, "%*17c%x", &nsv);
     getstrbuf(inblok, table_file);
     sscanf(inblkptr, "%*17c%x", &nic);
     if (nsv > NSV || nsv == 0 || nic > NIC || nic == 0) {
		 sprintf(errmsg, "Error while reading transition table ..\n"
		"   MAXNSV = %d; MAXNIC = %d; nsv = %d; nic = %d", NSV, NIC, nsv, nic);
         ITS_Error(errmsg, modulename, -1, -1);
       return(false);
     }

     /* Read transition table */
     for ( n = 0; n < nsv; ++n ) {
     if ((trans[n] = _ncalloc(nic + 1, 1)) == NULL) {
		sprintf(errmsg, "Memory allocation error: trans%d\n", n);
        ITS_Error(errmsg, modulename, -1, -1);
	   return(false);
	 }
	 getstrbuf(inblok, table_file);
	 for ( nn = 0; nn < nic; ++nn ) {
         if (sscanf (inblkptr+nn, "%Nc", &trans[n][nn]) == 0) {
			 sprintf(errmsg, "Error while reading transition table ..\n"
		   "Entry %d, %d; nsv: %d; nic: %d\n",
		   n, nn, nsv, nic
			 );
             ITS_Error(errmsg, modulename, -1, -1);
	       return(false);
	     }
	 }
	 /* Validate scanned transition table entry */
	 if (strspn(trans[n], "RST-") != nic) {
		sprintf(errmsg, "Invalid character(s) in transition table:\n   %s\n",
	       trans[n]);
         ITS_Error(errmsg, modulename, -1, -1);
		return(false);
	 }
     }

     getstrbuf(inblok, table_file);
     sscanf (inblkptr, "%*17c%x", &ndtc);
     if (ndtc > NDTC || ndtc == 0) {
		 sprintf(errmsg, "Error while reading decision / condition table ..\n"
	   "   NDTC = %d, ndtc = %d\n", NDTC, ndtc);
         ITS_Error(errmsg, modulename, -1, -1);
		 return(false);
     }

     for ( n = 0; n < ndtc; ++n ) {
	 /* Read decision table field */
	 if (
        (cond[n] = (unsigned char __near *) _ncalloc(nsv + 1, 1)) == NULL
	 ) {
		sprintf(errmsg, "Memory allocation error - decis / cond%d\n", n);
        ITS_Error(errmsg, modulename, -1, -1);
		return(false);
	 }
	 getstrbuf(inblok, table_file);
	 for ( nn = 0; nn < nic; ++nn ) {
         if (inblkptr[nn] == 'Y') {
            decis[nn] = cond[n];
         }
        else if (inblkptr[nn] == 'G')
            decis[nn] = NULL;
	 }
     /* Validate scanned decision table entry */
     if (strspn(inblkptr, "GYF-") != nic) {
        sprintf(errmsg, "Invalid decision table entry:\n   %s\n",
           inblkptr);
         ITS_Error(errmsg, modulename, -1, -1);
		return(false);
	 }
     /* Remove any white space before next field */
     while (isspace(inblkptr[nn]))
       ++nn;
	 /* Now read condition table field */
	 for ( nnn = nn, nn = 0; nn < nsv; ++nn, ++nnn ) {
         sscanf (inblkptr+nnn, "%Nc", &cond[n][nn]);
	 }
	 /* Validate scanned condition table entry */
	 if (strspn(cond[n], "NY-") != nsv) {
		sprintf(errmsg, "Invalid condition table entry:\n   %s\n",
	       cond[n]);
         ITS_Error(errmsg, modulename, -1, -1);
		return(false);
	 }
     }
/* Initialise state variables */
     if ((stvar = (unsigned char __near *) _ncalloc(nsv, 1)) == NULL) {
		 sprintf(errmsg, "Memory allocation error - stvar\n");
         ITS_Error(errmsg, modulename, -1, -1);
		 return(false);
     }
     for ( n = 0; n < nsv; ++n ) stvar[n] = 'N';

/* close the table file as it's no longer needed */
     fclose (table_file);
     inblkptr = NULL;

  return(true);
  #undef RCCFLD
}

/* Block i/o functions.
   getstrbuf & putstrbuf point to a string representing an ASCII file line
       terminated by cr/lf.

   9/11/90		N. Octon
*/


unsigned char *__near getstrbuf(unsigned char *buf, FILE *rfileptr) {
  unsigned char *tmpptr, *tmpptr2;
  static eofind = 0;

  if (eofind) {
    eofind = 0; /* Reset for next call */
    inblkptr = NULL;
    return NULL;
  }

  if (endinblk == NULL) { /*initialise */
    endinblk = buf + BLK_BUFSIZE - 1;
  }

  tmpptr = NULL;
  if (inblkptr != NULL)
    tmpptr = strchr(inblkptr, '\0');
  if (tmpptr != NULL) ++tmpptr;
  if (tmpptr == NULL
	|| strchr(tmpptr, '\n') == NULL
	|| tmpptr > endinblk) {
    if (inblkptr != NULL && inblkptr <= endinblk)
      inblkptr = strchr(inblkptr, '\0') + 1;
    if ((tmpptr = getblok(inblok, rfileptr)) == NULL) {
      if (inblkptr != NULL) { /* No more to input,
				but residue must be cleared */
	eofind = 1;
	return inblkptr; /* getblok sees to the buffer residue */
      }
      else
	return NULL;
    }
  }
  if ((tmpptr2 = strchr(tmpptr, '\n')) <= endinblk && tmpptr2 != NULL) {
    *tmpptr2 = '\0';
    inbuf_full = 0;
  }
  else /* Newline not found within scope of buffer */
    inbuf_full = !0;

  inblkptr = tmpptr;
  return inblkptr;
}

unsigned char *getblok(unsigned char *buf, FILE *rfileptr) {
  int l, r;

  if (inblkptr == NULL || inblkptr > endinblk)
    inblkptr = buf;
  /* Copy residue of buffer to buffer start & point after it */
  if (inblkptr > buf) {
    strcpy(buf, inblkptr);
    l = strlen(buf);
    inblkptr = buf + l;
  }
  else
    l = 0;


  /*Now read from file */
  if (!feof(rfileptr) && (r = fread(inblkptr, 1, BLK_BUFSIZE - l, rfileptr)) > 0
  && !ferror(rfileptr)) {
    inblkptr = buf;
    endinblk = buf + r + l - 1;
    /* Reinit. buffer from end of input string to buffer end */
    blk_clrbuf(buf+r+l, BLK_BUFSIZE - r - l);
    return buf;
  }
  else if (feof(rfileptr)) {
    if (l == 0)
      inblkptr = NULL;
    else
      inblkptr = buf;
    return NULL;
  }
  else
    return inblkptr = NULL; /* Error while reading */
}

void __near blk_clrbuf(unsigned char *buf, int length) {
  int i;

  for (i=0; i<length; ++i) /*Element buf[length] is always NULL */
    buf[i] = '\0';
}
